from __future__ import division
from numpy import sin, cos, pi, tan
import numpy as np


def advs(y1, y2, y3, a, b, nu, B1, B2, B3):

    A = {} 

    if (np.abs(b) > 0.000001) and (np.abs(b - pi) > 0.000001):
        # tolerance hard-coded to match Meade

        A['11'] = e11(y1, y2, y3, a, b, nu, B1, B2, B3)
        A['22'] = e22(y1, y2, y3, a, b, nu, B1, B2, B3)
        A['33'] = e33(y1, y2, y3, a, b, nu, B1, B2, B3)
        A['12'] = e12(y1, y2, y3, a, b, nu, B1, B2, B3)
        A['13'] = e13(y1, y2, y3, a, b, nu, B1, B2, B3)
        A['23'] = e23(y1, y2, y3, a, b, nu, B1, B2, B3)
    else:

        A['11'] = 0.
        A['22'] = 0.
        A['33'] = 0.
        A['12'] = 0.
        A['13'] = 0.
        A['23'] = 0.

    return A


def e11(y1, y2, y3, a, b, nu, B1, B2, B3):
    e11 = (B1 * (1 / 8 * ((2-2 * nu) * (2 * y2 / y1**2 / (1+y2**2 / y1**2)-y2 /
                        (y1 * cos(b)-y3 * sin(b))**2 * cos(b) /
                        (1+y2**2 / (y1 * cos(b)-y3 * sin(b))**2)+(y2 /
                            (y1**2+y2**2+y3**2)**(1 / 2) * sin(b) / (y1 *
                                (y1 * cos(b)-y3 * sin(b))+y2**2 *
                                cos(b)) * y1-y2 * (y1**2+y2**2+y3**2)**(1 /
                                2) * sin(b) / (y1 * (y1 * cos(b)-y3 *
                                        sin(b))+y2**2 * cos(b))**2 * (2 *
                                    y1 * cos(b)-y3 * sin(b))) / (1+y2**2
                                * (y1**2+y2**2+y3**2) * sin(b)**2 / (y1 *
                                    (y1 * cos(b)-y3 * sin(b))+y2**2 *
                                    cos(b))**2)-y2 / (y1 * cos(b)+(y3+2 *
                                    a) * sin(b))**2 * cos(b) / (1+y2**2 /
                                    (y1 * cos(b)+(y3+2 * a) *
                                     sin(b))**2)+(y2 / (y1**2+y2**2+(y3+2 *
                                         a)**2)**(1 / 2) * sin(b) / (y1 *
                                         (y1 * cos(b)+(y3+2 * a) *
                                          sin(b))+y2**2 * cos(b)) * y1-y2
                                 * (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                                 sin(b) / (y1 * (y1 * cos(b)+(y3+2 * a) *
                                         sin(b))+y2**2 * cos(b))**2 * (2
                                     * y1 * cos(b)+(y3+2 * a) * sin(b)))
    / (1+y2**2 * (y1**2+y2**2+(y3+2 * a)**2) * sin(b)**2 / (y1 * (y1 *
                    cos(b)+(y3+2 * a) * sin(b))+y2**2 *
                cos(b))**2))-y2 * (1 / (y1**2+y2**2+y3**2)**(1 / 2) /
        ((y1**2+y2**2+y3**2)**(1 / 2)-y3)+1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 /
        2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a))-y1 * y2 * (-1 /
        (y1**2+y2**2+y3**2)**(3 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y3) * y1-1
        / (y1**2+y2**2+y3**2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y3)**2 * y1-1 /
        (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                / 2)+y3+2 * a) * y1-1 / (y1**2+y2**2+(y3+2 * a)**2) /
        ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a)**2 * y1)-y2 * cos(b)
    * ((1 / (y1**2+y2**2+y3**2)**(1 / 2) * sin(b) * y1-1) /
            (y1**2+y2**2+y3**2)**(1 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y1 *
                sin(b)-y3 * cos(b))-((y1**2+y2**2+y3**2)**(1 / 2) *
                sin(b)-y1) / (y1**2+y2**2+y3**2)**(3 / 2) /
            ((y1**2+y2**2+y3**2)**(1 / 2)-y1 * sin(b)-y3 * cos(b)) *
            y1-((y1**2+y2**2+y3**2)**(1 / 2) * sin(b)-y1) /
            (y1**2+y2**2+y3**2)**(1 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y1 *
                sin(b)-y3 * cos(b))**2 * (1 / (y1**2+y2**2+y3**2)**(1 /
                    2) * y1-sin(b))+(1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 /
                        2) * sin(b) * y1-1) / (y1**2+y2**2+(y3+2 *
                            a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                                / 2)-y1 * sin(b)+(y3+2 * a) *
                            cos(b))-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                            sin(b)-y1) / (y1**2+y2**2+(y3+2 * a)**2)**(3 /
                            2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
                                sin(b)+(y3+2 * a) * cos(b)) *
            y1-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) /
            (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 *
                        a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
                cos(b))**2 * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                y1-sin(b)))) / pi / (1-nu)+1 / 4 * ((-2+2 * nu) * (1-2 *
            nu) * (y2 / y1**2 / (1+y2**2 / y1**2)-y2 / (y1 * cos(b)+(y3+2 *
                        a) * sin(b))**2 * cos(b) / (1+y2**2 / (y1 *
                            cos(b)+(y3+2 * a) * sin(b))**2)+(y2 /
                        (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b) / (y1
                            * (y1 * cos(b)+(y3+2 * a) * sin(b))+y2**2 *
                            cos(b)) * y1-y2 * (y1**2+y2**2+(y3+2 *
                                a)**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                    cos(b)+(y3+2 * a) * sin(b))+y2**2 *
                                cos(b))**2 * (2 * y1 * cos(b)+(y3+2 * a)
                                * sin(b))) / (1+y2**2 * (y1**2+y2**2+(y3+2 *
                                    a)**2) * sin(b)**2 / (y1 * (y1 *
                                        cos(b)+(y3+2 * a) * sin(b))+y2**2
                                    * cos(b))**2)) * 1/tan(b)**2-(1-2 * nu) *
        y2 / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a)**2 * ((1-2 * nu-a
                / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) * 1/tan(b)-y1 /
            ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a) * (nu+a /
            (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))) / (y1**2+y2**2+(y3+2 *
            a)**2)**(1 / 2) * y1+(1-2 * nu) * y2 / ((y1**2+y2**2+(y3+2 *
                    a)**2)**(1 / 2)+y3+2 * a) * (a / (y1**2+y2**2+(y3+2 *
                        a)**2)**(3 / 2) * y1 * 1/tan(b)-1 / ((y1**2+y2**2+(y3+2 *
                                a)**2)**(1 / 2)+y3+2 * a) * (nu+a /
                            (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+y1**2 /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a)**2 * (nu+a /
                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) / (y1**2+y2**2+(y3+2 *
                    a)**2)**(1 / 2)+y1**2 / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                        2)+y3+2 * a) * a / (y1**2+y2**2+(y3+2 * a)**2)**(3 /
                        2))-(1-2 * nu) * y2 * cos(b) * 1/tan(b) /
    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
     cos(b))**2 * (cos(b)+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) * (1
     / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * y1-sin(b))-(1-2 * nu) * y2 *
    cos(b) * 1/tan(b) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
            sin(b)+(y3+2 * a) * cos(b)) * a / (y1**2+y2**2+(y3+2 *
                a)**2)**(3 / 2) * y1-3 * a * y2 * (y3+a) * 1/tan(b) /
    (y1**2+y2**2+(y3+2 * a)**2)**(5 / 2) * y1-y2 * (y3+a) / (y1**2+y2**2+(y3+2
                * a)**2)**(3 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2
                * a) * ((-1+2 * nu) * 1/tan(b)+y1 / ((y1**2+y2**2+(y3+2 *
                                a)**2)**(1 / 2)+y3+2 * a) * (2 * nu+a /
                            (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+a * y1 /
                    (y1**2+y2**2+(y3+2 * a)**2)) * y1-y2 * (y3+a) /
    (y1**2+y2**2+(y3+2 * a)**2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 *
            a)**2 * ((-1+2 * nu) * 1/tan(b)+y1 / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                        / 2)+y3+2 * a) * (2 * nu+a / (y1**2+y2**2+(y3+2 *
                                a)**2)**(1 / 2))+a * y1 / (y1**2+y2**2+(y3+2 *
                                a)**2)) * y1+y2 * (y3+a) / (y1**2+y2**2+(y3+2 *
                                a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 *
                                        a)**2)**(1 / 2)+y3+2 * a) * (1 /
                                    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2
                                     * a) * (2 * nu+a / (y1**2+y2**2+(y3+2 *
                                                 a)**2)**(1 / 2))-y1**2 /
                                    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2
                                     * a)**2 * (2 * nu+a / (y1**2+y2**2+(y3+2 *
                                                 a)**2)**(1 / 2)) /
                                    (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1**2
                                    / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                                            2)+y3+2 * a) * a /
                                    (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2)+a /
                                    (y1**2+y2**2+(y3+2 * a)**2)-2 * a * y1**2 /
                                    (y1**2+y2**2+(y3+2 * a)**2)**2)-y2 * (y3+a)
    / (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * (cos(b) /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a)
                 * cos(b)) * (((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     cos(b)+y3+2 * a) * ((1-2 * nu) * cos(b)-a /
                     (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) * 1/tan(b)+(2-2 * nu)
                 * ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) *
                 cos(b))-a * (y3+2 * a) * cos(b) * 1/tan(b) /
                (y1**2+y2**2+(y3+2 * a)**2)) * y1-y2 * (y3+a) /
    (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                2)-y1 * sin(b)+(y3+2 * a) * cos(b))**2 * (cos(b) /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a)
                 * cos(b)) * (((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     cos(b)+y3+2 * a) * ((1-2 * nu) * cos(b)-a /
                     (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) * 1/tan(b)+(2-2 * nu)
                 * ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) *
                 cos(b))-a * (y3+2 * a) * cos(b) * 1/tan(b) /
                (y1**2+y2**2+(y3+2 * a)**2)) * (1 / (y1**2+y2**2+(y3+2 *
                            a)**2)**(1 / 2) * y1-sin(b))+y2 * (y3+a) /
    (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * (-cos(b) /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a)
                 * cos(b))**2 * (((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     cos(b)+y3+2 * a) * ((1-2 * nu) * cos(b)-a /
                     (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) * 1/tan(b)+(2-2 * nu)
                 * ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) *
                 cos(b)) * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                 y1-sin(b))+cos(b) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                     2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * (1 /
                     (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * cos(b) * y1 *
                     ((1-2 * nu) * cos(b)-a / (y1**2+y2**2+(y3+2 *
                             a)**2)**(1 / 2)) * 1/tan(b)+((y1**2+y2**2+(y3+2 *
                                 a)**2)**(1 / 2) * cos(b)+y3+2 * a) * a /
                     (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) * y1 * 1/tan(b)+(2-2 *
                         nu) * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                             sin(b) * y1-1) * cos(b))+2 * a * (y3+2 * a)
                * cos(b) * 1/tan(b) / (y1**2+y2**2+(y3+2 * a)**2)**2 * y1)) /
    pi / (1-nu))+B2 * (1 / 8 * ((-1+2 * nu) * (1 / (y1**2+y2**2+y3**2)**(1 /
                    2) * y1 / ((y1**2+y2**2+y3**2)**(1 / 2)-y3)+1 /
                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * y1 / ((y1**2+y2**2+(y3+2
                            * a)**2)**(1 / 2)+y3+2 * a)-cos(b) * ((1 /
                            (y1**2+y2**2+y3**2)**(1 / 2) * y1-sin(b)) /
                        ((y1**2+y2**2+y3**2)**(1 / 2)-y1 * sin(b)-y3 *
                         cos(b))+(1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                         y1-sin(b)) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                             2)-y1 * sin(b)+(y3+2 * a) * cos(b))))+2 * y1
            * (1 / (y1**2+y2**2+y3**2)**(1 / 2) / ((y1**2+y2**2+y3**2)**(1 /
                        2)-y3)+1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a))+y1**2 * (-1 /
            (y1**2+y2**2+y3**2)**(3 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y3) *
            y1-1 / (y1**2+y2**2+y3**2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y3)**2 *
            y1-1 / (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) / ((y1**2+y2**2+(y3+2 *
                        a)**2)**(1 / 2)+y3+2 * a) * y1-1 / (y1**2+y2**2+(y3+2 *
                        a)**2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 *
                        a)**2 * y1)+cos(b) * ((y1**2+y2**2+y3**2)**(1 / 2) *
                        sin(b)-y1) / (y1**2+y2**2+y3**2)**(1 / 2) /
            ((y1**2+y2**2+y3**2)**(1 / 2)-y1 * sin(b)-y3 * cos(b))+(y1 *
            cos(b)-y3 * sin(b)) * (1 / (y1**2+y2**2+y3**2)**(1 / 2) *
            sin(b) * y1-1) / (y1**2+y2**2+y3**2)**(1 / 2) /
    ((y1**2+y2**2+y3**2)**(1 / 2)-y1 * sin(b)-y3 * cos(b))-(y1 *
    cos(b)-y3 * sin(b)) * ((y1**2+y2**2+y3**2)**(1 / 2) * sin(b)-y1) /
    (y1**2+y2**2+y3**2)**(3 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y1 *
            sin(b)-y3 * cos(b)) * y1-(y1 * cos(b)-y3 * sin(b)) *
    ((y1**2+y2**2+y3**2)**(1 / 2) * sin(b)-y1) / (y1**2+y2**2+y3**2)**(1 /
    2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y1 * sin(b)-y3 * cos(b))**2 * (1 /
        (y1**2+y2**2+y3**2)**(1 / 2) * y1-sin(b))+cos(b) *
    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) / (y1**2+y2**2+(y3+2
        * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
        sin(b)+(y3+2 * a) * cos(b))+(y1 * cos(b)+(y3+2 * a) *
        sin(b)) * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b) *
        y1-1) / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 *
                    a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) * cos(b))-(y1
                * cos(b)+(y3+2 * a) * sin(b)) * ((y1**2+y2**2+(y3+2 *
                        a)**2)**(1 / 2) * sin(b)-y1) / (y1**2+y2**2+(y3+2 *
                        a)**2)**(3 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                            2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * y1-(y1
                            * cos(b)+(y3+2 * a) * sin(b)) *
    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) / (y1**2+y2**2+(y3+2
        * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
        sin(b)+(y3+2 * a) * cos(b))**2 * (1 / (y1**2+y2**2+(y3+2 *
                a)**2)**(1 / 2) * y1-sin(b))) / pi / (1-nu)+1 / 4 * ((1-2
            * nu) * (((2-2 * nu) * 1/tan(b)**2+nu) / (y1**2+y2**2+(y3+2 *
                        a)**2)**(1 / 2) * y1 / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                            / 2)+y3+2 * a)-((2-2 * nu) * 1/tan(b)**2+1) *
                cos(b) * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                    y1-sin(b)) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
                    sin(b)+(y3+2 * a) * cos(b)))-(1-2 * nu) /
        ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a)**2 * ((-1+2 * nu) * y1
        * 1/tan(b)+nu * (y3+2 * a)-a+a * y1 * 1/tan(b) / (y1**2+y2**2+(y3+2 *
                a)**2)**(1 / 2)+y1**2 / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                    2)+y3+2 * a) * (nu+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 /
                        2))) / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * y1+(1-2 *
                    nu) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a) *
        ((-1+2 * nu) * 1/tan(b)+a * 1/tan(b) / (y1**2+y2**2+(y3+2 * a)**2)**(1 /
            2)-a * y1**2 * 1/tan(b) / (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2)+2 * y1
         / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a) * (nu+a /
             (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))-y1**3 / ((y1**2+y2**2+(y3+2
                     * a)**2)**(1 / 2)+y3+2 * a)**2 * (nu+a /
                 (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) / (y1**2+y2**2+(y3+2 *
                     a)**2)**(1 / 2)-y1**3 / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                         2)+y3+2 * a) * a / (y1**2+y2**2+(y3+2 * a)**2)**(3 /
                         2))+(1-2 * nu) * 1/tan(b) / ((y1**2+y2**2+(y3+2 *
                                 a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
                         cos(b))**2 * ((y1 * cos(b)+(y3+2 * a) *
                             sin(b)) * cos(b)-a * ((y1**2+y2**2+(y3+2 *
                                     a)**2)**(1 / 2) * sin(b)-y1) /
                         (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / cos(b)) * (1
                         / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                         y1-sin(b))-(1-2 * nu) * 1/tan(b) /
    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
     cos(b)) * (cos(b)**2-a * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
         sin(b) * y1-1) / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / cos(b)+a
     * ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) /
     (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) / cos(b) * y1)-a * (y3+a) * 1/tan(b)
    / (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2)+3 * a * y1**2 * (y3+a) * 1/tan(b) /
    (y1**2+y2**2+(y3+2 * a)**2)**(5 / 2)-(y3+a) / ((y1**2+y2**2+(y3+2 *
                    a)**2)**(1 / 2)+y3+2 * a)**2 * (2 * nu+1 /
                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * ((1-2 * nu) * y1 *
                    1/tan(b)+a)-y1**2 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a) * (2 * nu+a /
                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))-a * y1**2 /
                (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2)) / (y1**2+y2**2+(y3+2 *
                    a)**2)**(1 / 2) * y1+(y3+a) / ((y1**2+y2**2+(y3+2 *
                            a)**2)**(1 / 2)+y3+2 * a) * (-1 /
                        (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) * ((1-2 * nu) * y1
                            * 1/tan(b)+a) * y1+1 / (y1**2+y2**2+(y3+2 *
                                a)**2)**(1 / 2) * (1-2 * nu) * 1/tan(b)-2 * y1 /
                        (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) /
                        ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a) * (2 *
                        nu+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+y1**3 /
                        (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) /
                        ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a) * (2 *
                        nu+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+y1**3 /
                        (y1**2+y2**2+(y3+2 * a)**2) / ((y1**2+y2**2+(y3+2 *
                                    a)**2)**(1 / 2)+y3+2 * a)**2 * (2 * nu+a /
                                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+y1**3 /
                        (y1**2+y2**2+(y3+2 * a)**2)**2 / ((y1**2+y2**2+(y3+2 *
                                    a)**2)**(1 / 2)+y3+2 * a) * a-2 * a /
                        (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) * y1+3 * a * y1**3
                        / (y1**2+y2**2+(y3+2 * a)**2)**(5 / 2))-(y3+a) * 1/tan(b)
    / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
            cos(b))**2 * (-cos(b) * sin(b)+a * y1 * (y3+2 * a) /
            (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) /
            cos(b)+((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) /
            (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * ((2-2 * nu) *
                cos(b)-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                    cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                        2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * (1+a /
                        (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / cos(b)))) *
    (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * y1-sin(b))+(y3+a) * 1/tan(b) /
    ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
     cos(b)) * (a * (y3+2 * a) / (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) /
     cos(b)-3 * a * y1**2 * (y3+2 * a) / (y1**2+y2**2+(y3+2 * a)**2)**(5 /
         2) / cos(b)+(1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b) *
             y1-1) / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * ((2-2 * nu) *
                 cos(b)-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                         2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * (1+a /
                         (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) /
                         cos(b)))-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     sin(b)-y1) / (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) *
     ((2-2 * nu) * cos(b)-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
         cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
         sin(b)+(y3+2 * a) * cos(b)) * (1+a / (y1**2+y2**2+(y3+2 *
                 a)**2)**(1 / 2) / cos(b))) * y1+((y1**2+y2**2+(y3+2 *
                 a)**2)**(1 / 2) * sin(b)-y1) / (y1**2+y2**2+(y3+2 *
                 a)**2)**(1 / 2) * (-1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                 cos(b) * y1 / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
                     sin(b)+(y3+2 * a) * cos(b)) * (1+a /
                     (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) /
                     cos(b))+((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                         2)-y1 * sin(b)+(y3+2 * a) * cos(b))**2 * (1+a /
                         (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / cos(b)) * (1
                         / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                         y1-sin(b))+((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                         cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                             / 2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * a /
                 (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) / cos(b) * y1))) /
    pi / (1-nu))+B3 * (1 / 8 * y2 * sin(b) * ((1 /
                (y1**2+y2**2+y3**2)**(1 / 2) * sin(b) * y1-1) /
            (y1**2+y2**2+y3**2)**(1 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y1 *
                sin(b)-y3 * cos(b))-((y1**2+y2**2+y3**2)**(1 / 2) *
                sin(b)-y1) / (y1**2+y2**2+y3**2)**(3 / 2) /
            ((y1**2+y2**2+y3**2)**(1 / 2)-y1 * sin(b)-y3 * cos(b)) *
            y1-((y1**2+y2**2+y3**2)**(1 / 2) * sin(b)-y1) /
            (y1**2+y2**2+y3**2)**(1 / 2) / ((y1**2+y2**2+y3**2)**(1 / 2)-y1 *
                sin(b)-y3 * cos(b))**2 * (1 / (y1**2+y2**2+y3**2)**(1 /
                    2) * y1-sin(b))+(1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 /
                        2) * sin(b) * y1-1) / (y1**2+y2**2+(y3+2 *
                            a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                                / 2)-y1 * sin(b)+(y3+2 * a) *
                            cos(b))-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                            sin(b)-y1) / (y1**2+y2**2+(y3+2 * a)**2)**(3 /
                            2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
                                sin(b)+(y3+2 * a) * cos(b)) *
            y1-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * sin(b)-y1) /
            (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 *
                        a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
                cos(b))**2 * (1 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                y1-sin(b))) / pi / (1-nu)+1 / 4 * ((1-2 * nu) * (-y2 /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 * a)**2 * (1+a /
                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) / (y1**2+y2**2+(y3+2 *
                    a)**2)**(1 / 2) * y1-y2 / ((y1**2+y2**2+(y3+2 * a)**2)**(1
                        / 2)+y3+2 * a) * a / (y1**2+y2**2+(y3+2 * a)**2)**(3 /
                        2) * y1+y2 * cos(b) / ((y1**2+y2**2+(y3+2 *
                                    a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
                            cos(b))**2 * (cos(b)+a / (y1**2+y2**2+(y3+2 *
                                    a)**2)**(1 / 2)) * (1 / (y1**2+y2**2+(y3+2
                                        * a)**2)**(1 / 2) * y1-sin(b))+y2 *
                cos(b) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 *
                    sin(b)+(y3+2 * a) * cos(b)) * a / (y1**2+y2**2+(y3+2
                        * a)**2)**(3 / 2) * y1)+y2 * (y3+a) /
            (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) * (a / (y1**2+y2**2+(y3+2 *
                        a)**2)+1 / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 *
                        a)) * y1-y2 * (y3+a) / (y1**2+y2**2+(y3+2 * a)**2)**(1
                        / 2) * (-2 * a / (y1**2+y2**2+(y3+2 * a)**2)**2 * y1-1
                            / ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)+y3+2 *
                                a)**2 / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                            y1)-y2 * (y3+a) * cos(b) / (y1**2+y2**2+(y3+2 *
                                    a)**2)**(3 / 2) / ((y1**2+y2**2+(y3+2 *
                                            a)**2)**(1 / 2)-y1 *
                                    sin(b)+(y3+2 * a) * cos(b)) *
    (((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * cos(b)+y3+2 * a) /
     ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
      cos(b)) * (cos(b)+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+a *
     (y3+2 * a) / (y1**2+y2**2+(y3+2 * a)**2)) * y1-y2 * (y3+a) * cos(b) /
    (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                2)-y1 * sin(b)+(y3+2 * a) * cos(b))**2 *
    (((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * cos(b)+y3+2 * a) /
     ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a) *
      cos(b)) * (cos(b)+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2))+a *
     (y3+2 * a) / (y1**2+y2**2+(y3+2 * a)**2)) * (1 / (y1**2+y2**2+(y3+2 *
             a)**2)**(1 / 2) * y1-sin(b))+y2 * (y3+a) * cos(b) /
    (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * (1 /
                (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) * cos(b) * y1 /
                ((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)-y1 * sin(b)+(y3+2 * a)
                 * cos(b)) * (cos(b)+a / (y1**2+y2**2+(y3+2 * a)**2)**(1
                     / 2))-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                     cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                         2)-y1 * sin(b)+(y3+2 * a) * cos(b))**2 *
                (cos(b)+a / (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2)) * (1 /
                    (y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                    y1-sin(b))-((y1**2+y2**2+(y3+2 * a)**2)**(1 / 2) *
                    cos(b)+y3+2 * a) / ((y1**2+y2**2+(y3+2 * a)**2)**(1 /
                        2)-y1 * sin(b)+(y3+2 * a) * cos(b)) * a /
                (y1**2+y2**2+(y3+2 * a)**2)**(3 / 2) * y1-2 * a * (y3+2 * a) /
                (y1**2+y2**2+(y3+2 * a)**2)**2 * y1)) / pi / (1-nu)))

    return e11

def e22(y1, y2, y3, a, b, nu, B1, B2, B3):
    e22 = (B1 * (1 / 8 * ((1-2 * nu) * (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) *
                        y2 / ((y1**2 + y2**2 + y3**2)**(1 / 2)-y3) + 1 / (y1**2
                            + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 / ((y1**2
                                + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                            a)-cos(b) * (1 / (y1**2 + y2**2 + y3**2)**(1 /
                                    2) * y2 / ((y1**2 + y2**2 + y3**2)**(1 /
                                            2)-y1 * sin(b)-y3 * cos(b)) +
                                1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                * y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                        2)-y1 * sin(b) + (y3 + 2 * a) *
                                    cos(b))))-2 * y2 * (1 / (y1**2 + y2**2 +
                                y3**2)**(1 / 2) / ((y1**2 + y2**2 + y3**2)**(1
                                        / 2)-y3) + 1 / (y1**2 + y2**2 + (y3 + 2
                                            * a)**2)**(1 / 2) / ((y1**2 + y2**2
                                                + (y3 + 2 * a)**2)**(1 / 2) +
                                            y3 + 2 * a)-cos(b) * (1 / (y1**2
                                                    + y2**2 + y3**2)**(1 / 2) /
                                                ((y1**2 + y2**2 + y3**2)**(1 /
                                                    2)-y1 * sin(b)-y3 *
                                                 cos(b)) + 1 / (y1**2 +
                                                 y2**2 + (y3 + 2 * a)**2)**(1 /
                                                 2) / ((y1**2 + y2**2 + (y3 + 2
                                                             * a)**2)**(1 /
                                                             2)-y1 * sin(b)
                                                     + (y3 + 2 * a) *
                                                     cos(b))))-y2**2 * (-1 /
                                             (y1**2 + y2**2 + y3**2)**(3 / 2) /
                                             ((y1**2 + y2**2 + y3**2)**(1 /
                                                 2)-y3) * y2-1 / (y1**2 + y2**2
                                                 + y3**2) / ((y1**2 + y2**2 +
                                                         y3**2)**(1 / 2)-y3)**2
                                             * y2-1 / (y1**2 + y2**2 + (y3 + 2
                                                     * a)**2)**(3 / 2) /
                                             ((y1**2 + y2**2 + (y3 + 2 *
                                                                a)**2)**(1 / 2)
                                              + y3 + 2 * a) * y2-1 / (y1**2 +
                                                  y2**2 + (y3 + 2 * a)**2) /
                                             ((y1**2 + y2**2 + (y3 + 2 *
                                                                a)**2)**(1 / 2)
                                              + y3 + 2 * a)**2 * y2-cos(b) *
                                             (-1 / (y1**2 + y2**2 + y3**2)**(3
                                                                             /
                                                                             2)
                                              / ((y1**2 + y2**2 + y3**2)**(1 /
                                                      2)-y1 * sin(b)-y3 *
                                                  cos(b)) * y2-1 / (y1**2 +
                                                  y2**2 + y3**2) / ((y1**2 +
                                                          y2**2 + y3**2)**(1 /
                                                              2)-y1 *
                                                      sin(b)-y3 *
                                                      cos(b))**2 * y2-1 /
                                              (y1**2 + y2**2 + (y3 + 2 *
                                                                a)**2)**(3 / 2)
                                              / ((y1**2 + y2**2 + (y3 + 2 *
                                                          a)**2)**(1 / 2)-y1 *
                                                  sin(b) + (y3 + 2 * a) *
                                                  cos(b)) * y2-1 / (y1**2 +
                                                  y2**2 + (y3 + 2 * a)**2) /
                                              ((y1**2 + y2**2 + (y3 + 2 *
                                                                 a)**2)**(1 /
                                                                 2)-y1 *
                                               sin(b) + (y3 + 2 * a) *
                                               cos(b))**2 * y2))) / pi /
    (1-nu) + 1 / 4 * ((1-2 * nu) * (((2-2 * nu) * 1/tan(b)**2-nu) / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 / ((y1**2 + y2**2 +
                        (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)-((2-2 * nu) *
                        1/tan(b)**2 + 1-2 * nu) * cos(b) / (y1**2 + y2**2 +
                        (y3 + 2 * a)**2)**(1 / 2) * y2 / ((y1**2 + y2**2 + (y3
                                    + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3
                                    + 2 * a) * cos(b))) + (1-2 * nu) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * (y1
            * 1/tan(b) * (1-2 * nu-a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                    2)) + nu * (y3 + 2 * a)-a + y2**2 / ((y1**2 + y2**2 + (y3 +
                            2 * a)**2)**(1 / 2) + y3 + 2 * a) * (nu + a /
                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))) / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2-(1-2 * nu) / ((y1**2
                        + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (a
                    * y1 * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)
                    * y2 + 2 * y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                        + y3 + 2 * a) * (nu + a / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2))-y2**3 / ((y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2
                    * (nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) /
                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y2**3 / ((y1**2
                            + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) *
                    a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)) + (1-2 *
                    nu) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b)
    / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
            * cos(b))**2 * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                2) * y2 + (1-2 * nu) * (y1 * cos(b) + (y3 + 2 * a) *
                    sin(b)) * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
                    cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)
    * y2 + 3 * a * y2 * (y3 + a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                a)**2)**(5 / 2) * y1-(y3 + a) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) + y3 + 2 * a)**2 * (-2 * nu + 1 /
                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((1-2 * nu) *
                        y1 * 1/tan(b)-a) + y2**2 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + y3 + 2 * a) * (2 * nu + a
                                / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) +
                    a * y2**2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 + (y3 + a) / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (-1 / (y1**2
                + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * ((1-2 * nu) * y1 *
                1/tan(b)-a) * y2 + 2 * y2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                    a) * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                            2))-y2**3 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 /
                            2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                                y3 + 2 * a) * (2 * nu + a / (y1**2 + y2**2 +
                                        (y3 + 2 * a)**2)**(1 / 2))-y2**3 /
            (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) + y3 + 2 * a)**2 * (2 * nu + a / (y1**2
                        + y2**2 + (y3 + 2 * a)**2)**(1 / 2))-y2**3 / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**2 / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) + y3 + 2 * a) * a + 2 * a / (y1**2
                        + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y2-3 * a * y2**3
            / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 / 2))-(y3 + a) / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
            * cos(b))**2 * (cos(b)**2-1 / (y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) * ((1-2 * nu) * (y1 * cos(b) + (y3 + 2 *
                            a) * sin(b)) * 1/tan(b) + a * cos(b)) + a * (y3
                        + 2 * a) * (y1 * cos(b) + (y3 + 2 * a) * sin(b))
            * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)-1 / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (y2**2 * cos(b)**2-a * (y1 * cos(b) + (y3 +
                        2 * a) * sin(b)) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2
                            * a)**2)**(1 / 2) * ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) * cos(b) + y3 + 2 * a)))
    / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 + (y3 + a) / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
            * cos(b)) * (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) *
            ((1-2 * nu) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b)
             + a * cos(b)) * y2-3 * a * (y3 + 2 * a) * (y1 * cos(b) + (y3
                 + 2 * a) * sin(b)) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                     a)**2)**(5 / 2) * y2 + 1 / (y1**2 + y2**2 + (y3 + 2 *
                         a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
                         * cos(b)) * (y2**2 * cos(b)**2-a * (y1 *
                             cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) /
                         (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((y1**2 +
                                 y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b)
                             + y3 + 2 * a)) * y2 + 1 / (y1**2 + y2**2 + (y3 + 2
                                 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 *
                                         a)**2)**(1 / 2)-y1 * sin(b) + (y3 +
                                         2 * a) * cos(b))**2 * (y2**2 *
                                         cos(b)**2-a * (y1 * cos(b) + (y3
                                                 + 2 * a) * sin(b)) * 1/tan(b)
                                         / (y1**2 + y2**2 + (y3 + 2 *
                                                 a)**2)**(1 / 2) * ((y1**2 +
                                                     y2**2 + (y3 + 2 *
                                                         a)**2)**(1 / 2) *
                                                 cos(b) + y3 + 2 * a)) *
    y2-1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 +
                    2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * (2 * y2 * cos(b)**2 + a * (y1 * cos(b) + (y3 + 2
                    * a) * sin(b)) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(3 / 2) * ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) *
            y2-a * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) /
            (y1**2 + y2**2 + (y3 + 2 * a)**2) * cos(b) * y2))) / pi /
    (1-nu)) + B2 * (1 / 8 * ((2-2 * nu) * (-2 / y1 / (1 + y2**2 / y1**2) + 1 /
                (y1 * cos(b)-y3 * sin(b)) / (1 + y2**2 / (y1 *
                        cos(b)-y3 * sin(b))**2) + ((y1**2 + y2**2 +
                        y3**2)**(1 / 2) * sin(b) / (y1 * (y1 * cos(b)-y3
                                * sin(b)) + y2**2 * cos(b)) + y2**2 /
                    (y1**2 + y2**2 + y3**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                            cos(b)-y3 * sin(b)) + y2**2 * cos(b))-2 *
                    y2**2 * (y1**2 + y2**2 + y3**2)**(1 / 2) * sin(b) / (y1
                        * (y1 * cos(b)-y3 * sin(b)) + y2**2 *
                        cos(b))**2 * cos(b)) / (1 + y2**2 * (y1**2 +
                        y2**2 + y3**2) * sin(b)**2 / (y1 * (y1 *
                                cos(b)-y3 * sin(b)) + y2**2 *
                            cos(b))**2) + 1 / (y1 * cos(b) + (y3 + 2 * a)
                        * sin(b)) / (1 + y2**2 / (y1 * cos(b) + (y3 + 2 *
                                a) * sin(b))**2) + ((y1**2 + y2**2 + (y3 + 2
                                    * a)**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                        cos(b) + (y3 + 2 * a) * sin(b)) +
                                    y2**2 * cos(b)) + y2**2 / (y1**2 + y2**2
                                    + (y3 + 2 * a)**2)**(1 / 2) * sin(b) /
                            (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) +
                             y2**2 * cos(b))-2 * y2**2 * (y1**2 + y2**2 +
                             (y3 + 2 * a)**2)**(1 / 2) * sin(b) / (y1 * (y1
                                     * cos(b) + (y3 + 2 * a) * sin(b)) +
                                 y2**2 * cos(b))**2 * cos(b)) / (1 +
                             y2**2 * (y1**2 + y2**2 + (y3 + 2 * a)**2) *
                             sin(b)**2 / (y1 * (y1 * cos(b) + (y3 + 2 *
                                         a) * sin(b)) + y2**2 *
                                 cos(b))**2)) + y1 * (1 / (y1**2 + y2**2 +
                             y3**2)**(1 / 2) / ((y1**2 + y2**2 + y3**2)**(1 /
                                     2)-y3) + 1 / (y1**2 + y2**2 + (y3 + 2 *
                                         a)**2)**(1 / 2) / ((y1**2 + y2**2 +
                                             (y3 + 2 * a)**2)**(1 / 2) + y3 + 2
                                         * a)) + y1 * y2 * (-1 / (y1**2 + y2**2
                                             + y3**2)**(3 / 2) / ((y1**2 +
                                                     y2**2 + y3**2)**(1 /
                                                         2)-y3) * y2-1 / (y1**2
                                                         + y2**2 + y3**2) /
                                         ((y1**2 + y2**2 + y3**2)**(1 /
                                             2)-y3)**2 * y2-1 / (y1**2 + y2**2
                                             + (y3 + 2 * a)**2)**(3 / 2) /
                                         ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                             / 2) + y3 + 2 * a) * y2-1 / (y1**2
                                             + y2**2 + (y3 + 2 * a)**2) /
                                         ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                             / 2) + y3 + 2 * a)**2 * y2)-(y1 *
                                         cos(b)-y3 * sin(b)) / (y1**2 +
                                         y2**2 + y3**2)**(1 / 2) / ((y1**2 +
                                                 y2**2 + y3**2)**(1 / 2)-y1 *
                                             sin(b)-y3 * cos(b))-(y1 *
                                             cos(b) + (y3 + 2 * a) *
                                             sin(b)) / (y1**2 + y2**2 + (y3
                                                 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
     cos(b))-y2 * (-(y1 * cos(b)-y3 * sin(b)) / (y1**2 + y2**2 +
         y3**2)**(3 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2)-y1 * sin(b)-y3
             * cos(b)) * y2-(y1 * cos(b)-y3 * sin(b)) / (y1**2 + y2**2
             + y3**2) / ((y1**2 + y2**2 + y3**2)**(1 / 2)-y1 * sin(b)-y3 *
                 cos(b))**2 * y2-(y1 * cos(b) + (y3 + 2 * a) * sin(b))
     / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
     y2-(y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2
             * a)**2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 *
             sin(b) + (y3 + 2 * a) * cos(b))**2 * y2)) / pi / (1-nu) +
    1 / 4 * ((2-2 * nu) * (1-2 * nu) * (-1 / y1 / (1 + y2**2 / y1**2) + 1 / (y1
                    * cos(b) + (y3 + 2 * a) * sin(b)) / (1 + y2**2 / (y1
                        * cos(b) + (y3 + 2 * a) * sin(b))**2) + ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b) / (y1 *
                        (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2**2 *
                        cos(b)) + y2**2 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) * sin(b) / (y1 * (y1 * cos(b)
                                + (y3 + 2 * a) * sin(b)) + y2**2 *
                            cos(b))-2 * y2**2 * (y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                    cos(b) + (y3 + 2 * a) * sin(b)) +
                                y2**2 * cos(b))**2 * cos(b)) / (1 + y2**2
                            * (y1**2 + y2**2 + (y3 + 2 * a)**2) * sin(b)**2
                            / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b))
                                + y2**2 * cos(b))**2)) * 1/tan(b)**2 + (1-2 *
                        nu) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3
                            + 2 * a) * ((-1 + 2 * nu + a / (y1**2 + y2**2 + (y3
                                            + 2 * a)**2)**(1 / 2)) * 1/tan(b) +
                                y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                        2) + y3 + 2 * a) * (nu + a / (y1**2 +
                                            y2**2 + (y3 + 2 * a)**2)**(1 /
                                            2)))-(1-2 * nu) * y2**2 / ((y1**2 +
                                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                                        y3 + 2 * a)**2 * ((-1 + 2 * nu + a /
                                                (y1**2 + y2**2 + (y3 + 2 *
                                                                  a)**2)**(1 /
                                                                  2)) * 1/tan(b)
                                            + y1 / ((y1**2 + y2**2 + (y3 + 2 *
                                                        a)**2)**(1 / 2) + y3 +
                                                2 * a) * (nu + a / (y1**2 +
                                                        y2**2 + (y3 + 2 *
                                                            a)**2)**(1 / 2))) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + (1-2 * nu) * y2 / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (-a / (y1**2
                + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y2 * 1/tan(b)-y1 / ((y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * (nu
                + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2-y2 / ((y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * a / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y1)-(1-2 * nu) * 1/tan(b)
    / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
            * cos(b)) * (1 + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
            / cos(b)) + (1-2 * nu) * y2**2 * 1/tan(b) / ((y1**2 + y2**2 + (y3
                    + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * (1 + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
            / cos(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + (1-2 *
            nu) * y2**2 * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                    2)-y1 * sin(b) + (y3 + 2 * a) * cos(b)) * a / (y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / cos(b)-a * (y3 +
                    a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) +
    3 * a * y2**2 * (y3 + a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 /
            2) + (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * ((1-2 * nu) *
    1/tan(b)-2 * nu * y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
        a)-a * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (1 / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) + 1 / ((y1**2 + y2**2 + (y3 +
                        2 * a)**2)**(1 / 2) + y3 + 2 * a)))-y2**2 * (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) + y3 + 2 * a) * ((1-2 * nu) * 1/tan(b)-2 * nu
                * y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                    a)-a * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (1
                        / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + 1 /
                        ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                         a)))-y2**2 * (y3 + a) / (y1**2 + y2**2 + (y3 + 2 *
                         a)**2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                         y3 + 2 * a)**2 * ((1-2 * nu) * 1/tan(b)-2 * nu * y1 /
                             ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 +
                              2 * a)-a * y1 / (y1**2 + y2**2 + (y3 + 2 *
                                      a)**2)**(1 / 2) * (1 / (y1**2 + y2**2 +
                                          (y3 + 2 * a)**2)**(1 / 2) + 1 /
                                      ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                          2) + y3 + 2 * a))) + y2 * (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) + y3 + 2 * a) * (2 * nu * y1 / ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 + a * y1 /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (1 / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) + 1 / ((y1**2 + y2**2
                            + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)) * y2-a *
                y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (-1 / (y1**2
                        + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y2-1 / ((y1**2 +
                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2
                    / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2)) + (y3 +
                a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
     cos(b)) * ((-2 + 2 * nu) * cos(b) + ((y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 +
                 (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
             cos(b)) * (1 + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
             cos(b)) + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)
     / cos(b))-y2**2 * (y3 + a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
         a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 *
         sin(b) + (y3 + 2 * a) * cos(b)) * ((-2 + 2 * nu) * cos(b) +
         ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a)
         / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2
                 * a) * cos(b)) * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                         a)**2)**(1 / 2) / cos(b)) + a * (y3 + 2 * a) /
         (y1**2 + y2**2 + (y3 + 2 * a)**2) / cos(b))-y2**2 * (y3 + a) *
    1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * ((-2 + 2 * nu) * cos(b) + ((y1**2 + y2**2 + (y3
                        + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) / cos(b)) + a * (y3 + 2 * a) /
            (y1**2 + y2**2 + (y3 + 2 * a)**2) / cos(b)) + y2 * (y3 + a) *
    1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3
                    + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
            cos(b) * y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 *
                sin(b) + (y3 + 2 * a) * cos(b)) * (1 + a / (y1**2 + y2**2
                    + (y3 + 2 * a)**2)**(1 / 2) / cos(b))-((y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 +
                2 * a) * cos(b))**2 * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) / cos(b)) / (y1**2 + y2**2 + (y3 + 2
                        * a)**2)**(1 / 2) * y2-((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3
                / 2) / cos(b) * y2-2 * a * (y3 + 2 * a) / (y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**2 / cos(b) * y2)) / pi / (1-nu)) +
    B3 * (1 / 8 * ((1-2 * nu) * sin(b) * (1 / (y1**2 + y2**2 + y3**2)**(1 /
                        2) * y2 / ((y1**2 + y2**2 + y3**2)**(1 / 2)-y1 *
                            sin(b)-y3 * cos(b)) + 1 / (y1**2 + y2**2 +
                            (y3 + 2 * a)**2)**(1 / 2) * y2 / ((y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) +
                                (y3 + 2 * a) * cos(b)))-2 * y2 * sin(b) *
                (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                                                          y3**2)**(1 / 2)-y1 *
                                                         sin(b)-y3 *
                                                         cos(b)) + 1 /
                 (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 +
                         (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 *
                             a) * cos(b)))-y2**2 * sin(b) * (-1 / (y1**2
                             + y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                                     y3**2)**(1 / 2)-y1 * sin(b)-y3 *
                                 cos(b)) * y2-1 / (y1**2 + y2**2 + y3**2) /
                         ((y1**2 + y2**2 + y3**2)**(1 / 2)-y1 * sin(b)-y3 *
                          cos(b))**2 * y2-1 / (y1**2 + y2**2 + (y3 + 2 *
                              a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                      a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2
                                      * a) * cos(b)) * y2-1 / (y1**2 + y2**2
                                      + (y3 + 2 * a)**2) / ((y1**2 + y2**2 +
                                          (y3 + 2 * a)**2)**(1 / 2)-y1 *
                                      sin(b) + (y3 + 2 * a) * cos(b))**2
                         * y2)) / pi / (1-nu) + 1 / 4 * ((1-2 * nu) *
                         (-sin(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                                                           2) *
                          y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 *
                              sin(b) + (y3 + 2 * a) * cos(b)) + y2 /
                          ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2
                           * a)**2 * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                                       a)**2)**(1 / 2)) / (y1**2 + y2**2 + (y3
                                       + 2 * a)**2)**(1 / 2) * y1 + y2 /
                          ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2
                           * a) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 /
                               2) * y1-(y1 * cos(b) + (y3 + 2 * a) *
                                   sin(b)) / ((y1**2 + y2**2 + (y3 + 2 *
                                           a)**2)**(1 / 2)-y1 * sin(b) + (y3
                                           + 2 * a) * cos(b))**2 *
                          (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                                                             a)**2)**(1 / 2)) /
                          (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2-(y1 *
                              cos(b) + (y3 + 2 * a) * sin(b)) / ((y1**2 +
                                  y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 *
                              sin(b) + (y3 + 2 * a) * cos(b)) * a /
                          (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y2)-y2 *
    (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (a / (y1**2 + y2**2
                + (y3 + 2 * a)**2) + 1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                    / 2) + y3 + 2 * a)) * y1 + y1 * (y3 + a) / (y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) * (-2 * a / (y1**2 + y2**2 + (y3 + 2
                            * a)**2)**2 * y2-1 / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + y3 + 2 * a)**2 / (y1**2 +
                                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2) + (y3 +
                            a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1
                                * sin(b) + (y3 + 2 * a) * cos(b))**2 *
    (sin(b) * (cos(b)-a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) +
     (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 *
             a)**2)**(1 / 2) * (1 + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2
                     * a)**2))-1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
     ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
      * cos(b)) * (y2**2 * cos(b) * sin(b)-a * (y1 * cos(b) + (y3 +
              2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
      * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a)))
    / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2-(y3 + a) / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
            * cos(b)) * (sin(b) * a / (y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(3 / 2) * y2-(y1 * cos(b) + (y3 + 2 * a) *
                    sin(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) *
            (1 + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)) * y2-2 *
            (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3
                    + 2 * a)**2)**(5 / 2) * a * (y3 + 2 * a) * y2 + 1 / (y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3
                            + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a)
                    * cos(b)) * (y2**2 * cos(b) * sin(b)-a * (y1 *
                        cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2
                        + (y3 + 2 * a)**2)**(1 / 2) * ((y1**2 + y2**2 + (y3 + 2
                                * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a)) *
            y2 + 1 / (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3
                        + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
                cos(b))**2 * (y2**2 * cos(b) * sin(b)-a * (y1 *
                    cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**(1 / 2) * ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * cos(b) + y3 + 2 * a)) *
            y2-1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2
                    + (y3 + 2 * a)**2)**(1 / 2)-y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (2 * y2 * cos(b) * sin(b) + a * (y1 *
                    cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**(3 / 2) * ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) *
                y2-a * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2) * cos(b) * y2))) / pi /
    (1-nu)))

    return e22

def e33(y1, y2, y3, a, b, nu, B1, B2, B3):
    e33 = (B1 * (1 / 8 * y2 * ( -1 / (y1**2 + y2**2 + y3**2)**(3 / 2) * y3 + 1
                    / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (2 * y3
                        + 4 * a) -cos(b) * ((1 / (y1**2 + y2**2 + y3**2)**(1
                                    / 2) * cos(b) * y3 -1) / (y1**2 + y2**2
                                    + y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                                            y3**2)**(1 / 2) -y1 * sin(b) -y3
                                        * cos(b)) -((y1**2 + y2**2 +
                                            y3**2)**(1 / 2) * cos(b) -y3) /
                            (y1**2 + y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2
                                    + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                                cos(b)) * y3 -((y1**2 + y2**2 + y3**2)**(1 /
                                    2) * cos(b) -y3) / (y1**2 + y2**2 +
                                    y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                                            y3**2)**(1 / 2) -y1 * sin(b) -y3
                                        * cos(b))**2 * (1 / (y1**2 + y2**2 +
                                            y3**2)**(1 / 2) * y3 -cos(b))
                            -(1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                    2) * cos(b) * (2 * y3 + 4 * a) + 1) /
                            (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                             sin(b) + (y3 + 2 * a) * cos(b)) + 1 / 2 *
                            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                             cos(b) + y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2
                                 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2
                                         * a)**2)**(1 / 2) -y1 * sin(b) +
                                 (y3 + 2 * a) * cos(b)) * (2 * y3 + 4 * a) +
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
            * (2 * y3 + 4 * a) + cos(b)))) / pi / (1 -nu) + 1 / 4 * ((2
        -2 * nu) * ((1 -2 * nu) * ( -y2 / (y1 * cos(b) + (y3 + 2 * a) *
                    sin(b))**2 * sin(b) / (1 + y2**2 / (y1 * cos(b) +
                        (y3 + 2 * a) * sin(b))**2) + (1 / 2 * y2 / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b) / (y1 *
                        (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2**2 *
                        cos(b)) * (2 * y3 + 4 * a) -y2 * (y1**2 + y2**2 +
                        (y3 + 2 * a)**2)**(1 / 2) * sin(b)**2 / (y1 * (y1 *
                                cos(b) + (y3 + 2 * a) * sin(b)) + y2**2 *
                            cos(b))**2 * y1) / (1 + y2**2 * (y1**2 + y2**2 +
                            (y3 + 2 * a)**2) * sin(b)**2 / (y1 * (y1 *
                                    cos(b) + (y3 + 2 * a) * sin(b)) +
                                y2**2 * cos(b))**2)) * 1/tan(b) -y2 / ((y1**2
                            + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                        a)**2 * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2)) * (1 / 2 / (y1**2 + y2**2
                                    + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3 + 4 *
                                    a) + 1) -1 / 2 * y2 / ((y1**2 + y2**2 + (y3
                                            + 2 * a)**2)**(1 / 2) + y3 + 2 * a)
            * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (2 * y3 + 4 * a)
            + y2 * cos(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1
                * sin(b) + (y3 + 2 * a) * cos(b))**2 * (cos(b) + a /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) * (1 / 2 / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3 + 4 * a) +
                cos(b)) + 1 / 2 * y2 * cos(b) / ((y1**2 + y2**2 + (y3 + 2
                        * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) *
    (2 * y3 + 4 * a)) + y2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 *
        nu / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) + a /
        (y1**2 + y2**2 + (y3 + 2 * a)**2)) -1 / 2 * y2 * (y3 + a) / (y1**2 +
            y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (2 * nu / ((y1**2 + y2**2 + (y3
                        + 2 * a)**2)**(1 / 2) + y3 + 2 * a) + a / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)) * (2 * y3 + 4 * a) + y2 * (y3 +
                a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ( -2 * nu /
                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                     a)**2 * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                             2) * (2 * y3 + 4 * a) + 1) -a / (y1**2 + y2**2 +
                             (y3 + 2 * a)**2)**2 * (2 * y3 + 4 * a)) + y2 *
    cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * (1 -2 * nu -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                    2) * cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2)) -a * (y3 + 2 * a) / (y1**2 + y2**2
                        + (y3 + 2 * a)**2)) -1 / 2 * y2 * (y3 + a) * cos(b)
    / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    * (1 -2 * nu -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3
                + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                    sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a /
                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -a * (y3 + 2 *
                    a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)) * (2 * y3 + 4 * a)
    -y2 * (y3 + a) * cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b))**2 * (1 -2 * nu -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
         * cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
             2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a /
             (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -a * (y3 + 2 * a) /
     (y1**2 + y2**2 + (y3 + 2 * a)**2)) * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(1 / 2) * (2 * y3 + 4 * a) + cos(b)) + y2 * (y3 +
             a) * cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * ( -(1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
         cos(b) * (2 * y3 + 4 * a) + 1) / ((y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
     (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) + ((y1**2 +
             y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
     ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
      * cos(b))**2 * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
          / 2)) * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3
              + 4 * a) + cos(b)) + 1 / 2 * ((y1**2 + y2**2 + (y3 + 2 *
                      a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) / ((y1**2 +
                      y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
                      2 * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                          a)**2)**(3 / 2) * (2 * y3 + 4 * a) -a / (y1**2 +
                          y2**2 + (y3 + 2 * a)**2) + a * (y3 + 2 * a) / (y1**2
                          + y2**2 + (y3 + 2 * a)**2)**2 * (2 * y3 + 4 * a))) /
    pi / (1 -nu)) + B2 * (1 / 8 * (( -1 + 2 * nu) * sin(b) * ((1 / (y1**2
                        + y2**2 + y3**2)**(1 / 2) * y3 -cos(b)) / ((y1**2 +
                            y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                        cos(b)) -(1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * (2 * y3 + 4 * a) + cos(b))
                / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                    (y3 + 2 * a) * cos(b))) -y1 * ( -1 / (y1**2 + y2**2 +
                    y3**2)**(3 / 2) * y3 + 1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(3 / 2) * (2 * y3 + 4 * a)) -sin(b) *
            ((y1**2 + y2**2 + y3**2)**(1 / 2) * cos(b) -y3) / (y1**2 + y2**2
            + y3**2)**(1 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 *
                sin(b) -y3 * cos(b)) + (y1 * cos(b) -y3 * sin(b)) *
            (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) * cos(b) * y3 -1) / (y1**2
                + y2**2 + y3**2)**(1 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2)
                    -y1 * sin(b) -y3 * cos(b)) -(y1 * cos(b) -y3 *
                    sin(b)) * ((y1**2 + y2**2 + y3**2)**(1 / 2) * cos(b)
                    -y3) / (y1**2 + y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                            y3**2)**(1 / 2) -y1 * sin(b) -y3 * cos(b)) *
            y3 -(y1 * cos(b) -y3 * sin(b)) * ((y1**2 + y2**2 + y3**2)**(1
                    / 2) * cos(b) -y3) / (y1**2 + y2**2 + y3**2)**(1 / 2) /
            ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
             cos(b))**2 * (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) * y3
             -cos(b)) -sin(b) * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                 2) * cos(b) + y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 *
                     a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                         2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) -(y1 *
                         cos(b) + (y3 + 2 * a) * sin(b)) * (1 / 2 /
                         (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b)
                         * (2 * y3 + 4 * a) + 1) / (y1**2 + y2**2 + (y3 + 2 *
                             a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2
                                     * a) * cos(b)) + 1 / 2 * (y1 *
                                     cos(b) + (y3 + 2 * a) * sin(b)) *
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    * (2 * y3 + 4 * a) + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * ((y1**2
                + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
            * (2 * y3 + 4 * a) + cos(b))) / pi / (1 -nu) + 1 / 4 * (( -2
            + 2 * nu) * (1 -2 * nu) * 1/tan(b) * ((1 / 2 / (y1**2 + y2**2 + (y3 +
                            2 * a)**2)**(1 / 2) * (2 * y3 + 4 * a) + 1) /
                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)
                -cos(b) * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                        2) * (2 * y3 + 4 * a) + cos(b)) / ((y1**2 + y2**2 +
                            (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2
                                * a) * cos(b))) + (2 -2 * nu) * y1 / ((y1**2
                                + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                            a)**2 * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 *
                                        a)**2)**(1 / 2)) * (1 / 2 / (y1**2 +
                                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2
                                        * y3 + 4 * a) + 1) + 1 / 2 * (2 -2 *
                                        nu) * y1 / ((y1**2 + y2**2 + (y3 + 2 *
                                                    a)**2)**(1 / 2) + y3 + 2 *
                                            a) * a / (y1**2 + y2**2 + (y3 + 2 *
                                                    a)**2)**(3 / 2) * (2 * y3 +
                                                    4 * a) + (2 -2 * nu) *
        sin(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b)
            + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3
                    + 2 * a)**2)**(1 / 2)) -(2 -2 * nu) * (y1 * cos(b) + (y3
                    + 2 * a) * sin(b)) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b))**2 * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2)) * (1 / 2 / (y1**2 + y2**2 + (y3 +
                                2 * a)**2)**(1 / 2) * (2 * y3 + 4 * a) +
                        cos(b)) -1 / 2 * (2 -2 * nu) * (y1 * cos(b) + (y3
                            + 2 * a) * sin(b)) / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                                    a) * cos(b)) * a / (y1**2 + y2**2 + (y3
                                        + 2 * a)**2)**(3 / 2) * (2 * y3 + 4 *
                                        a) + 1 / (y1**2 + y2**2 + (y3 + 2 *
                                                a)**2)**(1 / 2) * ((1 -2 * nu)
                                                * 1/tan(b) -2 * nu * y1 / ((y1**2
                                                        + y2**2 + (y3 + 2 *
                                                            a)**2)**(1 / 2) +
                                                    y3 + 2 * a) -a * y1 /
                                                (y1**2 + y2**2 + (y3 + 2 *
                                                                  a)**2)) -1 /
    2 * (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * ((1 -2 * nu) *
            1/tan(b) -2 * nu * y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                y3 + 2 * a) -a * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)) * (2 *
                y3 + 4 * a) + (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                    / 2) * (2 * nu * y1 / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + y3 + 2 * a)**2 * (1 / 2 /
                                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2
                                    * y3 + 4 * a) + 1) + a * y1 / (y1**2 +
                                    y2**2 + (y3 + 2 * a)**2)**2 * (2 * y3 + 4 *
                                    a)) -1 / ((y1**2 + y2**2 + (y3 + 2 *
                                            a)**2)**(1 / 2) -y1 * sin(b) +
                                    (y3 + 2 * a) * cos(b)) * (cos(b) *
                                    sin(b) + ((y1**2 + y2**2 + (y3 + 2 *
                                                a)**2)**(1 / 2) * cos(b) +
                                        y3 + 2 * a) * 1/tan(b) / (y1**2 + y2**2 +
                                            (y3 + 2 * a)**2)**(1 / 2) * ((2 -2
                                                    * nu) * cos(b) -((y1**2
                                                            + y2**2 + (y3 + 2 *
                                                                a)**2)**(1 / 2)
                                                        * cos(b) + y3 + 2 *
                                                        a) / ((y1**2 + y2**2 +
                                                                (y3 + 2 *
                                                                 a)**2)**(1 /
                                                                 2) -y1 *
                                                            sin(b) + (y3 + 2
                                                                * a) *
                                                            cos(b))) + a /
                                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                    * (sin(b) -(y3 + 2 * a) * (y1 *
                                            cos(b) + (y3 + 2 * a) *
                                            sin(b)) / (y1**2 + y2**2 + (y3 +
                                                2 * a)**2) -(y1 * cos(b) +
                                                (y3 + 2 * a) * sin(b)) *
                                        ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                            / 2) * cos(b) + y3 + 2 * a) /
                                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                            2) / ((y1**2 + y2**2 + (y3 + 2 *
                                                        a)**2)**(1 / 2) -y1 *
                                                sin(b) + (y3 + 2 * a) *
                                                cos(b)))) + (y3 + a) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b))**2 * (cos(b) * sin(b) + ((y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) * 1/tan(b) / (y1**2 +
             y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((2 -2 * nu) * cos(b)
             -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2
                 * a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                     sin(b) + (y3 + 2 * a) * cos(b))) + a / (y1**2 +
                 y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (sin(b) -(y3 + 2 * a) *
                 (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 +
                     (y3 + 2 * a)**2) -(y1 * cos(b) + (y3 + 2 * a) *
                         sin(b)) * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                             2) * cos(b) + y3 + 2 * a) / (y1**2 + y2**2 +
                             (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3
                                         + 2 * a)**2)**(1 / 2) -y1 * sin(b)
                                 + (y3 + 2 * a) * cos(b)))) * (1 / 2 /
                         (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3 +
                             4 * a) + cos(b)) -(y3 + a) / ((y1**2 + y2**2 +
                                 (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                             (y3 + 2 * a) * cos(b)) * ((1 / 2 / (y1**2 +
                                     y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                 cos(b) * (2 * y3 + 4 * a) + 1) * 1/tan(b) /
                             (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((2
                                     -2 * nu) * cos(b) -((y1**2 + y2**2 +
                                             (y3 + 2 * a)**2)**(1 / 2) *
                                         cos(b) + y3 + 2 * a) / ((y1**2 +
                                             y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                         -y1 * sin(b) + (y3 + 2 * a) *
                                         cos(b))) -1 / 2 * ((y1**2 + y2**2 +
                                         (y3 + 2 * a)**2)**(1 / 2) * cos(b)
                                     + y3 + 2 * a) * 1/tan(b) / (y1**2 + y2**2 +
                                         (y3 + 2 * a)**2)**(3 / 2) * ((2 -2 *
                                                 nu) * cos(b) -((y1**2 +
                                                         y2**2 + (y3 + 2 *
                                                             a)**2)**(1 / 2) *
                                                     cos(b) + y3 + 2 * a) /
                                             ((y1**2 + y2**2 + (y3 + 2 *
                                                                a)**2)**(1 / 2)
                                              -y1 * sin(b) + (y3 + 2 * a) *
                                              cos(b))) * (2 * y3 + 4 * a) +
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) *
    1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ( -(1 / 2 / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) * (2 * y3 + 4
                    * a) + 1) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1
                    * sin(b) + (y3 + 2 * a) * cos(b)) + ((y1**2 + y2**2 +
                        (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b))**2 * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) * (2 * y3 + 4 * a) + cos(b))) -1 / 2
    * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (sin(b) -(y3 + 2 * a)
            * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 +
                (y3 + 2 * a)**2) -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) *
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 *
             a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2
                     + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                 * cos(b))) * (2 * y3 + 4 * a) + a / (y1**2 + y2**2 + (y3 +
                 2 * a)**2)**(1 / 2) * ( -(y1 * cos(b) + (y3 + 2 * a) *
                     sin(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2) -(y3 + 2 *
                     a) * sin(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2) + (y3 +
                         2 * a) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) /
                 (y1**2 + y2**2 + (y3 + 2 * a)**2)**2 * (2 * y3 + 4 * a)
                 -sin(b) * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                     cos(b) + y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 *
                         a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                         * cos(b)) -(y1 * cos(b) + (y3 + 2 * a) *
                         sin(b)) * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) * cos(b) * (2 * y3 + 4 * a)
                         + 1) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                 ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                  (y3 + 2 * a) * cos(b)) + 1 / 2 * (y1 * cos(b) + (y3 + 2
                      * a) * sin(b)) * ((y1**2 + y2**2 + (y3 + 2 *
                              a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
                 (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 +
                         (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                             a) * cos(b)) * (2 * y3 + 4 * a) + (y1 *
                             cos(b) + (y3 + 2 * a) * sin(b)) * ((y1**2 +
                                 y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b)
                             + y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 +
                                             2 * a)**2)**(1 / 2) -y1 *
                                     sin(b) + (y3 + 2 * a) * cos(b))**2 *
    (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3 + 4 * a) +
     cos(b))))) / pi / (1 -nu)) + B3 * (1 / 8 * ((2 -2 * nu) * (y2 / (y1
 * cos(b) -y3 * sin(b))**2 * sin(b) / (1 + y2**2 / (y1 * cos(b) -y3
     * sin(b))**2) + (y2 / (y1**2 + y2**2 + y3**2)**(1 / 2) * sin(b) /
 (y1 * (y1 * cos(b) -y3 * sin(b)) + y2**2 * cos(b)) * y3 + y2 * (y1**2
     + y2**2 + y3**2)**(1 / 2) * sin(b)**2 / (y1 * (y1 * cos(b) -y3 *
             sin(b)) + y2**2 * cos(b))**2 * y1) / (1 + y2**2 * (y1**2 +
         y2**2 + y3**2) * sin(b)**2 / (y1 * (y1 * cos(b) -y3 * sin(b))
             + y2**2 * cos(b))**2) + y2 / (y1 * cos(b) + (y3 + 2 * a) *
         sin(b))**2 * sin(b) / (1 + y2**2 / (y1 * cos(b) + (y3 + 2 *
                 a) * sin(b))**2) -(1 / 2 * y2 / (y1**2 + y2**2 + (y3 + 2 *
                     a)**2)**(1 / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3
                             + 2 * a) * sin(b)) + y2**2 * cos(b)) * (2 *
                         y3 + 4 * a) -y2 * (y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) * sin(b)**2 / (y1 * (y1 *
                                     cos(b) + (y3 + 2 * a) * sin(b)) +
                                 y2**2 * cos(b))**2 * y1) / (1 + y2**2 *
                             (y1**2 + y2**2 + (y3 + 2 * a)**2) * sin(b)**2 /
                             (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b))
                              + y2**2 * cos(b))**2)) + y2 * sin(b) * ((1
                          / (y1**2 + y2**2 + y3**2)**(1 / 2) * cos(b) * y3
                          -1) / (y1**2 + y2**2 + y3**2)**(1 / 2) / ((y1**2 +
                                  y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                              cos(b)) -((y1**2 + y2**2 + y3**2)**(1 / 2) *
                              cos(b) -y3) / (y1**2 + y2**2 + y3**2)**(3 / 2)
                      / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                          cos(b)) * y3 -((y1**2 + y2**2 + y3**2)**(1 / 2) *
                          cos(b) -y3) / (y1**2 + y2**2 + y3**2)**(1 / 2) /
                      ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                       cos(b))**2 * (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) *
                       y3 -cos(b)) -(1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                               a)**2)**(1 / 2) * cos(b) * (2 * y3 + 4 * a) +
                       1) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                      ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                       sin(b) + (y3 + 2 * a) * cos(b)) + 1 / 2 * ((y1**2
                           + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3
                       + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) /
                      ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                       sin(b) + (y3 + 2 * a) * cos(b)) * (2 * y3 + 4 * a)
                      + ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b)
                          + y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                              / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                      2) -y1 * sin(b) + (y3 + 2 * a) *
                                  cos(b))**2 * (1 / 2 / (y1**2 + y2**2 + (y3
                                          + 2 * a)**2)**(1 / 2) * (2 * y3 + 4 *
                                          a) + cos(b)))) / pi / (1 -nu) +
    1 / 4 * ((2 -2 * nu) * ( -y2 / (y1 * cos(b) + (y3 + 2 * a) *
                    sin(b))**2 * sin(b) / (1 + y2**2 / (y1 * cos(b) +
                        (y3 + 2 * a) * sin(b))**2) + (1 / 2 * y2 / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b) / (y1 *
                        (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2**2 *
                        cos(b)) * (2 * y3 + 4 * a) -y2 * (y1**2 + y2**2 +
                        (y3 + 2 * a)**2)**(1 / 2) * sin(b)**2 / (y1 * (y1 *
                                cos(b) + (y3 + 2 * a) * sin(b)) + y2**2 *
                            cos(b))**2 * y1) / (1 + y2**2 * (y1**2 + y2**2 +
                            (y3 + 2 * a)**2) * sin(b)**2 / (y1 * (y1 *
                                    cos(b) + (y3 + 2 * a) * sin(b)) +
                                y2**2 * cos(b))**2)) -(2 -2 * nu) * y2 *
            sin(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                sin(b) + (y3 + 2 * a) * cos(b))**2 * (cos(b) + a /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) * (1 / 2 / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3 + 4 * a) +
                cos(b)) -1 / 2 * (2 -2 * nu) * y2 * sin(b) / ((y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2
                    * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(3 / 2) * (2 * y3 + 4 * a) + y2 * sin(b) /
            (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3
                        + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (1 + ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                    cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2)) + a * (y3 + 2 * a) / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2)) -1 / 2 * y2 * (y3 + a) *
    sin(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * (1 + ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2)) + a * (y3 + 2 * a) / (y1**2 + y2**2 +
                    (y3 + 2 * a)**2)) * (2 * y3 + 4 * a) -y2 * (y3 + a) *
    sin(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * (1 + ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2)) + a * (y3 + 2 * a) / (y1**2 + y2**2 +
                    (y3 + 2 * a)**2)) * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) * (2 * y3 + 4 * a) + cos(b)) +
    y2 * (y3 + a) * sin(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * ((1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
         cos(b) * (2 * y3 + 4 * a) + 1) / ((y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
     (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -((y1**2 +
             y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
     ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
      * cos(b))**2 * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
          / 2)) * (1 / 2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (2 * y3
              + 4 * a) + cos(b)) -1 / 2 * ((y1**2 + y2**2 + (y3 + 2 *
                      a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) / ((y1**2 +
                      y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
                      2 * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                          a)**2)**(3 / 2) * (2 * y3 + 4 * a) + a / (y1**2 +
                          y2**2 + (y3 + 2 * a)**2) -a * (y3 + 2 * a) / (y1**2 +
                          y2**2 + (y3 + 2 * a)**2)**2 * (2 * y3 + 4 * a))) /
    pi / (1 -nu)))
    return e33


def e12(y1, y2, y3, a, b, nu, B1, B2, B3):
    e12 = (1 / 2 * B1 * (1 / 8 * ((2 -2 * nu) * ( -2 / y1 / (1 + y2**2 / y1**2)
                        + 1 / (y1 * cos(b) -y3 * sin(b)) / (1 + y2**2 /
                            (y1 * cos(b) -y3 * sin(b))**2) + ((y1**2 +
                                    y2**2 + y3**2)**(1 / 2) * sin(b) / (y1 *
                                        (y1 * cos(b) -y3 * sin(b)) +
                                        y2**2 * cos(b)) + y2**2 / (y1**2 +
                                        y2**2 + y3**2)**(1 / 2) * sin(b) /
                                (y1 * (y1 * cos(b) -y3 * sin(b)) + y2**2
                                 * cos(b)) -2 * y2**2 * (y1**2 + y2**2 +
                                 y3**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                         cos(b) -y3 * sin(b)) + y2**2 *
                                     cos(b))**2 * cos(b)) / (1 + y2**2 *
                                 (y1**2 + y2**2 + y3**2) * sin(b)**2 / (y1 *
                                     (y1 * cos(b) -y3 * sin(b)) + y2**2 *
                                     cos(b))**2) + 1 / (y1 * cos(b) + (y3
                                     + 2 * a) * sin(b)) / (1 + y2**2 / (y1 *
                                         cos(b) + (y3 + 2 * a) *
                                         sin(b))**2) + ((y1**2 + y2**2 + (y3
                                             + 2 * a)**2)**(1 / 2) * sin(b)
                                     / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                                             sin(b)) + y2**2 * cos(b)) +
                                     y2**2 / (y1**2 + y2**2 + (y3 + 2 *
                                             a)**2)**(1 / 2) * sin(b) / (y1
                                             * (y1 * cos(b) + (y3 + 2 * a) *
                                                 sin(b)) + y2**2 *
                                             cos(b)) -2 * y2**2 * (y1**2 +
                                             y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                     * sin(b) / (y1 * (y1 * cos(b) + (y3
                                                 + 2 * a) * sin(b)) + y2**2
                                         * cos(b))**2 * cos(b)) / (1 +
                                     y2**2 * (y1**2 + y2**2 + (y3 + 2 * a)**2)
                                     * sin(b)**2 / (y1 * (y1 * cos(b) +
                                             (y3 + 2 * a) * sin(b)) + y2**2
                                         * cos(b))**2)) -y1 * (1 / (y1**2 +
                                     y2**2 + y3**2)**(1 / 2) / ((y1**2 + y2**2
                                             + y3**2)**(1 / 2) -y3) + 1 /
                                 (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                                 ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                                  y3 + 2 * a)) -y1 * y2 * ( -1 / (y1**2 + y2**2
                                      + y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                                              y3**2)**(1 / 2) -y3) * y2 -1 /
                                  (y1**2 + y2**2 + y3**2) / ((y1**2 + y2**2 +
                                          y3**2)**(1 / 2) -y3)**2 * y2 -1 /
                                  (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) /
                                  ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                                   y3 + 2 * a) * y2 -1 / (y1**2 + y2**2 + (y3 +
                                           2 * a)**2) / ((y1**2 + y2**2 + (y3 +
                                                   2 * a)**2)**(1 / 2) + y3 + 2
                                           * a)**2 * y2) -cos(b) * (((y1**2
                                                   + y2**2 + y3**2)**(1 / 2) *
                                               sin(b) -y1) / (y1**2 + y2**2
                                               + y3**2)**(1 / 2) / ((y1**2 +
                                                       y2**2 + y3**2)**(1 / 2)
                                                   -y1 * sin(b) -y3 *
                                                   cos(b)) + ((y1**2 + y2**2
                                                       + (y3 + 2 * a)**2)**(1 /
                                                       2) * sin(b) -y1) /
                                           (y1**2 + y2**2 + (y3 + 2 *
                                                             a)**2)**(1 / 2) /
                                           ((y1**2 + y2**2 + (y3 + 2 *
                                                              a)**2)**(1 / 2)
                                            -y1 * sin(b) + (y3 + 2 * a) *
                                            cos(b))) -y2 * cos(b) * (1 /
                                        (y1**2 + y2**2 + y3**2) * sin(b) *
                                        y2 / ((y1**2 + y2**2 + y3**2)**(1 / 2)
                                            -y1 * sin(b) -y3 * cos(b))
                                        -((y1**2 + y2**2 + y3**2)**(1 / 2) *
                                            sin(b) -y1) / (y1**2 + y2**2 +
                                            y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                                                    y3**2)**(1 / 2) -y1 *
                                                sin(b) -y3 * cos(b)) * y2
                                        -((y1**2 + y2**2 + y3**2)**(1 / 2) *
                                            sin(b) -y1) / (y1**2 + y2**2 +
                                            y3**2) / ((y1**2 + y2**2 +
                                                    y3**2)**(1 / 2) -y1 *
                                                sin(b) -y3 * cos(b))**2 *
                                        y2 + 1 / (y1**2 + y2**2 + (y3 + 2 *
                                                a)**2) * sin(b) * y2 /
                                        ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                            / 2) -y1 * sin(b) + (y3 + 2 * a)
                                         * cos(b)) -((y1**2 + y2**2 + (y3 +
                                                 2 * a)**2)**(1 / 2) *
                                         sin(b) -y1) / (y1**2 + y2**2 + (y3
                                             + 2 * a)**2)**(3 / 2) / ((y1**2 +
                                                 y2**2 + (y3 + 2 * a)**2)**(1 /
                                                 2) -y1 * sin(b) + (y3 + 2 *
                                                     a) * cos(b)) * y2
    -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b) -y1) / (y1**2 +
            y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))**2 * y2)) /
    pi / (1 -nu) + 1 / 4 * (( -2 + 2 * nu) * (1 -2 * nu) * ( -1 / y1 / (1 +
                    y2**2 / y1**2) + 1 / (y1 * cos(b) + (y3 + 2 * a) *
                        sin(b)) / (1 + y2**2 / (y1 * cos(b) + (y3 + 2 *
                                a) * sin(b))**2) + ((y1**2 + y2**2 + (y3 + 2
                                    * a)**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                        cos(b) + (y3 + 2 * a) * sin(b)) +
                                    y2**2 * cos(b)) + y2**2 / (y1**2 + y2**2
                                    + (y3 + 2 * a)**2)**(1 / 2) * sin(b) /
                            (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) +
                             y2**2 * cos(b)) -2 * y2**2 * (y1**2 + y2**2 +
                             (y3 + 2 * a)**2)**(1 / 2) * sin(b) / (y1 * (y1
                                     * cos(b) + (y3 + 2 * a) * sin(b)) +
                                 y2**2 * cos(b))**2 * cos(b)) / (1 +
                             y2**2 * (y1**2 + y2**2 + (y3 + 2 * a)**2) *
                             sin(b)**2 / (y1 * (y1 * cos(b) + (y3 + 2 *
                                         a) * sin(b)) + y2**2 *
                                 cos(b))**2)) * 1/tan(b)**2 + (1 -2 * nu) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * ((1 -2
                * nu -a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) * 1/tan(b)
            -y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) *
            (nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))) -(1 -2 * nu)
    * y2**2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 *
    ((1 -2 * nu -a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) * 1/tan(b) -y1 /
     ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (nu + a /
     (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))) / (y1**2 + y2**2 + (y3 + 2 *
     a)**2)**(1 / 2) + (1 -2 * nu) * y2 / ((y1**2 + y2**2 + (y3 + 2 *
             a)**2)**(1 / 2) + y3 + 2 * a) * (a / (y1**2 + y2**2 + (y3 + 2 *
                 a)**2)**(3 / 2) * y2 * 1/tan(b) + y1 / ((y1**2 + y2**2 + (y3 + 2
                         * a)**2)**(1 / 2) + y3 + 2 * a)**2 * (nu + a / (y1**2
                         + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) / (y1**2 + y2**2
                     + (y3 + 2 * a)**2)**(1 / 2) * y2 + y2 / ((y1**2 + y2**2 +
                         (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * a / (y1**2 +
                         y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y1) + (1 -2 * nu)
    * cos(b) * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -(1 -2 * nu) * y2**2 *
    cos(b) * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b))**2 * (cos(b) + a / (y1**2
                + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) / (y1**2 + y2**2 + (y3 + 2
                * a)**2)**(1 / 2) -(1 -2 * nu) * y2**2 * cos(b) * 1/tan(b) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) + a * (y3 +
     a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) -3 * a * y2**2 *
    (y3 + a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 / 2) + (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) + y3 + 2 * a) * (( -1 + 2 * nu) * 1/tan(b) +
                y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)
                * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) + a
                * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)) -y2**2 * (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) + y3 + 2 * a) * (( -1 + 2 * nu) * 1/tan(b) +
                y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)
                * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) + a
                * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)) -y2**2 * (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                / 2) + y3 + 2 * a)**2 * (( -1 + 2 * nu) * 1/tan(b) + y1 / ((y1**2
                        + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (2
                    * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) + a
                * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)) + y2 * (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) + y3 + 2 * a) * ( -y1 / ((y1**2 + y2**2 +
                        (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * (2 * nu +
                        a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 -y2 / ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * a /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y1 -2 * a * y1 /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**2 * y2) + (y3 + a) / (y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3
                            + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (cos(b) / ((y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                                (y3 + 2 * a) * cos(b)) * (((y1**2 + y2**2 +
                                        (y3 + 2 * a)**2)**(1 / 2) * cos(b) +
                                    y3 + 2 * a) * ((1 -2 * nu) * cos(b) -a /
                                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                            2)) * 1/tan(b) + (2 -2 * nu) *
                                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                 sin(b) -y1) * cos(b)) -a * (y3 + 2 * a)
                            * cos(b) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)) -y2**2 * (y3 + a) / (y1**2 + y2**2
                                + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                                (y3 + 2 * a) * cos(b)) * (cos(b) /
                                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1
                                 * sin(b) + (y3 + 2 * a) * cos(b)) *
                                (((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                  cos(b) + y3 + 2 * a) * ((1 -2 * nu) *
                                  cos(b) -a / (y1**2 + y2**2 + (y3 + 2 *
                                          a)**2)**(1 / 2)) * 1/tan(b) + (2 -2 *
                                      nu) * ((y1**2 + y2**2 + (y3 + 2 *
                                                  a)**2)**(1 / 2) * sin(b)
                                          -y1) * cos(b)) -a * (y3 + 2 * a) *
                                cos(b) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                                        a)**2)) -y2**2 * (y3 + a) / (y1**2 +
                                    y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2
                                        + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                                    sin(b) + (y3 + 2 * a) * cos(b))**2 *
    (cos(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                  (y3 + 2 * a) * cos(b)) * (((y1**2 + y2**2 + (y3 + 2 *
                              a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) * ((1
                              -2 * nu) * cos(b) -a / (y1**2 + y2**2 + (y3 +
                                      2 * a)**2)**(1 / 2)) * 1/tan(b) + (2 -2 *
                                  nu) * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                          / 2) * sin(b) -y1) * cos(b)) -a
     * (y3 + 2 * a) * cos(b) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)) +
    y2 * (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                a) * cos(b)) * ( -cos(b) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b))**2 * (((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                            2) * cos(b) + y3 + 2 * a) * ((1 -2 * nu) *
                            cos(b) -a / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2)) * 1/tan(b) + (2 -2 * nu) *
                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b)
                     -y1) * cos(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                     / 2) * y2 + cos(b) / ((y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                         * cos(b)) * (1 / (y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) * cos(b) * y2 * ((1 -2 *
                                     nu) * cos(b) -a / (y1**2 + y2**2 + (y3
                                             + 2 * a)**2)**(1 / 2)) * 1/tan(b) +
                         ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                          cos(b) + y3 + 2 * a) * a / (y1**2 + y2**2 + (y3 +
                              2 * a)**2)**(3 / 2) * y2 * 1/tan(b) + (2 -2 * nu) /
                         (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b)
                         * y2 * cos(b)) + 2 * a * (y3 + 2 * a) * cos(b) *
                1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**2 * y2)) / pi /
    (1 -nu)) + 1 / 2 * B2 * (1 / 8 * (( -1 + 2 * nu) * (1 / (y1**2 + y2**2 +
                    y3**2)**(1 / 2) * y2 / ((y1**2 + y2**2 + y3**2)**(1 / 2)
                        -y3) + 1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)
                -cos(b) * (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) * y2 /
                    ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                     cos(b)) + 1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                     2) * y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1
                         * sin(b) + (y3 + 2 * a) * cos(b)))) + y1**2 * (
                 -1 / (y1**2 + y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                         y3**2)**(1 / 2) -y3) * y2 -1 / (y1**2 + y2**2 + y3**2)
                 / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y3)**2 * y2 -1 / (y1**2 +
                     y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3
                             + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * y2 -1 /
                 (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2
                             * a)**2)**(1 / 2) + y3 + 2 * a)**2 * y2) + (y1 *
                     cos(b) -y3 * sin(b)) / (y1**2 + y2**2 + y3**2) *
            sin(b) * y2 / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 * sin(b)
                -y3 * cos(b)) -(y1 * cos(b) -y3 * sin(b)) * ((y1**2 +
                    y2**2 + y3**2)**(1 / 2) * sin(b) -y1) / (y1**2 + y2**2 +
                    y3**2)**(3 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 *
                        sin(b) -y3 * cos(b)) * y2 -(y1 * cos(b) -y3 *
                        sin(b)) * ((y1**2 + y2**2 + y3**2)**(1 / 2) *
                        sin(b) -y1) / (y1**2 + y2**2 + y3**2) / ((y1**2 +
                            y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                        cos(b))**2 * y2 + (y1 * cos(b) + (y3 + 2 * a) *
                        sin(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2) *
    sin(b) * y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) -(y1 * cos(b) + (y3 + 2 *
                a) * sin(b)) * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                sin(b) -y1) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * y2 -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) * ((y1**2 +
         y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b) -y1) / (y1**2 + y2**2 +
     (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
         sin(b) + (y3 + 2 * a) * cos(b))**2 * y2) / pi / (1 -nu) + 1 /
    4 * ((1 -2 * nu) * (((2 -2 * nu) * 1/tan(b)**2 + nu) / (y1**2 + y2**2 + (y3 +
                        2 * a)**2)**(1 / 2) * y2 / ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) + y3 + 2 * a) -((2 -2 * nu) *
                            1/tan(b)**2 + 1) * cos(b) / (y1**2 + y2**2 + (y3 +
                                2 * a)**2)**(1 / 2) * y2 / ((y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                                (y3 + 2 * a) * cos(b))) -(1 -2 * nu) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * ((
                -1 + 2 * nu) * y1 * 1/tan(b) + nu * (y3 + 2 * a) -a + a * y1 *
            1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y1**2 /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (nu + a
            / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))) / (y1**2 + y2**2 +
        (y3 + 2 * a)**2)**(1 / 2) * y2 + (1 -2 * nu) / ((y1**2 + y2**2 + (y3 +
                    2 * a)**2)**(1 / 2) + y3 + 2 * a) * ( -a * y1 * 1/tan(b) /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y2 -y1**2 /
                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 *
                (nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) / (y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 -y1**2 / ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * a /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y2) + (1 -2 * nu)
            * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                sin(b) + (y3 + 2 * a) * cos(b))**2 * ((y1 * cos(b) +
                    (y3 + 2 * a) * sin(b)) * cos(b) -a * ((y1**2 + y2**2
                        + (y3 + 2 * a)**2)**(1 / 2) * sin(b) -y1) / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) / cos(b)) / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2 -(1 -2 * nu) * 1/tan(b) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * ( -a / (y1**2 + y2**2 + (y3 + 2 * a)**2) * sin(b) * y2 /
     cos(b) + a * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b)
         -y1) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / cos(b) * y2) +
    3 * a * y2 * (y3 + a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 / 2)
    * y1 -(y3 + a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
            a)**2 * (2 * nu + 1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                ((1 -2 * nu) * y1 * 1/tan(b) + a) -y1**2 / (y1**2 + y2**2 + (y3 +
                    2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) + y3 + 2 * a) * (2 * nu + a /
                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -a * y1**2
                / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)) / (y1**2 + y2**2
                + (y3 + 2 * a)**2)**(1 / 2) * y2 + (y3 + a) / ((y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * ( -1 / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(3 / 2) * ((1 -2 * nu) * y1 *
                        1/tan(b) + a) * y2 + y1**2 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + y3 + 2 * a) * (2 * nu + a
                                / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) *
                    y2 + y1**2 / (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 +
                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2
                    * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))
                    * y2 + y1**2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**2 /
                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) *
                    a * y2 + 3 * a * y1**2 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(5 / 2) * y2) -(y3 + a) * 1/tan(b) / ((y1**2
                            + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b)
                        + (y3 + 2 * a) * cos(b))**2 * ( -cos(b) *
                        sin(b) + a * y1 * (y3 + 2 * a) / (y1**2 + y2**2 +
                            (y3 + 2 * a)**2)**(3 / 2) / cos(b) + ((y1**2 +
                                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                sin(b) -y1) / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) * ((2 -2 * nu) * cos(b)
                                    -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                            2) * cos(b) + y3 + 2 * a) /
                                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                     -y1 * sin(b) + (y3 + 2 * a) *
                                     cos(b)) * (1 + a / (y1**2 + y2**2 + (y3
                                             + 2 * a)**2)**(1 / 2) /
                                     cos(b)))) / (y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) * y2 + (y3 + a) * 1/tan(b) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * ( -3 * a * y1 * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 *
             a)**2)**(5 / 2) / cos(b) * y2 + 1 / (y1**2 + y2**2 + (y3 + 2 *
                 a)**2) * sin(b) * y2 * ((2 -2 * nu) * cos(b) -((y1**2 +
                         y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2
                     * a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                         sin(b) + (y3 + 2 * a) * cos(b)) * (1 + a /
                         (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                         cos(b))) -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                         2) * sin(b) -y1) / (y1**2 + y2**2 + (y3 + 2 *
                             a)**2)**(3 / 2) * ((2 -2 * nu) * cos(b)
                             -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                 cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 +
                                     (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b)
                                 + (y3 + 2 * a) * cos(b)) * (1 + a / (y1**2
                                     + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                                 cos(b))) * y2 + ((y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(1 / 2) * sin(b) -y1) / (y1**2
                                 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ( -1 /
                                 (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                 cos(b) * y2 / ((y1**2 + y2**2 + (y3 + 2 *
                                             a)**2)**(1 / 2) -y1 * sin(b) +
                                     (y3 + 2 * a) * cos(b)) * (1 + a /
                                     (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                     / cos(b)) + ((y1**2 + y2**2 + (y3 + 2 *
                                             a)**2)**(1 / 2) * cos(b) + y3 +
                                     2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                                                 a)**2)**(1 / 2) -y1 *
                                         sin(b) + (y3 + 2 * a) *
                                         cos(b))**2 * (1 + a / (y1**2 +
                                             y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                         / cos(b)) / (y1**2 + y2**2 + (y3 +
                                             2 * a)**2)**(1 / 2) * y2 + ((y1**2
                                                 + y2**2 + (y3 + 2 * a)**2)**(1
                                                 / 2) * cos(b) + y3 + 2 * a)
                                 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                     -y1 * sin(b) + (y3 + 2 * a) *
                                     cos(b)) * a / (y1**2 + y2**2 + (y3 + 2
                                         * a)**2)**(3 / 2) / cos(b) * y2)))
    / pi / (1 -nu)) + 1 / 2 * B3 * (1 / 8 * sin(b) * (((y1**2 + y2**2 +
                    y3**2)**(1 / 2) * sin(b) -y1) / (y1**2 + y2**2 +
                    y3**2)**(1 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 *
                        sin(b) -y3 * cos(b)) + ((y1**2 + y2**2 + (y3 + 2
                                * a)**2)**(1 / 2) * sin(b) -y1) / (y1**2 +
                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2
                                + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                            (y3 + 2 * a) * cos(b))) / pi / (1 -nu) + 1 /
        8 * y2 * sin(b) * (1 / (y1**2 + y2**2 + y3**2) * sin(b) * y2 /
            ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 * sin(b) -y3 * cos(b))
            -((y1**2 + y2**2 + y3**2)**(1 / 2) * sin(b) -y1) / (y1**2 +
                y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1
                    * sin(b) -y3 * cos(b)) * y2 -((y1**2 + y2**2 +
                        y3**2)**(1 / 2) * sin(b) -y1) / (y1**2 + y2**2 +
                        y3**2) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 *
                            sin(b) -y3 * cos(b))**2 * y2 + 1 / (y1**2 +
                            y2**2 + (y3 + 2 * a)**2) * sin(b) * y2 / ((y1**2
                                + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                            sin(b) + (y3 + 2 * a) * cos(b)) -((y1**2 +
                                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b)
                            -y1) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * y2 -((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) * sin(b) -y1) / (y1**2 + y2**2 + (y3
                        + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                            2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))**2 *
            y2) / pi / (1 -nu) + 1 / 4 * ((1 -2 * nu) * (1 / ((y1**2 + y2**2
                            + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (1 + a
                        / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -y2**2 /
                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                     a)**2 * (1 + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                             2)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                    -y2**2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 +
                        2 * a) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)
                    -cos(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                        -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
                    (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                                                         2)) +
                    y2**2 * cos(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                            / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))**2
                    * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                            2)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                    y2**2 * cos(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                            / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
                    a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)) -(y3 + a) /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (a / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2) + 1 / ((y1**2 + y2**2 + (y3 +
                                2 * a)**2)**(1 / 2) + y3 + 2 * a)) + y2**2 *
    (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (a / (y1**2 + y2**2
                + (y3 + 2 * a)**2) + 1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                    / 2) + y3 + 2 * a)) -y2 * (y3 + a) / (y1**2 + y2**2 + (y3 +
                    2 * a)**2)**(1 / 2) * ( -2 * a / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**2 * y2 -1 / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + y3 + 2 * a)**2 / (y1**2 +
                                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y2) + (y3 +
                            a) * cos(b) / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2
                                            * a)**2)**(1 / 2) -y1 * sin(b) +
                                    (y3 + 2 * a) * cos(b)) * (((y1**2 +
                                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                        cos(b) + y3 + 2 * a) / ((y1**2 +
                                            y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                        -y1 * sin(b) + (y3 + 2 * a) *
                                        cos(b)) * (cos(b) + a / (y1**2 +
                                            y2**2 + (y3 + 2 * a)**2)**(1 / 2))
                                    + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 +
                                            2 * a)**2)) -y2**2 * (y3 + a) *
    cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * (((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2)) + a * (y3 + 2 * a) / (y1**2 + y2**2 +
                    (y3 + 2 * a)**2)) -y2**2 * (y3 + a) * cos(b) / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b))**2 * (((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                            2) * cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 +
                                (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3
                                    + 2 * a) * cos(b)) * (cos(b) + a /
                                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))
                    + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)) +
    y2 * (y3 + a) * cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b)
     * y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
             2 * a) * cos(b)) * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                     a)**2)**(1 / 2)) -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                     2) * cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                             a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                     cos(b))**2 * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2
                             * a)**2)**(1 / 2)) / (y1**2 + y2**2 + (y3 + 2 *
                             a)**2)**(1 / 2) * y2 -((y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(1 / 2) * cos(b) + y3 + 2 * a)
     / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
             a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) *
     y2 -2 * a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**2 * y2)) /
    pi / (1 -nu)) + 1 / 2 * B1 * (1 / 8 * ((1 -2 * nu) * (1 / (y1**2 + y2**2
                    + y3**2)**(1 / 2) * y1 / ((y1**2 + y2**2 + y3**2)**(1 / 2)
                        -y3) + 1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)
                -cos(b) * ((1 / (y1**2 + y2**2 + y3**2)**(1 / 2) * y1
                        -sin(b)) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 *
                        sin(b) -y3 * cos(b)) + (1 / (y1**2 + y2**2 + (y3
                                + 2 * a)**2)**(1 / 2) * y1 -sin(b)) /
                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b)
                     + (y3 + 2 * a) * cos(b)))) -y2**2 * ( -1 / (y1**2 +
                 y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2)
                     -y3) * y1 -1 / (y1**2 + y2**2 + y3**2) / ((y1**2 + y2**2 +
                             y3**2)**(1 / 2) -y3)**2 * y1 -1 / (y1**2 + y2**2 +
                             (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3
                                         + 2 * a)**2)**(1 / 2) + y3 + 2 * a) *
             y1 -1 / (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 + (y3
                         + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * y1 -cos(b)
             * ( -1 / (y1**2 + y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                         y3**2)**(1 / 2) -y1 * sin(b) -y3 * cos(b)) * y1
                 -1 / (y1**2 + y2**2 + y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                         y3**2)**(1 / 2) -y1 * sin(b) -y3 * cos(b))**2 *
                 (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) * y1 -sin(b)) -1 /
                 (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 +
                         (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                             a) * cos(b)) * y1 -1 / (y1**2 + y2**2 + (y3 + 2
                                 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2
                                         * a)**2)**(1 / 2) -y1 * sin(b) +
                                 (y3 + 2 * a) * cos(b))**2 * (1 / (y1**2 +
                                     y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1
                                 -sin(b))))) / pi / (1 -nu) + 1 / 4 * ((1
                         -2 * nu) * (((2 -2 * nu) * 1/tan(b)**2 -nu) / (y1**2 +
                                 y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1 /
                             ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 +
                              2 * a) -((2 -2 * nu) * 1/tan(b)**2 + 1 -2 * nu) *
                             cos(b) * (1 / (y1**2 + y2**2 + (y3 + 2 *
                                         a)**2)**(1 / 2) * y1 -sin(b)) /
                             ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                              sin(b) + (y3 + 2 * a) * cos(b))) + (1 -2 *
                          nu) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                              y3 + 2 * a)**2 * (y1 * 1/tan(b) * (1 -2 * nu -a /
                                      (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                          2)) + nu * (y3 + 2 * a) -a + y2**2 /
                                  ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) +
                                   y3 + 2 * a) * (nu + a / (y1**2 + y2**2 + (y3
                                               + 2 * a)**2)**(1 / 2))) / (y1**2
                                       + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1
                     -(1 -2 * nu) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                         + y3 + 2 * a) * ((1 -2 * nu -a / (y1**2 + y2**2 + (y3
                                         + 2 * a)**2)**(1 / 2)) * 1/tan(b) + a *
                             y1**2 * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(3 / 2) -y2**2 / ((y1**2 + y2**2 +
                                         (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                                     a)**2 * (nu + a / (y1**2 + y2**2 + (y3 + 2
                                                 * a)**2)**(1 / 2)) / (y1**2 +
                                             y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                             * y1 -y2**2 / ((y1**2 + y2**2 + (y3 + 2 *
                                         a)**2)**(1 / 2) + y3 + 2 * a) * a /
                             (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y1)
    -(1 -2 * nu) * cos(b) * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) +
                a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) + (1 -2 * nu) *
    (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) / ((y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * (cos(b) + a / (y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2)) * (1 / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) * y1 -sin(b)) + (1 -2 * nu) * (y1 *
                    cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) / ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3
                        + 2 * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(3 / 2) * y1 -a * (y3 + a) * 1/tan(b) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) + 3 * a * y1**2 * (y3 + a) *
    1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 / 2) -(y3 + a) / ((y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * ( -2 * nu
            + 1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((1 -2 * nu) *
                y1 * 1/tan(b) -a) + y2**2 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 *
                    a) * (2 * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                            2)) + a * y2**2 / (y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(3 / 2)) / (y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * y1 + (y3 + a) / ((y1**2 +
                                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2
                                * a) * ( -1 / (y1**2 + y2**2 + (y3 + 2 *
                                            a)**2)**(3 / 2) * ((1 -2 * nu) * y1
                                            * 1/tan(b) -a) * y1 + 1 / (y1**2 +
                                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                    (1 -2 * nu) * 1/tan(b) -y2**2 / (y1**2 +
                                        y2**2 + (y3 + 2 * a)**2)**(3 / 2) /
                                    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                     + y3 + 2 * a) * (2 * nu + a / (y1**2 +
                                             y2**2 + (y3 + 2 * a)**2)**(1 / 2))
                                    * y1 -y2**2 / (y1**2 + y2**2 + (y3 + 2 *
                                            a)**2) / ((y1**2 + y2**2 + (y3 + 2
                                                    * a)**2)**(1 / 2) + y3 + 2
                                            * a)**2 * (2 * nu + a / (y1**2 +
                                                    y2**2 + (y3 + 2 *
                                                        a)**2)**(1 / 2)) * y1
                                    -y2**2 / (y1**2 + y2**2 + (y3 + 2 *
                                            a)**2)**2 / ((y1**2 + y2**2 + (y3 +
                                                    2 * a)**2)**(1 / 2) + y3 +
                                            2 * a) * a * y1 -3 * a * y2**2 /
                                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 / 2)
                                    * y1) -(y3 + a) / ((y1**2 + y2**2 + (y3 + 2
                                                * a)**2)**(1 / 2) -y1 *
                                        sin(b) + (y3 + 2 * a) *
                                        cos(b))**2 * (cos(b)**2 -1 /
                                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                            2) * ((1 -2 * nu) * (y1 * cos(b)
                                                    + (y3 + 2 * a) * sin(b))
                                                * 1/tan(b) + a * cos(b)) + a *
                                        (y3 + 2 * a) * (y1 * cos(b) + (y3 +
                                                2 * a) * sin(b)) * 1/tan(b) /
                                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 /
                                            2) -1 / (y1**2 + y2**2 + (y3 + 2 *
                                                    a)**2)**(1 / 2) / ((y1**2 +
                                                        y2**2 + (y3 + 2 *
                                                            a)**2)**(1 / 2) -y1
                                                    * sin(b) + (y3 + 2 * a)
                                                    * cos(b)) * (y2**2 *
                                                    cos(b)**2 -a * (y1 *
                                                        cos(b) + (y3 + 2 *
                                                            a) * sin(b)) *
                                                    1/tan(b) / (y1**2 + y2**2 +
                                                        (y3 + 2 * a)**2)**(1 /
                                                            2) * ((y1**2 +
                                                                    y2**2 + (y3
                                                                        + 2 *
                                                                        a)**2)**(1
                                                                        / 2) *
                                                                cos(b) + y3
                                                                + 2 * a))) * (1
                                                            / (y1**2 + y2**2 +
                                                                (y3 + 2 *
                                                                 a)**2)**(1 /
                                                                 2) * y1
                                                            -sin(b)) + (y3 +
                                                            a) / ((y1**2 +
                                                                    y2**2 + (y3
                                                                        + 2 *
                                                                        a)**2)**(1
                                                                        / 2)
                                                                -y1 * sin(b)
                                                                + (y3 + 2 * a)
                                                                * cos(b)) *
    (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * ((1 -2 * nu) * (y1 *
                                                                      cos(b)
                                                                      + (y3 + 2
                                                                          * a)
                                                                      *
                                                                      sin(b))
                                                       * 1/tan(b) + a *
                                                       cos(b)) * y1 -1 /
     (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (1 -2 * nu) * cos(b) *
     1/tan(b) + a * (y3 + 2 * a) * cos(b) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2
             * a)**2)**(3 / 2) -3 * a * (y3 + 2 * a) * (y1 * cos(b) + (y3 +
                 2 * a) * sin(b)) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 *
                     a)**2)**(5 / 2) * y1 + 1 / (y1**2 + y2**2 + (y3 + 2 *
                         a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                         * cos(b)) * (y2**2 * cos(b)**2 -a * (y1 *
                             cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) /
                         (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((y1**2 +
                                 y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b)
                             + y3 + 2 * a)) * y1 + 1 / (y1**2 + y2**2 + (y3 + 2
                                 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2
                                         * a)**2)**(1 / 2) -y1 * sin(b) +
                                 (y3 + 2 * a) * cos(b))**2 * (y2**2 *
                                 cos(b)**2 -a * (y1 * cos(b) + (y3 + 2 *
                                         a) * sin(b)) * 1/tan(b) / (y1**2 +
                                         y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                 ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                                  cos(b) + y3 + 2 * a)) * (1 / (y1**2 +
                                  y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1
                              -sin(b)) -1 / (y1**2 + y2**2 + (y3 + 2 *
                                  a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                          a)**2)**(1 / 2) -y1 * sin(b) + (y3
                                          + 2 * a) * cos(b)) * ( -a *
                                          cos(b) * 1/tan(b) / (y1**2 + y2**2 +
                                              (y3 + 2 * a)**2)**(1 / 2) *
                                          ((y1**2 + y2**2 + (y3 + 2 *
                                                             a)**2)**(1 / 2) *
                                           cos(b) + y3 + 2 * a) + a * (y1 *
                                           cos(b) + (y3 + 2 * a) *
                                           sin(b)) * 1/tan(b) / (y1**2 + y2**2
                                           + (y3 + 2 * a)**2)**(3 / 2) *
                                          ((y1**2 + y2**2 + (y3 + 2 *
                                                             a)**2)**(1 / 2) *
                                           cos(b) + y3 + 2 * a) * y1 -a *
                                          (y1 * cos(b) + (y3 + 2 * a) *
                                           sin(b)) * 1/tan(b) / (y1**2 + y2**2
                                           + (y3 + 2 * a)**2) * cos(b) *
                                          y1))) / pi / (1 -nu)) + 1 / 2 * B2
    * (1 / 8 * ((2 -2 * nu) * (2 * y2 / y1**2 / (1 + y2**2 / y1**2) -y2 / (y1 *
                        cos(b) -y3 * sin(b))**2 * cos(b) / (1 + y2**2
                        / (y1 * cos(b) -y3 * sin(b))**2) + (y2 / (y1**2 +
                            y2**2 + y3**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                    cos(b) -y3 * sin(b)) + y2**2 *
                                cos(b)) * y1 -y2 * (y1**2 + y2**2 +
                                y3**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                        cos(b) -y3 * sin(b)) + y2**2 *
                                    cos(b))**2 * (2 * y1 * cos(b) -y3 *
                                    sin(b))) / (1 + y2**2 * (y1**2 + y2**2 +
                                    y3**2) * sin(b)**2 / (y1 * (y1 *
                                            cos(b) -y3 * sin(b)) + y2**2
                                        * cos(b))**2) -y2 / (y1 * cos(b)
                                    + (y3 + 2 * a) * sin(b))**2 * cos(b)
                    / (1 + y2**2 / (y1 * cos(b) + (y3 + 2 * a) *
                            sin(b))**2) + (y2 / (y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                    cos(b) + (y3 + 2 * a) * sin(b)) +
                                y2**2 * cos(b)) * y1 -y2 * (y1**2 + y2**2 +
                                (y3 + 2 * a)**2)**(1 / 2) * sin(b) / (y1 *
                                    (y1 * cos(b) + (y3 + 2 * a) * sin(b))
                                    + y2**2 * cos(b))**2 * (2 * y1 *
                                    cos(b) + (y3 + 2 * a) * sin(b))) / (1
                                + y2**2 * (y1**2 + y2**2 + (y3 + 2 * a)**2) *
                                sin(b)**2 / (y1 * (y1 * cos(b) + (y3 + 2
                                            * a) * sin(b)) + y2**2 *
                                    cos(b))**2)) + y2 * (1 / (y1**2 + y2**2
                                + y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                                        y3**2)**(1 / 2) -y3) + 1 / (y1**2 +
                                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) /
                            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 +
                             2 * a)) + y1 * y2 * ( -1 / (y1**2 + y2**2 +
                                 y3**2)**(3 / 2) / ((y1**2 + y2**2 + y3**2)**(1
                                         / 2) -y3) * y1 -1 / (y1**2 + y2**2 +
                                         y3**2) / ((y1**2 + y2**2 + y3**2)**(1
                                                 / 2) -y3)**2 * y1 -1 / (y1**2
                                                 + y2**2 + (y3 + 2 * a)**2)**(3
                                                 / 2) / ((y1**2 + y2**2 + (y3 +
                                                             2 * a)**2)**(1 /
                                                             2) + y3 + 2 * a) *
                             y1 -1 / (y1**2 + y2**2 + (y3 + 2 * a)**2) /
                             ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 +
                              2 * a)**2 * y1) -y2 * (cos(b) / (y1**2 + y2**2
                                  + y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                                          y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                                      cos(b)) -(y1 * cos(b) -y3 *
                                      sin(b)) / (y1**2 + y2**2 + y3**2)**(3
                                      / 2) / ((y1**2 + y2**2 + y3**2)**(1 / 2)
                                          -y1 * sin(b) -y3 * cos(b)) * y1
                              -(y1 * cos(b) -y3 * sin(b)) / (y1**2 +
                                  y2**2 + y3**2)**(1 / 2) / ((y1**2 + y2**2 +
                                          y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                                      cos(b))**2 * (1 / (y1**2 + y2**2 +
                                          y3**2)**(1 / 2) * y1 -sin(b)) +
                              cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                  / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                                          / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                      cos(b)) -(y1 * cos(b) + (y3 + 2 *
                                          a) * sin(b)) / (y1**2 + y2**2 +
                                          (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 +
                                                  y2**2 + (y3 + 2 * a)**2)**(1
                                                  / 2) -y1 * sin(b) + (y3 +
                                                      2 * a) * cos(b)) * y1
    -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 *
                a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                -y1 * sin(b) + (y3 + 2 * a) * cos(b))**2 * (1 / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1 -sin(b)))) /
    pi / (1 -nu) + 1 / 4 * ((2 -2 * nu) * (1 -2 * nu) * (y2 / y1**2 / (1 +
                    y2**2 / y1**2) -y2 / (y1 * cos(b) + (y3 + 2 * a) *
                        sin(b))**2 * cos(b) / (1 + y2**2 / (y1 *
                            cos(b) + (y3 + 2 * a) * sin(b))**2) + (y2 /
                        (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * sin(b)
                        / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) +
                            y2**2 * cos(b)) * y1 -y2 * (y1**2 + y2**2 + (y3
                                + 2 * a)**2)**(1 / 2) * sin(b) / (y1 * (y1 *
                                    cos(b) + (y3 + 2 * a) * sin(b)) +
                                y2**2 * cos(b))**2 * (2 * y1 * cos(b) +
                                (y3 + 2 * a) * sin(b))) / (1 + y2**2 *
                            (y1**2 + y2**2 + (y3 + 2 * a)**2) * sin(b)**2 /
                            (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) +
                             y2**2 * cos(b))**2)) * 1/tan(b)**2 -(1 -2 * nu) *
            y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 *
            (( -1 + 2 * nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) *
             1/tan(b) + y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2
                 * a) * (nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)))
            / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1 + (1 -2 * nu) *
            y2 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (
                -a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y1 * 1/tan(b) +
                1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) *
                (nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) -y1**2 /
                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 *
                (nu + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) / (y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1**2 / ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * a /
                (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)) + (1 -2 * nu) * y2
    * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
            (y3 + 2 * a) * cos(b))**2 * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) / cos(b)) * (1 / (y1**2 + y2**2 + (y3 +
                        2 * a)**2)**(1 / 2) * y1 -sin(b)) + (1 -2 * nu) * y2
    * 1/tan(b) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
            (y3 + 2 * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                a)**2)**(3 / 2) / cos(b) * y1 + 3 * a * y2 * (y3 + a) *
    1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(5 / 2) * y1 -y2 * (y3 + a) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) + y3 + 2 * a) * ((1 -2 * nu) * 1/tan(b) -2 *
                nu * y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2
                    * a) -a * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) *
                (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + 1 / ((y1**2 +
                                                                        y2**2 +
                                                                        (y3 + 2
                                                                         *
                                                                         a)**2)**(1
                                                                         / 2) +
                                                                       y3 + 2 *
                                                                       a))) *
    y1 -y2 * (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2) / ((y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 * ((1 -2 * nu) *
                1/tan(b) -2 * nu * y1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                        2) + y3 + 2 * a) -a * y1 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) * (1 / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + 1 / ((y1**2 + y2**2 + (y3
                                            + 2 * a)**2)**(1 / 2) + y3 + 2 *
                                    a))) * y1 + y2 * (y3 + a) / (y1**2 + y2**2
                                + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * (
                                    -2 * nu / ((y1**2 + y2**2 + (y3 + 2 *
                                                a)**2)**(1 / 2) + y3 + 2 * a) +
                                    2 * nu * y1**2 / ((y1**2 + y2**2 + (y3 + 2
                                                * a)**2)**(1 / 2) + y3 + 2 *
                                        a)**2 / (y1**2 + y2**2 + (y3 + 2 *
                                                a)**2)**(1 / 2) -a / (y1**2 +
                                                y2**2 + (y3 + 2 * a)**2)**(1 /
                                                2) * (1 / (y1**2 + y2**2 + (y3
                                                            + 2 * a)**2)**(1 /
                                                            2) + 1 / ((y1**2 +
                                                                    y2**2 + (y3
                                                                        + 2 *
                                                                        a)**2)**(1
                                                                        / 2) +
                                                                y3 + 2 * a)) +
                                    a * y1**2 / (y1**2 + y2**2 + (y3 + 2 *
                                            a)**2)**(3 / 2) * (1 / (y1**2 +
                                                y2**2 + (y3 + 2 * a)**2)**(1 /
                                                2) + 1 / ((y1**2 + y2**2 + (y3
                                                            + 2 * a)**2)**(1 /
                                                            2) + y3 + 2 * a))
    -a * y1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ( -1 / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(3 / 2) * y1 -1 / ((y1**2 + y2**2 +
                    (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 / (y1**2 + y2**2
                    + (y3 + 2 * a)**2)**(1 / 2) * y1)) -y2 * (y3 + a) * 1/tan(b)
    / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    * (( -2 + 2 * nu) * cos(b) + ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                * cos(b) + y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (1 + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                    2) / cos(b)) + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 +
                        2 * a)**2) / cos(b)) * y1 -y2 * (y3 + a) * 1/tan(b) /
    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b))**2 * (( -2 + 2 * nu) * cos(b) + ((y1**2 + y2**2 + (y3
                        + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) /
            ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                        a)**2)**(1 / 2) / cos(b)) + a * (y3 + 2 * a) /
            (y1**2 + y2**2 + (y3 + 2 * a)**2) / cos(b)) * (1 / (y1**2 +
                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1 -sin(b)) + y2 * (y3 +
            a) * 1/tan(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) / ((y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 +
                    2 * a) * cos(b)) * (1 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(1 / 2) * cos(b) * y1 / ((y1**2 + y2**2
                                + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                            (y3 + 2 * a) * cos(b)) * (1 + a / (y1**2 + y2**2
                                + (y3 + 2 * a)**2)**(1 / 2) / cos(b))
                    -((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) +
                        y3 + 2 * a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                2) -y1 * sin(b) + (y3 + 2 * a) *
                            cos(b))**2 * (1 + a / (y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) / cos(b)) * (1 / (y1**2
                                    + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1
                                -sin(b)) -((y1**2 + y2**2 + (y3 + 2 *
                                        a)**2)**(1 / 2) * cos(b) + y3 + 2 *
                                a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
                                        2) -y1 * sin(b) + (y3 + 2 * a) *
                                    cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                                        a)**2)**(3 / 2) / cos(b) * y1 -2 * a
                    * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**2 /
                    cos(b) * y1)) / pi / (1 -nu)) + 1 / 2 * B3 * (1 / 8 *
            ((1 -2 * nu) * sin(b) * ((1 / (y1**2 + y2**2 + y3**2)**(1 / 2) *
                    y1 -sin(b)) / ((y1**2 + y2**2 + y3**2)**(1 / 2) -y1 *
                    sin(b) -y3 * cos(b)) + (1 / (y1**2 + y2**2 + (y3 + 2
                            * a)**2)**(1 / 2) * y1 -sin(b)) / ((y1**2 +
                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                        (y3 + 2 * a) * cos(b))) -y2**2 * sin(b) * ( -1 /
                    (y1**2 + y2**2 + y3**2)**(3 / 2) / ((y1**2 + y2**2 +
                            y3**2)**(1 / 2) -y1 * sin(b) -y3 * cos(b)) *
                    y1 -1 / (y1**2 + y2**2 + y3**2)**(1 / 2) / ((y1**2 + y2**2
                            + y3**2)**(1 / 2) -y1 * sin(b) -y3 *
                        cos(b))**2 * (1 / (y1**2 + y2**2 + y3**2)**(1 / 2) *
                        y1 -sin(b)) -1 / (y1**2 + y2**2 + (y3 + 2 *
                            a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                                    a) * cos(b)) * y1 -1 / (y1**2 + y2**2 +
                                    (y3 + 2 * a)**2)**(1 / 2) / ((y1**2 + y2**2
                                            + (y3 + 2 * a)**2)**(1 / 2) -y1 *
                                        sin(b) + (y3 + 2 * a) *
                                        cos(b))**2 * (1 / (y1**2 + y2**2 +
                                            (y3 + 2 * a)**2)**(1 / 2) * y1
                                        -sin(b)))) / pi / (1 -nu) + 1 / 4
    * ((1 -2 * nu) * ( -sin(b) * (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                        / 2) * y1 -sin(b)) / ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                        * cos(b)) -1 / ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) + y3 + 2 * a) * (1 + a / (y1**2
                                + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) + y1**2 /
                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a)**2 *
                (1 + a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)) / (y1**2 +
                    y2**2 + (y3 + 2 * a)**2)**(1 / 2) + y1**2 / ((y1**2 + y2**2
                        + (y3 + 2 * a)**2)**(1 / 2) + y3 + 2 * a) * a / (y1**2
                    + y2**2 + (y3 + 2 * a)**2)**(3 / 2) + cos(b) / ((y1**2 +
                        y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3
                        + 2 * a) * cos(b)) * (cos(b) + a / (y1**2 + y2**2
                            + (y3 + 2 * a)**2)**(1 / 2)) -(y1 * cos(b) + (y3
                            + 2 * a) * sin(b)) / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 *
                                    a) * cos(b))**2 * (cos(b) + a /
                                    (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))
                * (1 / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1
                    -sin(b)) -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) /
                ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) +
                 (y3 + 2 * a) * cos(b)) * a / (y1**2 + y2**2 + (y3 + 2 *
                     a)**2)**(3 / 2) * y1) + (y3 + a) / (y1**2 + y2**2 + (y3 +
                     2 * a)**2)**(1 / 2) * (a / (y1**2 + y2**2 + (y3 + 2 *
                             a)**2) + 1 / ((y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(1 / 2) + y3 + 2 * a)) -y1**2 *
    (y3 + a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2) * (a / (y1**2 + y2**2
                + (y3 + 2 * a)**2) + 1 / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                    / 2) + y3 + 2 * a)) + y1 * (y3 + a) / (y1**2 + y2**2 + (y3
                    + 2 * a)**2)**(1 / 2) * ( -2 * a / (y1**2 + y2**2 + (y3 + 2
                            * a)**2)**2 * y1 -1 / ((y1**2 + y2**2 + (y3 + 2 *
                                    a)**2)**(1 / 2) + y3 + 2 * a)**2 / (y1**2 +
                                y2**2 + (y3 + 2 * a)**2)**(1 / 2) * y1) + (y3 +
                            a) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2)
                                -y1 * sin(b) + (y3 + 2 * a) * cos(b))**2
    * (sin(b) * (cos(b) -a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2))
            + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 +
                (y3 + 2 * a)**2)**(1 / 2) * (1 + a * (y3 + 2 * a) / (y1**2 +
                        y2**2 + (y3 + 2 * a)**2)) -1 / (y1**2 + y2**2 + (y3 + 2
                        * a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                        * cos(b)) * (y2**2 * cos(b) * sin(b) -a * (y1
                            * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 +
                            y2**2 + (y3 + 2 * a)**2)**(1 / 2) * ((y1**2 + y2**2
                                + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 +
                            2 * a))) * (1 / (y1**2 + y2**2 + (y3 + 2 *
                                a)**2)**(1 / 2) * y1 -sin(b)) -(y3 + a) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * (sin(b) * a / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)
     * y1 + cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * (1 + a *
         (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)) -(y1 * cos(b) +
         (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(3 / 2)
     * (1 + a * (y3 + 2 * a) / (y1**2 + y2**2 + (y3 + 2 * a)**2)) * y1 -2 * (y1
         * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 *
             a)**2)**(5 / 2) * a * (y3 + 2 * a) * y1 + 1 / (y1**2 + y2**2 + (y3
                 + 2 * a)**2)**(3 / 2) / ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1
                     / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (y2**2
                     * cos(b) * sin(b) -a * (y1 * cos(b) + (y3 + 2 *
                             a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 *
                                 a)**2)**(1 / 2) * ((y1**2 + y2**2 + (y3 + 2 *
                                         a)**2)**(1 / 2) * cos(b) + y3 + 2 *
                                 a)) * y1 + 1 / (y1**2 + y2**2 + (y3 + 2 *
                                     a)**2)**(1 / 2) / ((y1**2 + y2**2 + (y3 +
                                             2 * a)**2)**(1 / 2) -y1 *
                                     sin(b) + (y3 + 2 * a) * cos(b))**2 *
     (y2**2 * cos(b) * sin(b) -a * (y1 * cos(b) + (y3 + 2 * a) *
                                          sin(b)) / (y1**2 + y2**2 + (y3 + 2
                                              * a)**2)**(1 / 2) * ((y1**2 +
                                                  y2**2 + (y3 + 2 * a)**2)**(1
                                                  / 2) * cos(b) + y3 + 2 *
                                              a)) * (1 / (y1**2 + y2**2 + (y3 +
                                                      2 * a)**2)**(1 / 2) * y1
                                              -sin(b)) -1 / (y1**2 + y2**2 +
                                              (y3 + 2 * a)**2)**(1 / 2) /
    ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) -y1 * sin(b) + (y3 + 2 * a)
     * cos(b)) * ( -a * cos(b) / (y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 /
         2) * ((y1**2 + y2**2 + (y3 + 2 * a)**2)**(1 / 2) * cos(b) + y3 + 2
             * a) + a * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 +
                 y2**2 + (y3 + 2 * a)**2)**(3 / 2) * ((y1**2 + y2**2 + (y3 + 2
                         * a)**2)**(1 / 2) * cos(b) + y3 + 2 * a) * y1 -a *
     (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1**2 + y2**2 + (y3 + 2 *
             a)**2) * cos(b) * y1))) / pi / (1 -nu)))
    return e12


def e13(y1, y2, y3, a, b, nu, B1, B2, B3):
    e13 = (1 / 2 * B1 * (1 / 8 * ((2 -2 * nu) * (y2 / (y1 * cos(b) -y3 *
                            sin(b)) ** 2 * sin(b) / (1 + y2 ** 2 / (y1 * cos(b)
                                -y3 * sin(b)) ** 2) + (y2 / (y1 ** 2 + y2 ** 2
                                + y3 ** 2) ** (1 / 2) * sin(b) / (y1 * (y1 *
                                        cos(b) -y3 * sin(b)) + y2 ** 2 *
                                    cos(b)) * y3 + y2 * (y1 ** 2 + y2 ** 2 + y3
                                    ** 2) ** (1 / 2) * sin(b) ** 2 / (y1 * (y1
                                            * cos(b) -y3 * sin(b)) + y2 ** 2 *
                                        cos(b)) ** 2 * y1) / (1 + y2 ** 2 * (y1
                                        ** 2 + y2 ** 2 + y3 ** 2) * sin(b) ** 2
                                    / (y1 * (y1 * cos(b) -y3 * sin(b)) + y2 **
                                        2 * cos(b)) ** 2) -y2 / (y1 * cos(b) +
                                    (y3 + 2 * a) * sin(b)) ** 2 * sin(b) / (1 +
                                    y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) *
                                        sin(b)) ** 2) + (1 / 2 * y2 / (y1 ** 2
                                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                        2) * sin(b) / (y1 * (y1 * cos(b) + (y3
                                                    + 2 * a) * sin(b)) + y2 **
                                            2 * cos(b)) * (2 * y3 + 4 * a) -y2
                                    * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                    ** (1 / 2) * sin(b) ** 2 / (y1 * (y1 *
                                            cos(b) + (y3 + 2 * a) * sin(b)) +
                                        y2 ** 2 * cos(b)) ** 2 * y1) / (1 + y2
                                    ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                        2) * sin(b) ** 2 / (y1 * (y1 * cos(b) +
                                                (y3 + 2 * a) * sin(b)) + y2 **
                                            2 * cos(b)) ** 2)) -y1 * y2 * ( -1
                                    / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2)
                                    / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2)
                                        -y3) * y3 -1 / (y1 ** 2 + y2 ** 2 + y3
                                            ** 2) ** (1 / 2) / ((y1 ** 2 + y2
                                                    ** 2 + y3 ** 2) ** (1 / 2)
                                                -y3) ** 2 * (1 / (y1 ** 2 + y2
                                                        ** 2 + y3 ** 2) ** (1 /
                                                            2) * y3 -1) -1 / 2
                                    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                    ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                                * a) ** 2) ** (1 / 2) + y3 + 2
                                        * a) * (2 * y3 + 4 * a) -1 / (y1 ** 2 +
                                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                            / 2) / ((y1 ** 2 + y2 ** 2 + (y3 +
                                                        2 * a) ** 2) ** (1 / 2)
                                                + y3 + 2 * a) ** 2 * (1 / 2 /
                                                    (y1 ** 2 + y2 ** 2 + (y3 +
                                                                          2 *
                                                                          a) **
                                                     2) ** (1 / 2) * (2 * y3 +
                                                         4 * a) + 1)) -y2 *
    cos(b) * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) * sin(b) * y3 / ((y1 ** 2 + y2
                    ** 2 + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) -((y1
                        ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b) -y1) /
            (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3
                    ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) * y3 -((y1 ** 2
                        + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b) -y1) / (y1 **
                        2 + y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2
                                + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 *
                            cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2)
                            ** (1 / 2) * y3 -cos(b)) + 1 / 2 / (y1 ** 2 + y2 **
                            2 + (y3 + 2 * a) ** 2) * sin(b) * (2 * y3 + 4 * a)
            / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
                + (y3 + 2 * a) * cos(b)) -1 / 2 * ((y1 ** 2 + y2 ** 2 + (y3 + 2
                        * a) ** 2) ** (1 / 2) * sin(b) -y1) / (y1 ** 2 + y2 **
                    2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (2 * y3 + 4 * a) -((y1 ** 2 + y2 **
                                2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1)
            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                    y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3
                    + 2 * a) * cos(b)) ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3
                            + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) +
                    cos(b)))) / pi / (1 -nu) + 1 / 4 * (( -2 + 2 * nu) * (1 -2
                * nu) * ( -y2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2 *
                    sin(b) / (1 + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) *
                            sin(b)) ** 2) + (1 / 2 * y2 / (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) / (y1 * (y1
                                    * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2
                                * cos(b)) * (2 * y3 + 4 * a) -y2 * (y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                        sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                                sin(b)) + y2 ** 2 * cos(b)) ** 2 * y1) / (1 +
                        y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) *
                        sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                                sin(b)) + y2 ** 2 * cos(b)) ** 2)) * 1/tan(b) **
            2 -(1 -2 * nu) * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                (1 / 2) + y3 + 2 * a) ** 2 * ((1 -2 * nu -a / (y1 ** 2 + y2 **
                            2 + (y3 + 2 * a) ** 2) ** (1 / 2)) * 1/tan(b) -y1 /
                    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 +
                     2 * a) * (nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                         ** (1 / 2))) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                             a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + 1) + (1
                         -2 * nu) * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                 2) ** (1 / 2) + y3 + 2 * a) * (1 / 2 * a / (y1
                                     ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
                                     / 2) * (2 * y3 + 4 * a) * 1/tan(b) + y1 /
                                 ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                     / 2) + y3 + 2 * a) ** 2 * (nu + a / (y1 **
                                         2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                         / 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                                             (y3 + 2 * a) ** 2) ** (1 / 2) * (2
                                                 * y3 + 4 * a) + 1) + 1 / 2 *
                                 y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                     ** (1 / 2) + y3 + 2 * a) * a / (y1 ** 2 +
                                     y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) *
                                 (2 * y3 + 4 * a)) -(1 -2 * nu) * y2 * cos(b) *
    1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
            (y3 + 2 * a) * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** (1 / 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b))
    -1 / 2 * (1 -2 * nu) * y2 * cos(b) * 1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                    * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (2 * y3 + 4 * a)
    + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * y2 * 1/tan(b) -3 /
    2 * a * y2 * (y3 + a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
    (5 / 2) * (2 * y3 + 4 * a) + y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
            a) * (( -1 + 2 * nu) * 1/tan(b) + y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                            * a) ** 2) ** (1 / 2) + y3 + 2 * a) * (2 * nu + a /
                        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + a
                * y1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -1 / 2 * y2 *
    (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * (( -1 +
                2 * nu) * 1/tan(b) + y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                        2) ** (1 / 2) + y3 + 2 * a) * (2 * nu + a / (y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + a * y1 /
            (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * (2 * y3 + 4 * a) -y2 *
    (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) ** 2 * ((
                -1 + 2 * nu) * 1/tan(b) + y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) + y3 + 2 * a) * (2 * nu + a / (y1 ** 2
                            + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + a * y1
            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * (1 / 2 / (y1 ** 2 + y2
                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + 1) +
    y2 * (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1
                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) *
    ( -y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a)
      ** 2 * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2))
      * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 +
              4 * a) + 1) -1 / 2 * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                  2) ** (1 / 2) + y3 + 2 * a) * a / (y1 ** 2 + y2 ** 2 + (y3 +
                      2 * a) ** 2) ** (3 / 2) * (2 * y3 + 4 * a) -a * y1 / (y1
                      ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * (2 * y3 + 4 *
                      a)) + y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                      / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                              2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
    (cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
               + (y3 + 2 * a) * cos(b)) * (((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                       ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) * ((1 -2 * nu) *
                       cos(b) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                       (1 / 2)) * 1/tan(b) + (2 -2 * nu) * ((y1 ** 2 + y2 ** 2 +
                               (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) *
               cos(b)) -a * (y3 + 2 * a) * cos(b) * 1/tan(b) / (y1 ** 2 + y2 ** 2
               + (y3 + 2 * a) ** 2)) -1 / 2 * y2 * (y3 + a) / (y1 ** 2 + y2 **
           2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                   a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
    (cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
               + (y3 + 2 * a) * cos(b)) * (((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                       ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) * ((1 -2 * nu) *
                       cos(b) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                       (1 / 2)) * 1/tan(b) + (2 -2 * nu) * ((y1 ** 2 + y2 ** 2 +
                               (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) *
               cos(b)) -a * (y3 + 2 * a) * cos(b) * 1/tan(b) / (y1 ** 2 + y2 ** 2
               + (y3 + 2 * a) ** 2)) * (2 * y3 + 4 * a) -y2 * (y3 + a) / (y1 **
           2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 +
               (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
           cos(b)) ** 2 * (cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
               (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (((y1 ** 2 + y2
                       ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 *
                   a) * ((1 -2 * nu) * cos(b) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2
                               * a) ** 2) ** (1 / 2)) * 1/tan(b) + (2 -2 * nu) *
               ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b)
                -y1) * cos(b)) -a * (y3 + 2 * a) * cos(b) * 1/tan(b) / (y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) + y2
    * (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2
                + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * ( -cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                            a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b)) ** 2 * (((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2) * cos(b) + y3 + 2 * a) * ((1 -2 * nu) * cos(b)
                        -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                            2)) * 1/tan(b) + (2 -2 * nu) * ((y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) *
                    cos(b)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                    ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) + cos(b) / ((y1 **
                        2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                    sin(b) + (y3 + 2 * a) * cos(b)) * ((1 / 2 / (y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) * (2
                            * y3 + 4 * a) + 1) * ((1 -2 * nu) * cos(b) -a / (y1
                                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2))
                    * 1/tan(b) + 1 / 2 * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                        ** (1 / 2) * cos(b) + y3 + 2 * a) * a / (y1 ** 2 + y2
                        ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (2 * y3 + 4 * a)
                    * 1/tan(b) + 1 / 2 * (2 -2 * nu) / (y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) ** (1 / 2) * sin(b) * (2 * y3 + 4 * a)
                    * cos(b)) -a * cos(b) * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 +
                        2 * a) ** 2) + a * (y3 + 2 * a) * cos(b) * 1/tan(b) / (y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * (2 * y3 + 4
                        * a))) / pi / (1 -nu)) + 1 / 2 * B2 * (1 / 8 * (( -1 +
                        2 * nu) * ((1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 /
                                    2) * y3 -1) / ((y1 ** 2 + y2 ** 2 + y3 **
                                        2) ** (1 / 2) -y3) + (1 / 2 / (y1 ** 2
                                            + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                        (1 / 2) * (2 * y3 + 4 * a) + 1) / ((y1
                                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                        ** (1 / 2) + y3 + 2 * a) -cos(b) * ((1
                                            / (y1 ** 2 + y2 ** 2 + y3 ** 2) **
                                            (1 / 2) * y3 -cos(b)) / ((y1 ** 2 +
                                                y2 ** 2 + y3 ** 2) ** (1 / 2)
                                            -y1 * sin(b) -y3 * cos(b)) + (1 / 2
                                            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                                ** 2) ** (1 / 2) * (2 * y3 + 4
                                                    * a) + cos(b)) / ((y1 ** 2
                                                        + y2 ** 2 + (y3 + 2 *
                                                            a) ** 2) ** (1 / 2)
                                                    -y1 * sin(b) + (y3 + 2 * a)
                                                    * cos(b)))) + y1 ** 2 * (
                                            -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2)
                                            ** (3 / 2) / ((y1 ** 2 + y2 ** 2 +
                                                    y3 ** 2) ** (1 / 2) -y3) *
                                            y3 -1 / (y1 ** 2 + y2 ** 2 + y3 **
                                                2) ** (1 / 2) / ((y1 ** 2 + y2
                                                        ** 2 + y3 ** 2) ** (1 /
                                                            2) -y3) ** 2 * (1 /
                                                            (y1 ** 2 + y2 ** 2
                                                             + y3 ** 2) ** (1 /
                                                                 2) * y3 -1) -1
                                            / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2
                                                    * a) ** 2) ** (3 / 2) /
                                            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                              ** 2) ** (1 / 2) + y3 + 2 * a) *
                                            (2 * y3 + 4 * a) -1 / (y1 ** 2 + y2
                                                ** 2 + (y3 + 2 * a) ** 2) ** (1
                                                / 2) / ((y1 ** 2 + y2 ** 2 +
                                                        (y3 + 2 * a) ** 2) **
                                                    (1 / 2) + y3 + 2 * a) ** 2
                                            * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3
                                                        + 2 * a) ** 2) ** (1 /
                                                        2) * (2 * y3 + 4 * a) +
                                                1)) -sin(b) * ((y1 ** 2 + y2 **
                                                    2 + y3 ** 2) ** (1 / 2) *
                                                sin(b) -y1) / (y1 ** 2 + y2 **
                                                2 + y3 ** 2) ** (1 / 2) / ((y1
                                                        ** 2 + y2 ** 2 + y3 **
                                                        2) ** (1 / 2) -y1 *
                                                    sin(b) -y3 * cos(b)) + (y1
                                                    * cos(b) -y3 * sin(b)) /
    (y1 ** 2 + y2 ** 2 + y3 ** 2) * sin(b) * y3 / ((y1 ** 2 + y2 ** 2 + y3 **
                2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) -(y1 * cos(b) -y3 *
                sin(b)) * ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b)
                -y1) / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 ** 2 +
                        y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 *
                    cos(b)) * y3 -(y1 * cos(b) -y3 * sin(b)) * ((y1 ** 2 + y2
                        ** 2 + y3 ** 2) ** (1 / 2) * sin(b) -y1) / (y1 ** 2 +
                        y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 +
                                y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b))
    ** 2 * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * y3 -cos(b)) + sin(b)
    * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) / (y1
            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 **
                2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) + 1 / 2 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2
            + y2 ** 2 + (y3 + 2 * a) ** 2) * sin(b) * (2 * y3 + 4 * a) / ((y1
                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
            (y3 + 2 * a) * cos(b)) -1 / 2 * (y1 * cos(b) + (y3 + 2 * a) *
            sin(b)) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
            sin(b) -y1) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * (2 * y3 + 4 * a) -(y1 * cos(b) + (y3 + 2 * a) *
        sin(b)) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b)
        -y1) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                * a) * cos(b)) ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                        a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b))) / pi
    / (1 -nu) + 1 / 4 * ((1 -2 * nu) * (((2 -2 * nu) * 1/tan(b) ** 2 + nu) * (1 /
                    2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2
                        * y3 + 4 * a) + 1) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                            ** 2) ** (1 / 2) + y3 + 2 * a) -((2 -2 * nu) *
                            1/tan(b) ** 2 + 1) * cos(b) * (1 / 2 / (y1 ** 2 + y2
                                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3
                                + 4 * a) + cos(b)) / ((y1 ** 2 + y2 ** 2 + (y3
                                        + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
                                + (y3 + 2 * a) * cos(b))) -(1 -2 * nu) / ((y1
                                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                            + y3 + 2 * a) ** 2 * (( -1 + 2 * nu) * y1 * 1/tan(b)
                                + nu * (y3 + 2 * a) -a + a * y1 * 1/tan(b) / (y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) + y1 ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 +
                                                2 * a) ** 2) ** (1 / 2) + y3 +
                                        2 * a) * (nu + a / (y1 ** 2 + y2 ** 2 +
                                                (y3 + 2 * a) ** 2) ** (1 / 2)))
            * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2
                    * y3 + 4 * a) + 1) + (1 -2 * nu) / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * (nu -1 /
                        2 * a * y1 * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                            ** 2) ** (3 / 2) * (2 * y3 + 4 * a) -y1 ** 2 / ((y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) + y3 + 2 * a) ** 2 * (nu + a / (y1 ** 2
                                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                        2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3
                                                + 2 * a) ** 2) ** (1 / 2) * (2
                                                * y3 + 4 * a) + 1) -1 / 2 * y1
                        ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                2) + y3 + 2 * a) * a / (y1 ** 2 + y2 ** 2 + (y3
                                    + 2 * a) ** 2) ** (3 / 2) * (2 * y3 + 4 *
                                    a)) + (1 -2 * nu) * 1/tan(b) / ((y1 ** 2 + y2
                                        ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                                    -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2
    * ((y1 * cos(b) + (y3 + 2 * a) * sin(b)) * cos(b) -a * ((y1 ** 2 + y2 ** 2
                    + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) / (y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / cos(b)) * (1 / 2 /
            (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 *
                a) + cos(b)) -(1 -2 * nu) * 1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3
                        + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) * (cos(b) * sin(b) -1 / 2 * a / (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) * sin(b) * (2 * y3 + 4 * a) / cos(b) + 1
                / 2 * a * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                    sin(b) -y1) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
                    / 2) / cos(b) * (2 * y3 + 4 * a)) -a / (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (3 / 2) * y1 * 1/tan(b) + 3 / 2 * a *
    y1 * (y3 + a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (5 / 2)
    * (2 * y3 + 4 * a) + 1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                2) + y3 + 2 * a) * (2 * nu + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                        a) ** 2) ** (1 / 2) * ((1 -2 * nu) * y1 * 1/tan(b) + a)
                -y1 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
                ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                 a) * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                     (1 / 2)) -a * y1 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                         ** 2) ** (3 / 2)) -(y3 + a) / ((y1 ** 2 + y2 ** 2 +
                             (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) ** 2 *
    (2 * nu + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((1 -2 *
                                                                         nu) *
                                                                        y1 *
                                                                        1/tan(b)
                                                                        + a)
     -y1 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2
             + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * (2 * nu
         + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) -a * y1 ** 2
     / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)) * (1 / 2 / (y1 ** 2
         + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + 1) +
    (y3 + a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
            a) * ( -1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)
                * ((1 -2 * nu) * y1 * 1/tan(b) + a) * (2 * y3 + 4 * a) + 1 / 2 *
                y1 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) /
                ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                 a) * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                     (1 / 2)) * (2 * y3 + 4 * a) + y1 ** 2 / (y1 ** 2 + y2 ** 2
                         + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2
                             + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) ** 2
                * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                        2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                        ** (1 / 2) * (2 * y3 + 4 * a) + 1) + 1 / 2 * y1 ** 2 /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 / ((y1 ** 2 + y2
                        ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * a
                * (2 * y3 + 4 * a) + 3 / 2 * a * y1 ** 2 / (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (5 / 2) * (2 * y3 + 4 * a)) + 1/tan(b)
    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * ( -cos(b) * sin(b) + a * y1 * (y3 + 2 * a) /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / cos(b) +
                ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b)
                 -y1) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                ((2 -2 * nu) * cos(b) -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                    ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (1 + a / (y1 ** 2 + y2 ** 2 + (y3 +
                                    2 * a) ** 2) ** (1 / 2) / cos(b)))) -(y3 +
                        a) * 1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                            ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) **
    2 * ( -cos(b) * sin(b) + a * y1 * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 +
                    2 * a) ** 2) ** (3 / 2) / cos(b) + ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) / (y1 ** 2
                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((2 -2 *
                            nu) * cos(b) -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                    2) ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1
                                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                    (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                    cos(b)) * (1 + a / (y1 ** 2 + y2 ** 2 + (y3
                                            + 2 * a) ** 2) ** (1 / 2) /
                                    cos(b)))) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 *
                                    a) + cos(b)) + (y3 + a) * 1/tan(b) / ((y1 **
                                        2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                        / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                    cos(b)) * (a / (y1 ** 2 + y2 ** 2 + (y3 + 2
                                            * a) ** 2) ** (3 / 2) / cos(b) * y1
                                    -3 / 2 * a * y1 * (y3 + 2 * a) / (y1 ** 2 +
                                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (5 / 2)
                                    / cos(b) * (2 * y3 + 4 * a) + 1 / 2 / (y1
                                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) *
                                    sin(b) * (2 * y3 + 4 * a) * ((2 -2 * nu) *
                                        cos(b) -((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                                    a) ** 2) ** (1 / 2) *
                                            cos(b) + y3 + 2 * a) / ((y1 ** 2 +
                                                y2 ** 2 + (y3 + 2 * a) ** 2) **
                                            (1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                                            * cos(b)) * (1 + a / (y1 ** 2 + y2
                                                ** 2 + (y3 + 2 * a) ** 2) ** (1
                                                / 2) / cos(b))) -1 / 2 * ((y1
                                                ** 2 + y2 ** 2 + (y3 + 2 * a)
                                                ** 2) ** (1 / 2) * sin(b) -y1)
                                    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                    ** (3 / 2) * ((2 -2 * nu) * cos(b) -((y1 **
                                                2 + y2 ** 2 + (y3 + 2 * a) **
                                                2) ** (1 / 2) * cos(b) + y3 + 2
                                            * a) / ((y1 ** 2 + y2 ** 2 + (y3 +
                                                        2 * a) ** 2) ** (1 / 2)
                                                -y1 * sin(b) + (y3 + 2 * a) *
                                                cos(b)) * (1 + a / (y1 ** 2 +
                                                    y2 ** 2 + (y3 + 2 * a) **
                                                    2) ** (1 / 2) / cos(b))) *
    (2 * y3 + 4 * a) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
            sin(b) -y1) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
    ( -(1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) *
        (2 * y3 + 4 * a) + 1) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (1 + a / (y1 ** 2
                    + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / cos(b)) + ((y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) +
                y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                        2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (1 + a
                        / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
                        cos(b)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                            2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) + 1 / 2
      * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 *
          a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
              + (y3 + 2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
              ** 2) ** (3 / 2) / cos(b) * (2 * y3 + 4 * a)))) / pi / (1 -nu)) +
    1 / 2 * B3 * (1 / 8 * y2 * sin(b) * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) *
                sin(b) * y3 / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 *
                    sin(b) -y3 * cos(b)) -((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1
                        / 2) * sin(b) -y1) / (y1 ** 2 + y2 ** 2 + y3 ** 2) **
                (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 *
                    sin(b) -y3 * cos(b)) * y3 -((y1 ** 2 + y2 ** 2 + y3 ** 2)
                    ** (1 / 2) * sin(b) -y1) / (y1 ** 2 + y2 ** 2 + y3 ** 2) **
                (1 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 *
                    sin(b) -y3 * cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2 + y3 **
                        2) ** (1 / 2) * y3 -cos(b)) + 1 / 2 / (y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2) * sin(b) * (2 * y3 + 4 * a) /
                ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                 sin(b) + (y3 + 2 * a) * cos(b)) -1 / 2 * ((y1 ** 2 + y2 ** 2 +
                     (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) / (y1 ** 2 +
                     y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2
                         ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
                     (y3 + 2 * a) * cos(b)) * (2 * y3 + 4 * a) -((y1 ** 2 + y2
                         ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) -y1) /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2
                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
                    + (y3 + 2 * a) * cos(b)) ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2
                        + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) +
                    cos(b))) / pi / (1 -nu) + 1 / 4 * ((1 -2 * nu) * ( -y2 /
                    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 +
                     2 * a) ** 2 * (1 + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                             ** 2) ** (1 / 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                                 (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 *
                                     a) + 1) -1 / 2 * y2 / ((y1 ** 2 + y2 ** 2
                                         + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 +
                                     2 * a) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2
                                             * a) ** 2) ** (3 / 2) * (2 * y3 +
                                             4 * a) + y2 * cos(b) / ((y1 ** 2 +
                                                     y2 ** 2 + (y3 + 2 * a) **
                                                     2) ** (1 / 2) -y1 * sin(b)
                                                 + (y3 + 2 * a) * cos(b)) ** 2
                    * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) +
                    1 / 2 * y2 * cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                            2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
                    * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) *
                    (2 * y3 + 4 * a)) -y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) * (a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                    a) ** 2) + 1 / ((y1 ** 2 + y2 ** 2 + (y3 +
                                            2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                                    a)) + 1 / 2 * y2 * (y3 + a) / (y1 ** 2 + y2
                                    ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (a /
                                    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) + 1
                                    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                        ** (1 / 2) + y3 + 2 * a)) * (2 * y3 + 4
                                    * a) -y2 * (y3 + a) / (y1 ** 2 + y2 ** 2 +
                                        (y3 + 2 * a) ** 2) ** (1 / 2) * ( -a /
                                            (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                             ** 2) ** 2 * (2 * y3 + 4 * a) -1 /
                                            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                              ** 2) ** (1 / 2) + y3 + 2 * a) **
                                            2 * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                                                    (y3 + 2 * a) ** 2) ** (1 /
                                                        2) * (2 * y3 + 4 * a) +
                                                1)) + y2 * cos(b) / (y1 ** 2 +
                                                y2 ** 2 + (y3 + 2 * a) ** 2) **
    (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
            + (y3 + 2 * a) * cos(b)) * (((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                    2) ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2
                        + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                        * a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3
                                + 2 * a) ** 2) ** (1 / 2)) + a * (y3 + 2 * a) /
            (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -1 / 2 * y2 * (y3 + a) *
    cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                * a) * cos(b)) * (((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                    (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3
                            + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2)) + a * (y3 + 2 *
                                a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) *
    (2 * y3 + 4 * a) -y2 * (y3 + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (((y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b)
                    + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b)
                        + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                            2)) + a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 +
                                2 * a) ** 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 *
                                    a) + cos(b)) + y2 * (y3 + a) * cos(b) / (y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                            2) ** (1 / 2) -y1 * sin(b) + (y3 +
                                                2 * a) * cos(b)) * ((1 / 2 /
                                                    (y1 ** 2 + y2 ** 2 + (y3 +
                                                                          2 *
                                                                          a) **
                                                     2) ** (1 / 2) * cos(b) *
                                                    (2 * y3 + 4 * a) + 1) /
                                                ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                                                       a) ** 2)
                                                 ** (1 / 2) -y1 * sin(b) + (y3
                                                     + 2 * a) * cos(b)) *
                                                (cos(b) + a / (y1 ** 2 + y2 **
                                                               2 + (y3 + 2 * a)
                                                               ** 2) ** (1 /
                                                                   2)) -((y1 **
                                                                       2 + y2
                                                                       ** 2 +
                                                                       (y3 + 2
                                                                        * a) **
                                                                       2) ** (1
                                                                           / 2)
                                                                   * cos(b) +
                                                                   y3 + 2 * a)
    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3
                        + 2 * a) ** 2) ** (1 / 2)) * (1 / 2 / (y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) +
                    cos(b)) -1 / 2 * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                    ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                a) ** 2) ** (3 / 2) * (2 * y3 + 4 * a) + a /
    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) -a * (y3 + 2 * a) / (y1 ** 2 + y2
            ** 2 + (y3 + 2 * a) ** 2) ** 2 * (2 * y3 + 4 * a))) / pi / (1 -nu))
    + 1 / 2 * B1 * (1 / 8 * y2 * ( -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 /
                    2) * y1 + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
                        / 2) * y1 -cos(b) * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2)
                            * cos(b) * y1 / ((y1 ** 2 + y2 ** 2 + y3 ** 2) **
                                (1 / 2) -y1 * sin(b) -y3 * cos(b)) -((y1 ** 2 +
                                    y2 ** 2 + y3 ** 2) ** (1 / 2) * cos(b) -y3)
                            / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1
                                    ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 *
                                sin(b) -y3 * cos(b)) * y1 -((y1 ** 2 + y2 ** 2
                                    + y3 ** 2) ** (1 / 2) * cos(b) -y3) / (y1
                                    ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1
                                            ** 2 + y2 ** 2 + y3 ** 2) ** (1 /
                                                2) -y1 * sin(b) -y3 * cos(b))
                            ** 2 * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 /
                                    2) * y1 -sin(b)) -1 / (y1 ** 2 + y2 ** 2 +
                                    (y3 + 2 * a) ** 2) * cos(b) * y1 / ((y1 **
                                            2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                        (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                        cos(b)) + ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                                * a) ** 2) ** (1 / 2) * cos(b)
                                        + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 +
                                            (y3 + 2 * a) ** 2) ** (3 / 2) /
                            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                             -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * y1 + ((y1
                                 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                             * cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3
                                 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 **
                                     2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                                 sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (1 /
                                 (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                     / 2) * y1 -sin(b)))) / pi / (1 -nu) + 1 /
    4 * ((2 -2 * nu) * ((1 -2 * nu) * (y2 / y1 ** 2 / (1 + y2 ** 2 / y1 ** 2)
                    -y2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2 * cos(b) /
                    (1 + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2)
                    + (y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                        * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b))
                            + y2 ** 2 * cos(b)) * y1 -y2 * (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) / (y1 * (y1
                                    * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2
                                * cos(b)) ** 2 * (2 * y1 * cos(b) + (y3 + 2 *
                                    a) * sin(b))) / (1 + y2 ** 2 * (y1 ** 2 +
                                    y2 ** 2 + (y3 + 2 * a) ** 2) * sin(b) ** 2
                                / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) +
                                    y2 ** 2 * cos(b)) ** 2)) * 1/tan(b) -y1 /
                ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                 a) ** 2 * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                         2) ** (1 / 2)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                         2) ** (1 / 2) * y2 -y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                     * a) ** 2) ** (1 / 2) + y3 + 2 * a) * a /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * y1 + y2 *
                cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                    -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (cos(b) + a /
                    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) * (1 /
                    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y1
                    -sin(b)) + y2 * cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * y1) -y2 * (y3 +
        a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (2 * nu /
            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) +
            a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * y1 + y2 * (y3 + a) /
    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ( -2 * nu / ((y1 ** 2
                    + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) **
            2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y1 -2 * a
            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * y1) -y2 * (y3 + a)
    * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                * a) * cos(b)) * (1 -2 * nu -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                        sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1 **
                            2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) -a *
                (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * y1
    -y2 * (y3 + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
            2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (1 -2 * nu -((y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 +
                    2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                            2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b)
                            + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                / 2)) -a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2)) * (1 / (y1 ** 2 + y2 ** 2 +
                                    (y3 + 2 * a) ** 2) ** (1 / 2) * y1 -sin(b))
    + y2 * (y3 + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
            2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                sin(b) + (y3 + 2 * a) * cos(b)) * ( -1 / (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) * y1 / ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                        sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1 **
                            2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + ((y1
                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                        cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                        * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3
                                + 2 * a) ** 2) ** (1 / 2)) * (1 / (y1 ** 2 + y2
                                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y1
                            -sin(b)) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                            ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2
                                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                            sin(b) + (y3 + 2 * a) * cos(b)) * a / (y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * y1 + 2 * a *
                (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 *
                y1)) / pi / (1 -nu)) + 1 / 2 * B2 * (1 / 8 * (( -1 + 2 * nu) *
                sin(b) * ((1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * y1
                        -sin(b)) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2)
                        -y1 * sin(b) -y3 * cos(b)) -(1 / (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) * y1 -sin(b)) / ((y1
                                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                            -y1 * sin(b) + (y3 + 2 * a) * cos(b))) -1 / (y1 **
                        2 + y2 ** 2 + y3 ** 2) ** (1 / 2) + 1 / (y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * ( -1 /
                            (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2) * y1 + 1 /
                            (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)
                            * y1) + cos(b) * ((y1 ** 2 + y2 ** 2 + y3 ** 2) **
                                (1 / 2) * cos(b) -y3) / (y1 ** 2 + y2 ** 2 + y3
                                ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + y3 **
                                        2) ** (1 / 2) -y1 * sin(b) -y3 *
                                    cos(b)) + (y1 * cos(b) -y3 * sin(b)) / (y1
                                    ** 2 + y2 ** 2 + y3 ** 2) * cos(b) * y1 /
                ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 *
                 cos(b)) -(y1 * cos(b) -y3 * sin(b)) * ((y1 ** 2 + y2 ** 2 + y3
                     ** 2) ** (1 / 2) * cos(b) -y3) / (y1 ** 2 + y2 ** 2 + y3
                     ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 /
                             2) -y1 * sin(b) -y3 * cos(b)) * y1 -(y1 * cos(b)
                             -y3 * sin(b)) * ((y1 ** 2 + y2 ** 2 + y3 ** 2) **
                             (1 / 2) * cos(b) -y3) / (y1 ** 2 + y2 ** 2 + y3 **
                             2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) **
                                 (1 / 2) -y1 * sin(b) -y3 * cos(b)) ** 2 * (1 /
                                 (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * y1
                                 -sin(b)) -cos(b) * ((y1 ** 2 + y2 ** 2 + (y3 +
                                         2 * a) ** 2) ** (1 / 2) * cos(b) + y3
                                 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                     ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 +
                                             (y3 + 2 * a) ** 2) ** (1 / 2) -y1
                                         * sin(b) + (y3 + 2 * a) * cos(b)) -(y1
                                         * cos(b) + (y3 + 2 * a) * sin(b)) /
    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) * cos(b) * y1 / ((y1 ** 2 + y2 ** 2
                + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * ((y1 ** 2 + y2 **
                2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) / (y1
            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 **
                2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * y1 + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a)
    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2
                + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                2) * y1 -sin(b))) / pi / (1 -nu) + 1 / 4 * (( -2 + 2 * nu) * (1
                -2 * nu) * 1/tan(b) * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                        2) ** (1 / 2) * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                ** 2) ** (1 / 2) + y3 + 2 * a) -cos(b) * (1 /
                                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) * y1 -sin(b)) / ((y1 ** 2 + y2 ** 2 +
                                        (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                                    sin(b) + (y3 + 2 * a) * cos(b))) -(2 -2 *
                                nu) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                    ** (1 / 2) + y3 + 2 * a) * (2 * nu + a /
                                    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                    (1 / 2)) + (2 -2 * nu) * y1 ** 2 / ((y1 **
                                            2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                        (1 / 2) + y3 + 2 * a) ** 2 * (2 * nu +
                                            a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                                    a) ** 2) ** (1 / 2)) / (y1
                                                ** 2 + y2 ** 2 + (y3 + 2 * a)
                                                ** 2) ** (1 / 2) + (2 -2 * nu)
            * y1 ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) +
                y3 + 2 * a) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
                    / 2) + (2 -2 * nu) * cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                                a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2
                                    + (y3 + 2 * a) ** 2) ** (1 / 2)) -(2 -2 *
                                nu) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
            ** 2) ** (1 / 2)) * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
            (1 / 2) * y1 -sin(b)) -(2 -2 * nu) * (y1 * cos(b) + (y3 + 2 * a) *
            sin(b)) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2
                * a) ** 2) ** (3 / 2) * y1 -(y3 + a) / (y1 ** 2 + y2 ** 2 + (y3
                    + 2 * a) ** 2) ** (3 / 2) * ((1 -2 * nu) * 1/tan(b) -2 * nu *
                    y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) +
                        y3 + 2 * a) -a * y1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                            ** 2)) * y1 + (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 +
                                2 * a) ** 2) ** (1 / 2) * ( -2 * nu / ((y1 ** 2
                                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                        2) + y3 + 2 * a) + 2 * nu * y1 ** 2 /
                                ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                    / 2) + y3 + 2 * a) ** 2 / (y1 ** 2 + y2 **
                                    2 + (y3 + 2 * a) ** 2) ** (1 / 2) -a / (y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) + 2 * a
                                * y1 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                    ** 2) ** 2) + (y3 + a) / ((y1 ** 2 + y2 **
                                        2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                                    sin(b) + (y3 + 2 * a) * cos(b)) ** 2 *
    (cos(b) * sin(b) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                        cos(b) + y3 + 2 * a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) * ((2 -2 * nu) * cos(b)
                            -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2
                                        ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                                    -y1 * sin(b) + (y3 + 2 * a) * cos(b))) + a
     / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (sin(b) -(y3 + 2 *
             a) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2 + y2 ** 2 +
                 (y3 + 2 * a) ** 2) -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) *
         ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2
          * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2
                  + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3
                  + 2 * a) * cos(b)))) * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
              ** 2) ** (1 / 2) * y1 -sin(b)) -(y3 + a) / ((y1 ** 2 + y2 ** 2 +
                  (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
              cos(b)) * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) * cos(b) *
              y1 * 1/tan(b) * ((2 -2 * nu) * cos(b) -((y1 ** 2 + y2 ** 2 + (y3 +
                              2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) /
                  ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                   sin(b) + (y3 + 2 * a) * cos(b))) -((y1 ** 2 + y2 ** 2 + (y3
                       + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) *
              1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * ((2
                      -2 * nu) * cos(b) -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                              2) ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2
                                  + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1
                              * sin(b) + (y3 + 2 * a) * cos(b))) * y1 + ((y1 **
                              2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                          cos(b) + y3 + 2 * a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 +
                          (y3 + 2 * a) ** 2) ** (1 / 2) * ( -1 / (y1 ** 2 + y2
                                  ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b)
                              * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                  ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                  cos(b)) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                      ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) /
                              ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                  2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2
                              * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                  (1 / 2) * y1 -sin(b))) -a / (y1 ** 2 + y2 **
                              2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (sin(b) -(y3
                                  + 2 * a) * (y1 * cos(b) + (y3 + 2 * a) *
                                      sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                          a) ** 2) -(y1 * cos(b) + (y3 + 2 * a)
                                          * sin(b)) * ((y1 ** 2 + y2 ** 2 + (y3
                                                  + 2 * a) ** 2) ** (1 / 2) *
                                          cos(b) + y3 + 2 * a) / (y1 ** 2 + y2
                                          ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                              / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                      / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                  cos(b))) * y1 + a / (y1 ** 2 + y2 ** 2 + (y3
                                  + 2 * a) ** 2) ** (1 / 2) * ( -(y3 + 2 * a) *
                                  cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                      2) + 2 * (y3 + 2 * a) * (y1 * cos(b) +
                                          (y3 + 2 * a) * sin(b)) / (y1 ** 2 +
                                          y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 *
                                  y1 -cos(b) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                              a) ** 2) ** (1 / 2) * cos(b) + y3
                                      + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                              a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                                                  y2 ** 2 + (y3 + 2 * a) ** 2)
                                              ** (1 / 2) -y1 * sin(b) + (y3 + 2
                                                  * a) * cos(b)) -(y1 * cos(b)
                                                  + (y3 + 2 * a) * sin(b)) /
                                  (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) *
                                  cos(b) * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                              a) ** 2) ** (1 / 2) -y1 * sin(b)
                                      + (y3 + 2 * a) * cos(b)) + (y1 * cos(b) +
                                      (y3 + 2 * a) * sin(b)) * ((y1 ** 2 + y2
                                          ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                                      * cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 **
                                      2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1
                                          ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                      ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                      cos(b)) * y1 + (y1 * cos(b) + (y3 + 2 *
                                          a) * sin(b)) * ((y1 ** 2 + y2 ** 2 +
                                              (y3 + 2 * a) ** 2) ** (1 / 2) *
                                          cos(b) + y3 + 2 * a) / (y1 ** 2 + y2
                                          ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                    ** 2) ** (1 / 2) * y1 -sin(b))))) / pi / (1 -nu)) + 1 / 2 *
    B3 * (1 / 8 * ((2 -2 * nu) * ( -y2 / (y1 * cos(b) -y3 * sin(b)) ** 2 *
                    cos(b) / (1 + y2 ** 2 / (y1 * cos(b) -y3 * sin(b)) ** 2) +
                    (y2 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b) /
                     (y1 * (y1 * cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) * y1
                     -y2 * (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b) /
                     (y1 * (y1 * cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) ** 2
                     * (2 * y1 * cos(b) -y3 * sin(b))) / (1 + y2 ** 2 * (y1 **
                         2 + y2 ** 2 + y3 ** 2) * sin(b) ** 2 / (y1 * (y1 *
                                 cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) ** 2)
                    + y2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2 * cos(b)
                    / (1 + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) **
                        2) -(y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 *
                                            a) * sin(b)) + y2 ** 2 * cos(b)) *
                            y1 -y2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                            (1 / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 *
                                        a) * sin(b)) + y2 ** 2 * cos(b)) ** 2 *
                            (2 * y1 * cos(b) + (y3 + 2 * a) * sin(b))) / (1 +
                                y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                    2) * sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3
                                                + 2 * a) * sin(b)) + y2 ** 2 *
                                        cos(b)) ** 2)) + y2 * sin(b) * (1 / (y1
                                    ** 2 + y2 ** 2 + y3 ** 2) * cos(b) * y1 /
                                ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 *
                                 sin(b) -y3 * cos(b)) -((y1 ** 2 + y2 ** 2 + y3
                                     ** 2) ** (1 / 2) * cos(b) -y3) / (y1 ** 2
                                     + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 **
                                             2 + y2 ** 2 + y3 ** 2) ** (1 / 2)
                                         -y1 * sin(b) -y3 * cos(b)) * y1 -((y1
                                             ** 2 + y2 ** 2 + y3 ** 2) ** (1 /
                                                 2) * cos(b) -y3) / (y1 ** 2 +
                                                 y2 ** 2 + y3 ** 2) ** (1 / 2)
                                / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1
                                    * sin(b) -y3 * cos(b)) ** 2 * (1 / (y1 ** 2
                                        + y2 ** 2 + y3 ** 2) ** (1 / 2) * y1
                                    -sin(b)) -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2
                                        * a) ** 2) * cos(b) * y1 / ((y1 ** 2 +
                                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                            / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                        cos(b)) + ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                                * a) ** 2) ** (1 / 2) * cos(b)
                                        + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 +
                                            (y3 + 2 * a) ** 2) ** (3 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * y1 + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
            2) * cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** (1 / 2) * y1 -sin(b)))) / pi / (1 -nu) +
    1 / 4 * ((2 -2 * nu) * (y2 / y1 ** 2 / (1 + y2 ** 2 / y1 ** 2) -y2 / (y1 *
                    cos(b) + (y3 + 2 * a) * sin(b)) ** 2 * cos(b) / (1 + y2 **
                    2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2) + (y2 / (y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b)
                    / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                        cos(b)) * y1 -y2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                        2) ** (1 / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 *
                                    a) * sin(b)) + y2 ** 2 * cos(b)) ** 2 * (2
                                * y1 * cos(b) + (y3 + 2 * a) * sin(b))) / (1 +
                            y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) *
                            sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                                    sin(b)) + y2 ** 2 * cos(b)) ** 2)) -(2 -2 *
                        nu) * y2 * sin(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                            cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2)) * (1 / (y1 ** 2
                                    + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                                y1 -sin(b)) -(2 -2 * nu) * y2 * sin(b) / ((y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
            a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * y1 -y2 *
    (y3 + a) * sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * (1 + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                    ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
        (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + a *
        (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * y1 -y2 * (y3
        + a) * sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) ** 2 * (1 + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
            (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
        (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + a *
        (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * (1 / (y1 ** 2
            + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y1 -sin(b)) + y2 * (y3
        + a) * sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
            2) * cos(b) * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                    2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a /
                    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) -((y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b)
                    + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 *
        (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) * (1
            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y1 -sin(b))
        -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2
            * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                sin(b) + (y3 + 2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3
                    + 2 * a) ** 2) ** (3 / 2) * y1 -2 * a * (y3 + 2 * a) / (y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * y1)) / pi / (1
            -nu)))
    return e13


def e23(y1, y2, y3, a, b, nu, B1, B2, B3):
    e23 = (1 / 2 * B1 * (1 / 8 * ((1 -2 * nu) * ((1 / (y1 ** 2 + y2 ** 2 + y3
                                ** 2) ** (1 / 2) * y3 -1) / ((y1 ** 2 + y2 ** 2
                                    + y3 ** 2) ** (1 / 2) -y3) + (1 / 2 / (y1
                                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                    (1 / 2) * (2 * y3 + 4 * a) + 1) / ((y1 ** 2
                                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                        2) + y3 + 2 * a) -cos(b) * ((1 / (y1 **
                                                2 + y2 ** 2 + y3 ** 2) ** (1 /
                                                    2) * y3 -cos(b)) / ((y1 **
                                                        2 + y2 ** 2 + y3 ** 2)
                                                    ** (1 / 2) -y1 * sin(b) -y3
                                                    * cos(b)) + (1 / 2 / (y1 **
                                                        2 + y2 ** 2 + (y3 + 2 *
                                                            a) ** 2) ** (1 / 2)
                                                    * (2 * y3 + 4 * a) +
                                                    cos(b)) / ((y1 ** 2 + y2 **
                                                        2 + (y3 + 2 * a) ** 2)
                                                    ** (1 / 2) -y1 * sin(b) +
                                                    (y3 + 2 * a) * cos(b))))
                    -y2 ** 2 * ( -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2)
                        / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y3) * y3
                        -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1 **
                                2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y3) ** 2 *
                        (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * y3 -1)
                        -1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
                            / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                (1 / 2) + y3 + 2 * a) * (2 * y3 + 4 * a) -1 /
                        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
                        ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) +
                         y3 + 2 * a) ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 +
                                     2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a)
                             + 1) -cos(b) * ( -1 / (y1 ** 2 + y2 ** 2 + y3 **
                                     2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3
                                             ** 2) ** (1 / 2) -y1 * sin(b) -y3
                                         * cos(b)) * y3 -1 / (y1 ** 2 + y2 ** 2
                                         + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2
                                                 ** 2 + y3 ** 2) ** (1 / 2) -y1
                                             * sin(b) -y3 * cos(b)) ** 2 * (1 /
                                             (y1 ** 2 + y2 ** 2 + y3 ** 2) **
                                             (1 / 2) * y3 -cos(b)) -1 / 2 / (y1
                                             ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                             2) ** (3 / 2) / ((y1 ** 2 + y2 **
                                                     2 + (y3 + 2 * a) ** 2) **
                                                 (1 / 2) -y1 * sin(b) + (y3 + 2
                                                     * a) * cos(b)) * (2 * y3 +
                                                     4 * a) -1 / (y1 ** 2 + y2
                                                         ** 2 + (y3 + 2 * a) **
                                                         2) ** (1 / 2) / ((y1
                                                                 ** 2 + y2 ** 2
                                                                 + (y3 + 2 * a)
                                                                 ** 2) ** (1 /
                                                                     2) -y1 *
                                                             sin(b) + (y3 + 2 *
                                                                 a) * cos(b))
    ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 *
                y3 + 4 * a) + cos(b))))) / pi / (1 -nu) + 1 / 4 * ((1 -2 * nu)
    * (((2 -2 * nu) * 1/tan(b) ** 2 -nu) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2
                    * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + 1) / ((y1 ** 2 +
                    y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) -((2
                    -2 * nu) * 1/tan(b) ** 2 + 1 -2 * nu) * cos(b) * (1 / 2 / (y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 *
                        y3 + 4 * a) + cos(b)) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                        * cos(b))) + (1 -2 * nu) / ((y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) ** 2 * (y1 *
                        1/tan(b) * (1 -2 * nu -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                    a) ** 2) ** (1 / 2)) + nu * (y3 + 2 * a) -a
                        + y2 ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                            (1 / 2) + y3 + 2 * a) * (nu + a / (y1 ** 2 + y2 **
                                    2 + (y3 + 2 * a) ** 2) ** (1 / 2))) * (1 /
                            2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                2) * (2 * y3 + 4 * a) + 1) -(1 -2 * nu) / ((y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) + y3 + 2 * a) * (1 / 2 * a * y1 * 1/tan(b)
                                    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                    ** (3 / 2) * (2 * y3 + 4 * a) + nu -y2 ** 2
                                    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                        ** (1 / 2) + y3 + 2 * a) ** 2 * (nu + a
                                        / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                            2) ** (1 / 2)) * (1 / 2 / (y1 ** 2
                                                + y2 ** 2 + (y3 + 2 * a) ** 2)
                                            ** (1 / 2) * (2 * y3 + 4 * a) + 1)
                                    -1 / 2 * y2 ** 2 / ((y1 ** 2 + y2 ** 2 +
                                            (y3 + 2 * a) ** 2) ** (1 / 2) + y3
                                        + 2 * a) * a / (y1 ** 2 + y2 ** 2 + (y3
                                                + 2 * a) ** 2) ** (3 / 2) * (2
                                                * y3 + 4 * a)) -(1 -2 * nu) *
    sin(b) * 1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2
                + (y3 + 2 * a) ** 2) ** (1 / 2)) + (1 -2 * nu) * (y1 * cos(b) +
            (y3 + 2 * a) * sin(b)) * 1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                    a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) *
    (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 *
                                                                   a) + cos(b))
    + 1 / 2 * (1 -2 * nu) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)
    * (2 * y3 + 4 * a) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)
    * y1 * 1/tan(b) + 3 / 2 * a * y1 * (y3 + a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 +
            (y3 + 2 * a) ** 2) ** (5 / 2) * (2 * y3 + 4 * a) + 1 / ((y1 ** 2 +
                    y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * (
                -2 * nu + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                    2) * ((1 -2 * nu) * y1 * 1/tan(b) -a) + y2 ** 2 / (y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                        a) * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                2) ** (1 / 2)) + a * y2 ** 2 / (y1 ** 2 + y2 **
                                2 + (y3 + 2 * a) ** 2) ** (3 / 2)) -(y3 + a) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) ** 2 * (
    -2 * nu + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((1 -2 *
            nu) * y1 * 1/tan(b) -a) + y2 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
            ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                    2) + y3 + 2 * a) * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) ** (1 / 2)) + a * y2 ** 2 / (y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)) * (1 / 2 / (y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 *
                        y3 + 4 * a) + 1) + (y3 + a) / ((y1 ** 2 + y2 ** 2 + (y3
                                + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * ( -1
                            / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
                                / 2) * ((1 -2 * nu) * y1 * 1/tan(b) -a) * (2 * y3
                                    + 4 * a) -1 / 2 * y2 ** 2 / (y1 ** 2 + y2
                                        ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) /
                            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                             + y3 + 2 * a) * (2 * nu + a / (y1 ** 2 + y2 ** 2 +
                                     (y3 + 2 * a) ** 2) ** (1 / 2)) * (2 * y3 +
                                     4 * a) -y2 ** 2 / (y1 ** 2 + y2 ** 2 + (y3
                                             + 2 * a) ** 2) ** (1 / 2) / ((y1
                                                 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                                 ** 2) ** (1 / 2) + y3 + 2 * a)
                            ** 2 * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                        a) ** 2) ** (1 / 2)) * (1 / 2 / (y1 **
                                        2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                        / 2) * (2 * y3 + 4 * a) + 1) -1 / 2 *
                            y2 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                            ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                (1 / 2) + y3 + 2 * a) * a * (2 * y3 + 4 * a) -3
                            / 2 * a * y2 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                    a) ** 2) ** (5 / 2) * (2 * y3 + 4 * a)) + 1
    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * (cos(b) ** 2 -1 / (y1 ** 2 + y2 ** 2 + (y3 +
                        2 * a) ** 2) ** (1 / 2) * ((1 -2 * nu) * (y1 * cos(b) +
                            (y3 + 2 * a) * sin(b)) * 1/tan(b) + a * cos(b)) + a *
                (y3 + 2 * a) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) -1 / (y1 **
                    2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
                    (y3 + 2 * a) * cos(b)) * (y2 ** 2 * cos(b) ** 2 -a * (y1 *
                        cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) / (y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) +
                        y3 + 2 * a))) -(y3 + a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                            * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b)) ** 2 * (cos(b) ** 2 -1 / (y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) ** (1 / 2) * ((1 -2 * nu) * (y1 *
                                cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) + a *
                            cos(b)) + a * (y3 + 2 * a) * (y1 * cos(b) + (y3 + 2
                                * a) * sin(b)) * 1/tan(b) / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (3 / 2) -1 / (y1 ** 2 +
                                    y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
                    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                     sin(b) + (y3 + 2 * a) * cos(b)) * (y2 ** 2 * cos(b) ** 2
                     -a * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) / (y1
                         ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((y1
                             ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                         cos(b) + y3 + 2 * a))) * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                     (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b))
    + (y3 + a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3
                    + 2 * a) ** 2) ** (3 / 2) * ((1 -2 * nu) * (y1 * cos(b) +
                        (y3 + 2 * a) * sin(b)) * 1/tan(b) + a * cos(b)) * (2 * y3
                    + 4 * a) -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                        / 2) * (1 -2 * nu) * sin(b) * 1/tan(b) + a * (y1 * cos(b)
                            + (y3 + 2 * a) * sin(b)) * 1/tan(b) / (y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) + a * (y3 + 2
                            * a) * sin(b) * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 +
                                    2 * a) ** 2) ** (3 / 2) -3 / 2 * a * (y3 +
                                    2 * a) * (y1 * cos(b) + (y3 + 2 * a) *
                                        sin(b)) * 1/tan(b) / (y1 ** 2 + y2 ** 2 +
                                        (y3 + 2 * a) ** 2) ** (5 / 2) * (2 * y3
                                            + 4 * a) + 1 / 2 / (y1 ** 2 + y2 **
                                                2 + (y3 + 2 * a) ** 2) ** (3 /
                                                2) / ((y1 ** 2 + y2 ** 2 + (y3
                                                            + 2 * a) ** 2) **
                                                    (1 / 2) -y1 * sin(b) + (y3
                                                        + 2 * a) * cos(b)) *
            (y2 ** 2 * cos(b) ** 2 -a * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) *
             1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((y1
                     ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) +
                 y3 + 2 * a)) * (2 * y3 + 4 * a) + 1 / (y1 ** 2 + y2 ** 2 + (y3
                     + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                             * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                     * cos(b)) ** 2 * (y2 ** 2 * cos(b) ** 2 -a * (y1 * cos(b)
                         + (y3 + 2 * a) * sin(b)) * 1/tan(b) / (y1 ** 2 + y2 ** 2
                         + (y3 + 2 * a) ** 2) ** (1 / 2) * ((y1 ** 2 + y2 ** 2
                             + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2
                         * a)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                             2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) -1 /
    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * ( -a * sin(b) * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                    a) ** 2) ** (1 / 2) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                        2) ** (1 / 2) * cos(b) + y3 + 2 * a) + 1 / 2 * a * (y1
                        * cos(b) + (y3 + 2 * a) * sin(b)) * 1/tan(b) / (y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) +
                        y3 + 2 * a) * (2 * y3 + 4 * a) -a * (y1 * cos(b) + (y3
                                + 2 * a) * sin(b)) * 1/tan(b) / (y1 ** 2 + y2 **
                                2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (1 / 2 /
                                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                    2) * cos(b) * (2 * y3 + 4 * a) + 1)))) / pi
    / (1 -nu)) + 1 / 2 * B2 * (1 / 8 * ((2 -2 * nu) * (y2 / (y1 * cos(b) -y3 *
                    sin(b)) ** 2 * sin(b) / (1 + y2 ** 2 / (y1 * cos(b) -y3 *
                        sin(b)) ** 2) + (y2 / (y1 ** 2 + y2 ** 2 + y3 ** 2) **
                    (1 / 2) * sin(b) / (y1 * (y1 * cos(b) -y3 * sin(b)) + y2 **
                        2 * cos(b)) * y3 + y2 * (y1 ** 2 + y2 ** 2 + y3 ** 2)
                    ** (1 / 2) * sin(b) ** 2 / (y1 * (y1 * cos(b) -y3 * sin(b))
                        + y2 ** 2 * cos(b)) ** 2 * y1) / (1 + y2 ** 2 * (y1 **
                        2 + y2 ** 2 + y3 ** 2) * sin(b) ** 2 / (y1 * (y1 *
                                cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) ** 2)
                -y2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2 * sin(b) / (1
                    + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2) +
                (1 / 2 * y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                                                          2) *
                 sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2
                     * cos(b)) * (2 * y3 + 4 * a) -y2 * (y1 ** 2 + y2 ** 2 +
                     (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) ** 2 / (y1 * (y1 *
                             cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                         cos(b)) ** 2 * y1) / (1 + y2 ** 2 * (y1 ** 2 + y2 ** 2
                         + (y3 + 2 * a) ** 2) * sin(b) ** 2 / (y1 * (y1 *
                             cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                         cos(b)) ** 2)) + y1 * y2 * ( -1 / (y1 ** 2 + y2 ** 2 +
                     y3 ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1
                             / 2) -y3) * y3 -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2)
                 ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y3) **
                 2 * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * y3 -1) -1
                 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) /
                 ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                  a) * (2 * y3 + 4 * a) -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                      ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                              2) ** (1 / 2) + y3 + 2 * a) ** 2 * (1 / 2 / (y1
                                  ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                  2) * (2 * y3 + 4 * a) + 1)) -y2 * ( -sin(b) /
                              (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1
                                      ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1
                                  * sin(b) -y3 * cos(b)) -(y1 * cos(b) -y3 *
                                  sin(b)) / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3
                                  / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 /
                                          2) -y1 * sin(b) -y3 * cos(b)) * y3
                              -(y1 * cos(b) -y3 * sin(b)) / (y1 ** 2 + y2 ** 2
                                  + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 +
                                          y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3
                                      * cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2
                                          + y3 ** 2) ** (1 / 2) * y3 -cos(b)) +
                              sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                              ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                      ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                                          a) * cos(b)) -1 / 2 * (y1 * cos(b) +
                                          (y3 + 2 * a) * sin(b)) / (y1 ** 2 +
                                          y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 /
                                          2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                                      a) ** 2) ** (1 / 2) -y1 *
                                              sin(b) + (y3 + 2 * a) * cos(b)) *
    (2 * y3 + 4 * a) -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2 + y2 **
            2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                    a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 *
                y3 + 4 * a) + cos(b)))) / pi / (1 -nu) + 1 / 4 * ((2 -2 * nu) *
        (1 -2 * nu) * ( -y2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2 *
            sin(b) / (1 + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) ** 2)
            + (1 / 2 * y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2 **
                    2 * cos(b)) * (2 * y3 + 4 * a) -y2 * (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) ** 2 / (y1 * (y1 *
                            cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 * cos(b))
                ** 2 * y1) / (1 + y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) * sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3 + 2 * a)
                                * sin(b)) + y2 ** 2 * cos(b)) ** 2)) * 1/tan(b)
        ** 2 -(1 -2 * nu) * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                / 2) + y3 + 2 * a) ** 2 * (( -1 + 2 * nu + a / (y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2) ** (1 / 2)) * 1/tan(b) + y1 / ((y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2
                    * a) * (nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2))) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                            ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + 1) + (1 -2 *
                            nu) * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                ** (1 / 2) + y3 + 2 * a) * ( -1 / 2 * a / (y1
                                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 /
                                    2) * (2 * y3 + 4 * a) * 1/tan(b) -y1 / ((y1
                                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                        ** (1 / 2) + y3 + 2 * a) ** 2 * (nu + a
                                        / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                            2) ** (1 / 2)) * (1 / 2 / (y1 ** 2
                                                + y2 ** 2 + (y3 + 2 * a) ** 2)
                                            ** (1 / 2) * (2 * y3 + 4 * a) + 1)
                                -1 / 2 * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                            a) ** 2) ** (1 / 2) + y3 + 2 * a) *
                                a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                (3 / 2) * (2 * y3 + 4 * a)) + (1 -2 * nu) * y2
    * 1/tan(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
            + (y3 + 2 * a) * cos(b)) ** 2 * (1 + a / (y1 ** 2 + y2 ** 2 + (y3 +
                    2 * a) ** 2) ** (1 / 2) / cos(b)) * (1 / 2 / (y1 ** 2 + y2
                    ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) +
                cos(b)) + 1 / 2 * (1 -2 * nu) * y2 * 1/tan(b) / ((y1 ** 2 + y2 **
                    2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                    a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** (3 / 2) / cos(b) * (2 * y3 + 4 * a) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                a) ** 2) ** (3 / 2) * y2 * 1/tan(b) + 3 / 2 * a * y2 * (y3 + a) *
    1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (5 / 2) * (2 * y3 + 4 *
            a) + y2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                a) * ((1 -2 * nu) * 1/tan(b) -2 * nu * y1 / ((y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) -a * y1
                    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (1 /
                        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + 1
                        / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) +
                            y3 + 2 * a))) -1 / 2 * y2 * (y3 + a) / (y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                        a) * ((1 -2 * nu) * 1/tan(b) -2 * nu * y1 / ((y1 ** 2 +
                                    y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) +
                                y3 + 2 * a) -a * y1 / (y1 ** 2 + y2 ** 2 + (y3
                                        + 2 * a) ** 2) ** (1 / 2) * (1 / (y1 **
                                            2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                        (1 / 2) + 1 / ((y1 ** 2 + y2 ** 2 + (y3
                                                    + 2 * a) ** 2) ** (1 / 2) +
                                            y3 + 2 * a))) * (2 * y3 + 4 * a)
    -y2 * (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1
                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) **
    2 * ((1 -2 * nu) * 1/tan(b) -2 * nu * y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                    ** 2) ** (1 / 2) + y3 + 2 * a) -a * y1 / (y1 ** 2 + y2 ** 2
                    + (y3 + 2 * a) ** 2) ** (1 / 2) * (1 / (y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) + 1 / ((y1 ** 2 + y2 ** 2
                                + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a)))
    * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4
                * a) + 1) + y2 * (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                    (1 / 2) + y3 + 2 * a) * (2 * nu * y1 / ((y1 ** 2 + y2 ** 2
                                + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a)
                        ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                            ** (1 / 2) * (2 * y3 + 4 * a) + 1) + 1 / 2 * a * y1
                        / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) *
                        (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                         + 1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                 2) + y3 + 2 * a)) * (2 * y3 + 4 * a) -a * y1 /
                        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (
                            -1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                            (3 / 2) * (2 * y3 + 4 * a) -1 / ((y1 ** 2 + y2 ** 2
                                    + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                                a) ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2
                                            * a) ** 2) ** (1 / 2) * (2 * y3 + 4
                                            * a) + 1))) + y2 * 1/tan(b) / (y1 **
                                    2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
    / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * (( -2 + 2 * nu) * cos(b) + ((y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 *
                    a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                        -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (1 + a / (y1 **
                            2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
                        cos(b)) + a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) / cos(b)) -1 / 2 * y2 * (y3 + a) *
    1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                * a) * cos(b)) * (( -2 + 2 * nu) * cos(b) + ((y1 ** 2 + y2 ** 2
                        + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a)
                / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                    sin(b) + (y3 + 2 * a) * cos(b)) * (1 + a / (y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2) ** (1 / 2) / cos(b)) + a * (y3 +
                    2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) / cos(b))
    * (2 * y3 + 4 * a) -y2 * (y3 + a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (( -2 +
                    2 * nu) * cos(b) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                        ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2
                            + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3
                            + 2 * a) * cos(b)) * (1 + a / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2) / cos(b)) + a *
                (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) /
                cos(b)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                (1 / 2) * (2 * y3 + 4 * a) + cos(b)) + y2 * (y3 + a) * 1/tan(b) /
    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * ((1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                    2) * cos(b) * (2 * y3 + 4 * a) + 1) / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (1 + a / (y1 ** 2 + y2 ** 2 + (y3 +
                                    2 * a) ** 2) ** (1 / 2) / cos(b)) -((y1 **
                                    2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                                * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 +
                                    (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
                                + (y3 + 2 * a) * cos(b)) ** 2 * (1 + a / (y1 **
                                    2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                                / cos(b)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 +
                                        2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 *
                                        a) + cos(b)) -1 / 2 * ((y1 ** 2 + y2 **
                                            2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                                        cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2
                                            ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                            2) -y1 * sin(b) + (y3 + 2 * a) *
                                        cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3
                                            + 2 * a) ** 2) ** (3 / 2) / cos(b)
            * (2 * y3 + 4 * a) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) /
            cos(b) -a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** 2 / cos(b) * (2 * y3 + 4 * a))) / pi / (1 -nu)) + 1 / 2 * B3 * (1 / 8 *
((1 -2 * nu) * sin(b) * ((1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * y3
        -cos(b)) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 *
        cos(b)) + (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
        (2 * y3 + 4 * a) + cos(b)) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
        ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))) -y2 ** 2 * sin(b) * (
    -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3 **
            2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) * y3 -1 / (y1 ** 2 + y2 **
            2 + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2)
                -y1 * sin(b) -y3 * cos(b)) ** 2 * (1 / (y1 ** 2 + y2 ** 2 + y3
                    ** 2) ** (1 / 2) * y3 -cos(b)) -1 / 2 / (y1 ** 2 + y2 ** 2
                    + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + (y3
                            + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (2 * y3 + 4 * a) -1 / (y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                            sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (1 / 2 / (y1
                                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                            * (2 * y3 + 4 * a) + cos(b)))) / pi / (1 -nu) + 1 /
4 * ((1 -2 * nu) * ( -sin(b) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
            ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) / ((y1 ** 2 + y2 ** 2 + (y3
                    + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) + y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
            + y3 + 2 * a) ** 2 * (1 + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                    2) ** (1 / 2)) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + 1) + 1 / 2 * y1 /
        ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * a /
        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (2 * y3 + 4 * a) +
        sin(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2
                + (y3 + 2 * a) ** 2) ** (1 / 2)) -(y1 * cos(b) + (y3 + 2 * a) *
            sin(b)) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2
                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) * (1 / 2 / (y1 ** 2 + y2
                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) +
            cos(b)) -1 / 2 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / ((y1 ** 2
                + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
        ** (3 / 2) * (2 * y3 + 4 * a)) + y1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
        ** 2) ** (1 / 2) * (a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) + 1 /
            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a))
    -1 / 2 * y1 * (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2)
    * (a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) + 1 / ((y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a)) * (2 * y3 + 4
                * a) + y1 * (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** (1 / 2) * ( -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * (2 * y3
                + 4 * a) -1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                        2) + y3 + 2 * a) ** 2 * (1 / 2 / (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) +
                        1)) -1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                            / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
    (sin(b) * (cos(b) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2))
     + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
             a) ** 2) ** (1 / 2) * (1 + a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 +
                 (y3 + 2 * a) ** 2)) -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                         / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (y2 ** 2
                         * cos(b) * sin(b) -a * (y1 * cos(b) + (y3 + 2 * a) *
                             sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                         ** (1 / 2) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                             ** (1 / 2) * cos(b) + y3 + 2 * a))) + (y3 + a) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) ** 2 * (sin(b) * (cos(b) -a / (y1 ** 2 + y2 ** 2 + (y3 +
                    2 * a) ** 2) ** (1 / 2)) + (y1 * cos(b) + (y3 + 2 * a) *
                sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
        (1 + a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -1 /
        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 **
                2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * (y2 ** 2 * cos(b) * sin(b) -a * (y1 * cos(b) + (y3 + 2 *
                    a) * sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
            (1 / 2) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                cos(b) + y3 + 2 * a))) * (1 / 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2
                * a) ** 2) ** (1 / 2) * (2 * y3 + 4 * a) + cos(b)) -(y3 + a) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * (1 / 2 * sin(b) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                a) ** 2) ** (3 / 2) * (2 * y3 + 4 * a) + sin(b) / (y1 ** 2 + y2
                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (1 + a * (y3 + 2 * a) /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -1 / 2 * (y1 * cos(b)
                    + (y3 + 2 * a) * sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                        a) ** 2) ** (3 / 2) * (1 + a * (y3 + 2 * a) / (y1 ** 2
                            + y2 ** 2 + (y3 + 2 * a) ** 2)) * (2 * y3 + 4 * a)
        + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2
                * a) ** 2) ** (1 / 2) * (a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                    ** 2) -a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** 2 * (2 * y3 + 4 * a)) + 1 / 2 / (y1 ** 2 + y2
                        ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
                        (y3 + 2 * a) * cos(b)) * (y2 ** 2 * cos(b) * sin(b) -a
                        * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2 + y2
                            ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                            cos(b) + y3 + 2 * a)) * (2 * y3 + 4 * a) + 1 / (y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1
                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1
                        * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (y2 ** 2 *
                        cos(b) * sin(b) -a * (y1 * cos(b) + (y3 + 2 * a) *
                            sin(b)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                        ** (1 / 2) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                            ** (1 / 2) * cos(b) + y3 + 2 * a)) * (1 / 2 / (y1
                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2
                            * y3 + 4 * a) + cos(b)) -1 / (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2
                                    + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                                sin(b) + (y3 + 2 * a) * cos(b)) * ( -a * sin(b)
                                / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                                    / 2) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                            ** 2) ** (1 / 2) * cos(b) + y3 + 2
                                        * a) + 1 / 2 * a * (y1 * cos(b) + (y3 +
                                                2 * a) * sin(b)) / (y1 ** 2 +
                                                y2 ** 2 + (y3 + 2 * a) ** 2) **
                                (3 / 2) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                        2) ** (1 / 2) * cos(b) + y3 + 2 * a) *
                                (2 * y3 + 4 * a) -a * (y1 * cos(b) + (y3 + 2 *
                                        a) * sin(b)) / (y1 ** 2 + y2 ** 2 + (y3
                                            + 2 * a) ** 2) ** (1 / 2) * (1 / 2
                                            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                                ** 2) ** (1 / 2) * cos(b) * (2
                                                    * y3 + 4 * a) + 1)))) / pi
    / (1 -nu)) + 1 / 2 * B1 * (1 / 8 * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1
                / 2) -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
            -cos(b) * (((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * cos(b) -y3)
                / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2 **
                        2 + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b))
                -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) +
                    y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                        / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)))) / pi
        / (1 -nu) + 1 / 8 * y2 * ( -1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 /
                2) * y2 + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 /
                    2) * y2 -cos(b) * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) *
                        cos(b) * y2 / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2)
                            -y1 * sin(b) -y3 * cos(b)) -((y1 ** 2 + y2 ** 2 +
                                y3 ** 2) ** (1 / 2) * cos(b) -y3) / (y1 ** 2 +
                                y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 ** 2 + y2
                                        ** 2 + y3 ** 2) ** (1 / 2) -y1 * sin(b)
                                    -y3 * cos(b)) * y2 -((y1 ** 2 + y2 ** 2 +
                                        y3 ** 2) ** (1 / 2) * cos(b) -y3) / (y1
                                        ** 2 + y2 ** 2 + y3 ** 2) / ((y1 ** 2 +
                                                y2 ** 2 + y3 ** 2) ** (1 / 2)
                                            -y1 * sin(b) -y3 * cos(b)) ** 2 *
                        y2 -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) *
                        cos(b) * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                            ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) +
                        ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                         cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                             a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 +
                                     2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3
                                     + 2 * a) * cos(b)) * y2 + ((y1 ** 2 + y2
                                         ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                                     cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2
                                     + (y3 + 2 * a) ** 2) / ((y1 ** 2 + y2 ** 2
                                         + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                                     sin(b) + (y3 + 2 * a) * cos(b)) ** 2 *
    y2)) / pi / (1 -nu) + 1 / 4 * ((2 -2 * nu) * ((1 -2 * nu) * ( -1 / y1 / (1
                + y2 ** 2 / y1 ** 2) + 1 / (y1 * cos(b) + (y3 + 2 * a) *
                    sin(b)) / (1 + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) *
                        sin(b)) ** 2) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                        2) ** (1 / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 *
                                    a) * sin(b)) + y2 ** 2 * cos(b)) + y2 ** 2
                    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                    sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2
                        ** 2 * cos(b)) -2 * y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3
                            + 2 * a) ** 2) ** (1 / 2) * sin(b) / (y1 * (y1 *
                                cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                            cos(b)) ** 2 * cos(b)) / (1 + y2 ** 2 * (y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) * sin(b) ** 2 / (y1 *
                            (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                            cos(b)) ** 2)) * 1/tan(b) + 1 / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) * (2 * nu +
                        a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2))
        -y2 ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2
            * a) ** 2 * (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                ** (1 / 2)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                2) -y2 ** 2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                        2) + y3 + 2 * a) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                            a) ** 2) ** (3 / 2) -cos(b) / ((y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
                            (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + y2
    ** 2 * cos(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
            sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2
                ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) / (y1 ** 2 + y2 ** 2 +
            (y3 + 2 * a) ** 2) ** (1 / 2) + y2 ** 2 * cos(b) / ((y1 ** 2 + y2
                    ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                    * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                    2) ** (3 / 2)) + (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                        a) ** 2) ** (1 / 2) * (2 * nu / ((y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) + a
                        / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -y2 ** 2 *
    (y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * (2 * nu /
            ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) +
            a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) + y2 * (y3 + a) / (y1
            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ( -2 * nu / ((y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                a) ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
            y2 -2 * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * y2) +
    (y3 + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * (1 -2 * nu -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
            ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                    a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
        (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) -a *
        (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -y2 ** 2 * (y3
        + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * (1 -2 * nu -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
            ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                    a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) *
        (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) -a *
        (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -y2 ** 2 * (y3
        + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                * a) * cos(b)) ** 2 * (1 -2 * nu -((y1 ** 2 + y2 ** 2 + (y3 + 2
                            * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1
                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1
                        * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1
                            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) -a
                * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) + y2
    * (y3 + a) * cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) * ( -1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
            2) * cos(b) * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                    2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a /
                    (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) + ((y1
                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b)
                    + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) ** 2 *
        (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) / (y1
            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y2 + ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a)
        / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
            (y3 + 2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
            2) ** (3 / 2) * y2 + 2 * a * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** 2 * y2)) / pi / (1 -nu)) + 1 / 2 * B2 *
    (1 / 8 * (( -1 + 2 * nu) * sin(b) * (1 / (y1 ** 2 + y2 ** 2 + y3 ** 2) **
                                         (1 / 2) * y2 / ((y1 ** 2 + y2 ** 2 +
                                                 y3 ** 2) ** (1 / 2) -y1 *
                                             sin(b) -y3 * cos(b)) -1 / (y1 ** 2
                                             + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                         (1 / 2) * y2 / ((y1 ** 2 + y2 ** 2 +
                                                 (y3 + 2 * a) ** 2) ** (1 / 2)
                                             -y1 * sin(b) + (y3 + 2 * a) *
                                             cos(b))) -y1 * ( -1 / (y1 ** 2 +
                                             y2 ** 2 + y3 ** 2) ** (3 / 2) * y2
                                         + 1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                                 a) ** 2) ** (3 / 2) * y2) +
              (y1 * cos(b) -y3 * sin(b)) / (y1 ** 2 + y2 ** 2 + y3 ** 2) *
              cos(b) * y2 / ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 *
                  sin(b) -y3 * cos(b)) -(y1 * cos(b) -y3 * sin(b)) * ((y1 ** 2
                      + y2 ** 2 + y3 ** 2) ** (1 / 2) * cos(b) -y3) / (y1 ** 2
                      + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 +
                              y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) *
              y2 -(y1 * cos(b) -y3 * sin(b)) * ((y1 ** 2 + y2 ** 2 + y3 ** 2)
                  ** (1 / 2) * cos(b) -y3) / (y1 ** 2 + y2 ** 2 + y3 ** 2) /
              ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 *
               cos(b)) ** 2 * y2 -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1
               ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) * cos(b) * y2 / ((y1 ** 2 +
                   y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                   2 * a) * cos(b)) + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) *
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a)
    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2
                + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) * y2 + (y1 * cos(b) + (y3 + 2 * a) * sin(b)) * ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a)
    / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                    a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    ** 2 * y2) / pi / (1 -nu) + 1 / 4 * (( -2 + 2 * nu) * (1 -2 * nu) * 1/tan(b)
        * (1 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y2 / ((y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 *
                a) -cos(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
            * y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                sin(b) + (y3 + 2 * a) * cos(b))) + (2 -2 * nu) * y1 / ((y1 ** 2
                + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) ** 2 *
        (2 * nu + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)) / (y1
            ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y2 + (2 -2 * nu) *
        y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a)
        * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) * y2 -(2 -2 *
            nu) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / ((y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                    ** 2) ** (1 / 2)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
        ** (1 / 2) * y2 -(2 -2 * nu) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) /
        ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3
            + 2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
        (3 / 2) * y2 -(y3 + a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3
            / 2) * ((1 -2 * nu) * 1/tan(b) -2 * nu * y1 / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) + y3 + 2 * a) -a * y1 /
                (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) * y2 + (y3 + a) / (y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * nu *
                    y1 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) +
                        y3 + 2 * a) ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                            2) ** (1 / 2) * y2 + 2 * a * y1 / (y1 ** 2 + y2 **
                                2 + (y3 + 2 * a) ** 2) ** 2 * y2) + (y3 + a) /
    ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
        * a) * cos(b)) ** 2 * (cos(b) * sin(b) + ((y1 ** 2 + y2 ** 2 + (y3 + 2
                    * a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) * 1/tan(b) / (y1
                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * ((2 -2 * nu) *
                cos(b) -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                    cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)))
        + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * (sin(b) -(y3
                + 2 * a) * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 ** 2 +
                    y2 ** 2 + (y3 + 2 * a) ** 2) -(y1 * cos(b) + (y3 + 2 * a) *
                    sin(b)) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                        2) * cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3
                                    + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
                            (y3 + 2 * a) * cos(b)))) / (y1 ** 2 + y2 ** 2 + (y3
                        + 2 * a) ** 2) ** (1 / 2) * y2 -(y3 + a) / ((y1 ** 2 +
                            y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                        sin(b) + (y3 + 2 * a) * cos(b)) * (1 / (y1 ** 2 + y2 **
                            2 + (y3 + 2 * a) ** 2) * cos(b) * y2 * 1/tan(b) * ((2
                                -2 * nu) * cos(b) -((y1 ** 2 + y2 ** 2 + (y3 +
                                            2 * a) ** 2) ** (1 / 2) * cos(b) +
                                    y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                                * a) ** 2) ** (1 / 2) -y1 *
                                        sin(b) + (y3 + 2 * a) * cos(b))) -((y1
                                        ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                                    (1 / 2) * cos(b) + y3 + 2 * a) * 1/tan(b) /
                        (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) *
                        ((2 -2 * nu) * cos(b) -((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                    a) ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a)
                         / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                             -y1 * sin(b) + (y3 + 2 * a) * cos(b))) * y2 + ((y1
                             ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                         cos(b) + y3 + 2 * a) * 1/tan(b) / (y1 ** 2 + y2 ** 2 +
                         (y3 + 2 * a) ** 2) ** (1 / 2) * ( -cos(b) / (y1 ** 2 +
                                 y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * y2 /
                             ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) +
                             ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 **
                                     2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                                 sin(b) + (y3 + 2 * a) * cos(b)) ** 2 / (y1 **
                                 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                             y2) -a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** (3 / 2) * (sin(b) -(y3 + 2 * a) * (y1 * cos(b) + (y3 + 2 * a) * sin(b))
            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) -(y1 * cos(b) + (y3 + 2 *
                    a) * sin(b)) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                    (1 / 2) * cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 +
                        2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                                * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                                a) * cos(b))) * y2 + a / (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2) * (2 * (y3 + 2 * a) *
                                (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 **
                                    2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** 2 * y2
                                -(y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (y1 **
                                    2 + y2 ** 2 + (y3 + 2 * a) ** 2) * cos(b) *
                                y2 / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                                    ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                                    cos(b)) + (y1 * cos(b) + (y3 + 2 * a) *
                                    sin(b)) * ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                            a) ** 2) ** (1 / 2) * cos(b) + y3 +
                                    2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                                        ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2
                                                + (y3 + 2 * a) ** 2) ** (1 / 2)
                                            -y1 * sin(b) + (y3 + 2 * a) *
                                            cos(b)) * y2 + (y1 * cos(b) + (y3 +
                                                2 * a) * sin(b)) * ((y1 ** 2 +
                                                    y2 ** 2 + (y3 + 2 * a) **
                                                    2) ** (1 / 2) * cos(b) + y3
                                                + 2 * a) / (y1 ** 2 + y2 ** 2 +
                                                    (y3 + 2 * a) ** 2) / ((y1
                                                            ** 2 + y2 ** 2 +
                                                            (y3 + 2 * a) ** 2)
                                                        ** (1 / 2) -y1 * sin(b)
                                                        + (y3 + 2 * a) *
                                                        cos(b)) ** 2 * y2))) /
    pi / (1 -nu)) + 1 / 2 * B3 * (1 / 8 * ((2 -2 * nu) * (1 / (y1 * cos(b) -y3
                    * sin(b)) / (1 + y2 ** 2 / (y1 * cos(b) -y3 * sin(b)) ** 2)
                + ((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b) / (y1 *
                        (y1 * cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) + y2 **
                    2 / (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b) / (y1
                        * (y1 * cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) -2 *
                    y2 ** 2 * (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * sin(b)
                    / (y1 * (y1 * cos(b) -y3 * sin(b)) + y2 ** 2 * cos(b)) ** 2
                    * cos(b)) / (1 + y2 ** 2 * (y1 ** 2 + y2 ** 2 + y3 ** 2) *
                    sin(b) ** 2 / (y1 * (y1 * cos(b) -y3 * sin(b)) + y2 ** 2 *
                        cos(b)) ** 2) -1 / (y1 * cos(b) + (y3 + 2 * a) *
                    sin(b)) / (1 + y2 ** 2 / (y1 * cos(b) + (y3 + 2 * a) *
                        sin(b)) ** 2) -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                    ** (1 / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                            sin(b)) + y2 ** 2 * cos(b)) + y2 ** 2 / (y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) / (y1
                        * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                        cos(b)) -2 * y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                            a) ** 2) ** (1 / 2) * sin(b) / (y1 * (y1 * cos(b) +
                                (y3 + 2 * a) * sin(b)) + y2 ** 2 * cos(b)) ** 2
                    * cos(b)) / (1 + y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 *
                            a) ** 2) * sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3 +
                                    2 * a) * sin(b)) + y2 ** 2 * cos(b)) ** 2))
    + sin(b) * (((y1 ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * cos(b) -y3) / (y1
                ** 2 + y2 ** 2 + y3 ** 2) ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + y3
                        ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) -((y1 ** 2
                            + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * cos(b)
                        + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
            ** (1 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2)
                -y1 * sin(b) + (y3 + 2 * a) * cos(b))) + y2 * sin(b) * (1 / (y1
                ** 2 + y2 ** 2 + y3 ** 2) * cos(b) * y2 / ((y1 ** 2 + y2 ** 2 +
                        y3 ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) -((y1 **
                            2 + y2 ** 2 + y3 ** 2) ** (1 / 2) * cos(b) -y3) /
            (y1 ** 2 + y2 ** 2 + y3 ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + y3
                    ** 2) ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) * y2 -((y1 ** 2
                        + y2 ** 2 + y3 ** 2) ** (1 / 2) * cos(b) -y3) / (y1 **
                        2 + y2 ** 2 + y3 ** 2) / ((y1 ** 2 + y2 ** 2 + y3 ** 2)
                            ** (1 / 2) -y1 * sin(b) -y3 * cos(b)) ** 2 * y2 -1
            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) * cos(b) * y2 / ((y1 ** 2
                    + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) +
                (y3 + 2 * a) * cos(b)) + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                    2) ** (1 / 2) * cos(b) + y3 + 2 * a) / (y1 ** 2 + y2 ** 2 +
                    (y3 + 2 * a) ** 2) ** (3 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 +
                                2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                                * a) * cos(b)) * y2 + ((y1 ** 2 + y2 ** 2 + (y3
                                        + 2 * a) ** 2) ** (1 / 2) * cos(b) + y3
                                + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                    2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) **
                                            2) ** (1 / 2) -y1 * sin(b) + (y3 +
                                                2 * a) * cos(b)) ** 2 * y2)) /
    pi / (1 -nu) + 1 / 4 * ((2 -2 * nu) * ( -1 / y1 / (1 + y2 ** 2 / y1 ** 2) +
                1 / (y1 * cos(b) + (y3 + 2 * a) * sin(b)) / (1 + y2 ** 2 / (y1
                        * cos(b) + (y3 + 2 * a) * sin(b)) ** 2) + ((y1 ** 2 +
                        y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) * sin(b) / (y1
                        * (y1 * cos(b) + (y3 + 2 * a) * sin(b)) + y2 ** 2 *
                        cos(b)) + y2 ** 2 / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2) * sin(b) / (y1 * (y1 * cos(b) + (y3 +
                                    2 * a) * sin(b)) + y2 ** 2 * cos(b)) -2 *
                    y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                        2) * sin(b) / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                                sin(b)) + y2 ** 2 * cos(b)) ** 2 * cos(b)) / (1
                        + y2 ** 2 * (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) *
                        sin(b) ** 2 / (y1 * (y1 * cos(b) + (y3 + 2 * a) *
                                sin(b)) + y2 ** 2 * cos(b)) ** 2)) + (2 -2 *
                    nu) * sin(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                        (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b)
                        + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                            2)) -(2 -2 * nu) * y2 ** 2 * sin(b) / ((y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                            sin(b) + (y3 + 2 * a) * cos(b)) ** 2 * (cos(b) + a
                            / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 /
                                2)) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
    ** (1 / 2) -(2 -2 * nu) * y2 ** 2 * sin(b) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                    * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) * cos(b))
    * a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (3 / 2) + (y3 + a) *
    sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2 +
                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2
                * a) * cos(b)) * (1 + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)
                    ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 +
                        (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 *
                            a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2 +
                                (y3 + 2 * a) ** 2) ** (1 / 2)) + a * (y3 + 2 *
                                a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2))
    -y2 ** 2 * (y3 + a) * sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
    (3 / 2) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
            + (y3 + 2 * a) * cos(b)) * (1 + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                    ** 2) ** (1 / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 +
                        2 * a) * cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2 +
                            (y3 + 2 * a) ** 2) ** (1 / 2)) + a * (y3 + 2 * a) /
            (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2)) -y2 ** 2 * (y3 + a) *
    sin(b) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) / ((y1 ** 2 + y2 ** 2 +
                (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
            cos(b)) ** 2 * (1 + ((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1
                    / 2) * cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2
                            * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a) *
                    cos(b)) * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a)
                        ** 2) ** (1 / 2)) + a * (y3 + 2 * a) / (y1 ** 2 + y2 **
                        2 + (y3 + 2 * a) ** 2)) + y2 * (y3 + a) * sin(b) / (y1
                    ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) / ((y1 ** 2
                        + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 * sin(b)
                    + (y3 + 2 * a) * cos(b)) * (1 / (y1 ** 2 + y2 ** 2 + (y3 +
                            2 * a) ** 2) ** (1 / 2) * cos(b) * y2 / ((y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) -y1 *
                            sin(b) + (y3 + 2 * a) * cos(b)) * (cos(b) + a / (y1
                                ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2))
                    -((y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                        cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 + 2 *
                                a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3 + 2 * a)
                        * cos(b)) ** 2 * (cos(b) + a / (y1 ** 2 + y2 ** 2 + (y3
                                + 2 * a) ** 2) ** (1 / 2)) / (y1 ** 2 + y2 ** 2
                            + (y3 + 2 * a) ** 2) ** (1 / 2) * y2 -((y1 ** 2 +
                                y2 ** 2 + (y3 + 2 * a) ** 2) ** (1 / 2) *
                            cos(b) + y3 + 2 * a) / ((y1 ** 2 + y2 ** 2 + (y3 +
                                    2 * a) ** 2) ** (1 / 2) -y1 * sin(b) + (y3
                                    + 2 * a) * cos(b)) * a / (y1 ** 2 + y2 ** 2
                                    + (y3 + 2 * a) ** 2) ** (3 / 2) * y2 -2 * a
                    * (y3 + 2 * a) / (y1 ** 2 + y2 ** 2 + (y3 + 2 * a) ** 2) **
                    2 * y2)) / pi / (1 -nu)))
    return e23


